@@ -31,6 +31,23 @@
 extern "C" {
 #endif
 
+/* String defines used for About fields */
+#define AJ_APP_ID_STR                  "AppId"
+#define AJ_DEFAULT_LANGUAGE_STR        "DefaultLanguage"
+#define AJ_DEVICE_NAME_STR             "DeviceName"
+#define AJ_DEVICE_ID_STR               "DeviceId"
+#define AJ_APP_NAME_STR                "AppName"
+#define AJ_MANUFACTURER_STR            "Manufacturer"
+#define AJ_MODEL_NUMBER_STR            "ModelNumber"
+#define AJ_SUPPORTED_LANGUAGES_STR     "SupportedLanguages"
+#define AJ_DESCRIPTION_STR             "Description"
+#define AJ_DATE_OF_MANUFACTURE_STR     "DateOfManufacture"
+#define AJ_SOFTWARE_VERSION_STR        "SoftwareVersion"
+#define AJ_AJSOFTWARE_VERSION_STR      "AJSoftwareVersion"
+#define AJ_HARDWARE_VERSION_STR        "HardwareVersion"
+#define AJ_SUPPORT_URL_STR             "SupportUrl"
+
+
 /**
  * Prototype for a function provided by the property store for getting ANNOUNCE
  * and ABOUT properties.  If the language tag given is not supported, use the
@@ -83,6 +100,13 @@ AJ_Status AJ_AboutAnnounce(AJ_BusAttachment* bus);
  */
 AJ_Status AJ_AboutUnannounce(AJ_BusAttachment* bus);
 
+/**
+ * Checks whether the icon has been set
+ *
+ * @return      TRUE if the icon struct in the About data has been filled; FALSE otherwise
+ */
+uint8_t AJ_AboutHasIcon();
+
 /**
  * Set a device icon to be returned by About
  *
