@@ -40,8 +40,9 @@ extern "C" {
 #define AUTH_SUITE_ECDHE_NULL  (AUTH_KEYX_ECDHE | 0x0001)
 #define AUTH_SUITE_ECDHE_PSK   (AUTH_KEYX_ECDHE | 0x0002)
 #define AUTH_SUITE_ECDHE_ECDSA (AUTH_KEYX_ECDHE | 0x0004)
+#define AUTH_SUITE_ECDHE_SPEKE (AUTH_KEYX_ECDHE | 0x0008)
 
-#define AJ_AUTH_SUITES_NUM     3    /**< Number of supported authentication suites */
+#define AJ_AUTH_SUITES_NUM     4    /**< Number of supported authentication suites */
 
 #define AUTH_CLIENT            0
 #define AUTH_SERVER            1
@@ -93,13 +94,28 @@ typedef struct _PSKContext {
 typedef struct _ECDSAContext {
     AJ_ECCPublicKey* key;                          /**< Array of public keys (subject + issuers) */
     size_t num;                                    /**< Number of public keys */
-    uint8_t manifest[AJ_SHA256_DIGEST_LENGTH];     /**< Manifest digest */
-    size_t size;                                   /**< Manifest size */
+    uint8_t thumbprint[AJ_SHA256_DIGEST_LENGTH];   /**< Identity certificate SHA-256 thumbprint */
+    size_t thumbprintSize;                         /**< Thumbprint size (should always be 0 or AJ_SHA256_DIGEST_LENGTH) */
 } ECDSAContext;
 
+/**
+ * Context for EC-SPEKE authentication
+ * The local GUID is read from the keystore, so we need to keep it in memory,
+ * while the remote GUID is already in memory during authentication.
+ */
+typedef struct _SPEKEContext {
+    AJ_GUID localGUID;                             /**< Our GUID */
+    const AJ_GUID* remoteGUID;                     /**< Our peer's GUID */
+} SPEKEContext;
+
+/**
+ * The KeyAuthenticationContext will hold the suite-specific context,
+ * depending on AJ_AuthenticationContext.suite.
+ */
 typedef struct _KeyAuthenticationContext {
     PSKContext psk;                                /**< Context for PSK authentication */
     ECDSAContext ecdsa;                            /**< Context for ECDSA authentication */
+    SPEKEContext speke;                            /**< Context for EC-SPEKE authentication */
 } KeyAuthenticationContext;
 
 /**
@@ -118,6 +134,7 @@ typedef struct _AJ_AuthenticationContext {
     uint16_t slot;                                 /**< NVRAM slot of membership certificate */
     uint8_t code;                                  /**< Send Membership code (NONE, MORE, LAST) */
     uint8_t digest[AJ_SHA256_DIGEST_LENGTH];       /**< Conversation digest */
+    uint8_t sensitiveMode;                         /**< Conversation digest tracing sensitive mode */
 } AJ_AuthenticationContext;
 
 /**
@@ -187,102 +204,6 @@ uint8_t AJ_IsSuiteEnabled(AJ_BusAttachment* bus, uint32_t suite, uint32_t versio
  */
 void AJ_EnableSuite(AJ_BusAttachment* bus, uint32_t suite);
 
-
-/**
- * Initialize/reset a conversation hash
- *
- * @param ctx          The authentication context
- *
- * @return
- *         - AJ_OK on success
- *         - An error status otherwise
- */
-AJ_Status AJ_ConversationHash_Initialize(AJ_AuthenticationContext* ctx);
-
-/**
- * Determine whether the conversation hash is initialized
- *
- * @param ctx          The authentication context
- *
- * @return
- *         - The value 1 if initialized
- *         - The value 0 if not initialized
- */
-uint8_t AJ_ConversationHash_IsInitialized(AJ_AuthenticationContext* ctx);
-
-/**
- * Update the conversation hash with a uint8_t
- *
- * @param ctx                   The authentication context
- * @param conversationVersion   The minimum authentication version which applies to this update
- * @param byte                  The byte to hash
- *
- */
-void AJ_ConversationHash_Update_UInt8(AJ_AuthenticationContext* ctx, uint32_t conversationVersion, uint8_t byte);
-
-/**
- * Update the conversation hash with a byte array
- *
- * @param ctx                   The authentication context
- * @param conversationVersion   The minimum authentication version which applies to this update
- * @param buf                   The input array to hash
- * @param bufSize               The size of buf
- *
- */
-void AJ_ConversationHash_Update_UInt8Array(AJ_AuthenticationContext* ctx, uint32_t conversationVersion, const uint8_t* buf, size_t bufSize);
-
-/**
- * Update the conversation hash with a string. This function does not assume a null-terminated
- * string; it will hash exactly the number of characters indicated by strSize.
- *
- * @param ctx                   The authentication context
- * @param conversationVersion   The minimum authentication version which applies to this update
- * @param str                   The string content to hash
- * @param strSize               The length of the string
- *
- */
-void AJ_ConversationHash_Update_String(AJ_AuthenticationContext* ctx, uint32_t conversationVersion, const char* str, size_t strSize);
-
-/**
- * Update the conversation hash with a message. If the message has been created locally with calls to AJ_Marshal*
- * functions, this function must be called BEFORE AJ_DeliverMsg or AJ_DeliverMsgPartial is called on it.
- *
- * If isMarshaledMessage is HASH_MSG_UNMARSHALED, the message will be reset to the beginning as a consequence
- * of calling this function. Any subsequent argument unmarshaling calls will start at the beginning of the message.
- * It is therefore recommended to call this function before or after all argument unmarshaling.
- *
- * @param ctx                   The authentication context
- * @param conversationVersion   The minimum authentication version which applies to this update
- * @param msg                   The pointer to a message that was unmarshaled by an earlier call to AJ_UnmarshalMsg
- *                              or created by calls to AJ_Marshal*
- * @param isMarshaledMessage    HASH_MSG_MARSHALED if msg was created locally through AJ_Marshal* calls,
- *                              or HASH_MSG_UNMARSHALED if received through a call to AJ_UnmarshalMsg
- *
- */
-void AJ_ConversationHash_Update_Message(AJ_AuthenticationContext* ctx, uint32_t conversationVersion, AJ_Message* msg, uint8_t isMarshaledMessage);
-
-/**
- * Get the conversation hash
- *
- * @param ctx           The authentication context
- *
- * @return
- *         - AJ_OK on success
- *         - An error status otherwise
- */
-AJ_Status AJ_ConversationHash_GetDigest(AJ_AuthenticationContext* ctx);
-
-/**
- * Reset the conversation hash
- *
- * @param ctx           The authentication context
- *
- * @return
- *         - AJ_OK on success
- *         - An error status otherwise
- */
-AJ_Status AJ_ConversationHash_Reset(AJ_AuthenticationContext* ctx);
-
 #ifdef __cplusplus
 }
 #endif
