@@ -41,8 +41,8 @@ extern "C" {
 #define AJ_MAX_TIMERS               4           //maximum number of timers              (aj_helper.c)
 #define AJ_ROUTING_NODE_BLACKLIST_SIZE 16       //maximum number of blacklisted routing nodes
 #define AJ_ROUTING_NODE_RESPONSELIST_SIZE 3     //maximum number of routing node responses to track
-#define AJ_TX_DATA_SIZE             3000        //minimum size of network transmit buffer
-#define AJ_RX_DATA_SIZE             3000        //minimum size of network receive buffer
+#define AJ_TX_DATA_SIZE             5000        //minimum size of network transmit buffer
+#define AJ_RX_DATA_SIZE             5000        //minimum size of network receive buffer
 
 /* Auth options */
 #define AJ_NONCE_LEN                28          //Length of the nonce.
@@ -92,6 +92,11 @@ extern "C" {
 #define AJ_MAX_NUM_OF_OBJ_DESC      (32)           //number of object descriptions in an Announcement payload (aj_about.c)
 #define AJ_MAX_NUM_OF_INTERFACES    (16)           //number of interfaces per object description in an Annoucement payload (aj_about.c)
 
+/* String defines */
+#define AJ_PASSCODE_STR                 "Passcode"
+#define AJ_REALM_NAME_STR               "RealmName"
+#define AJ_MAX_LENGTH_STR               "MaxLength"
+
 /* Below sets the actual #define's based on values above */
 #if _AJ_AUTH_DEBUG
 #define AUTH_DEBUG
