@@ -41,7 +41,7 @@ extern "C" {
 #define AJ_CRED_TYPE_PEM            0x0004 /**< PEM encoded type */
 #define AJ_CRED_TYPE_PUBLIC         0x0005 /**< public key type */
 #define AJ_CRED_TYPE_CERTIFICATE    0x0006 /**< Certificate type */
-#define AJ_CRED_TYPE_MANIFEST       0x0007 /**< manifest type */
+#define AJ_CRED_TYPE_MANIFESTS      0x0007 /**< manifests type */
 #define AJ_CRED_TYPE_POLICY         0x0008 /**< policy type */
 #define AJ_CRED_TYPE_CONFIG         0x0009 /**< config type */
 
@@ -49,13 +49,13 @@ extern "C" {
  * Type high byte is basic type (low byte) context specific
  */
 #define AJ_GENERIC_MASTER_SECRET    0x0000 /**< Peer master secret */
-#define AJ_GENERIC_ECDSA_MANIFEST   0x0100 /**< Manifest digest from ECDSA authentication */
+#define AJ_GENERIC_ECDSA_THUMBPRINT 0x0100 /**< Identity certificate thumbprint from ECDSA authentication */
 #define AJ_GENERIC_ECDSA_KEYS       0x0200 /**< Public keys from ECDSA authentication */
 #define AJ_ECC_SIG                  0x0000 /**< ECC key for communication */
 #define AJ_CERTIFICATE_OEM_X509     0x0000 /**< Manufacturer certificate */
-#define AJ_CERTIFICATE_IDN_X509     0x0100 /**< Identity certificate */
-#define AJ_CERTIFICATE_MBR_X509     0x0200 /**< Membership certificate */
-#define AJ_CERTIFICATE_UNR_X509     (AJ_CERTIFICATE_IDN_X509 | AJ_CERTIFICATE_MBR_X509) /**< Unrestricted certificate */
+#define AJ_CERTIFICATE_IDN_X509     0x0100 /**< AllJoyn identity certificate */
+#define AJ_CERTIFICATE_MBR_X509     0x0200 /**< AllJoyn membership certificate */
+#define AJ_CERTIFICATE_UNR_X509     (AJ_CERTIFICATE_IDN_X509 | AJ_CERTIFICATE_MBR_X509) /**< Unrestricted AllJoyn certificate */
 #define AJ_CERTIFICATE_INV_X509     0x0400 /**< Invalid certificate (EKUs present but no AllJoyn EKUs) */
 #define AJ_POLICY_DEFAULT           0x0000 /**< Default policy */
 #define AJ_POLICY_INSTALLED         0x0100 /**< Installed policy */
@@ -225,6 +225,17 @@ AJ_Status AJ_CredentialSetECCPrivateKey(uint16_t type, const AJ_CredField* id, u
  */
 AJ_Status AJ_CredentialGetECCPrivateKey(uint16_t type, const AJ_CredField* id, uint32_t* expiration, AJ_ECCPrivateKey* prv);
 
+/**
+ * Delete a credential from a specified slot in NVRAM
+ *
+ * @param type         Credential type
+ * @param slot         NVRAM slot
+ *
+ * @return
+ *          - AJ_OK if the credentials were deleted
+ */
+AJ_Status AJ_CredentialDeleteSlot(uint16_t type, uint16_t slot);
+
 /**
  * Delete a credential from NVRAM
  *
