@@ -22,11 +22,11 @@
  *    OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  ******************************************************************************/
 
-#include "aj_target.h"
-#include "aj_status.h"
-#include "aj_bus.h"
-#include "aj_util.h"
-#include "aj_guid.h"
+#include <ajtcl/aj_target.h>
+#include <ajtcl/aj_status.h>
+#include <ajtcl/aj_bus.h>
+#include <ajtcl/aj_util.h>
+#include <ajtcl/aj_guid.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -201,8 +201,22 @@ struct _AJ_Message {
     struct _AJ_Arg* outer;     /**< Container arg current being marshaled */
     uint32_t timeout;          /**< Remaining time to wait for all bytes of this message */
     uint32_t authVersion;      /**< Authentication version used */
+    uint8_t expired;           /**< For indicating whether the Rx message has expired */
+    AJ_MsgHeader raw;          /**< The raw original message header (before endian swaps) */
 };
 
+/**
+ * Reply context for deferred (asynchronous) method replies.
+ */
+typedef struct _AJ_MsgReplyContext {
+    AJ_BusAttachment* bus;             /**< Bus attachment */
+    uint32_t serialNum;                /**< Serial number from the method call */
+    uint32_t msgId;                    /**< Message id from the method call */
+    uint32_t sessionId;                /**< Session id from the method call */
+    uint8_t flags;                     /**< Flags from the method call */
+    char sender[AJ_MAX_NAME_SIZE + 1]; /**< Sender field from the method call */
+} AJ_MsgReplyContext;
+
 /**
  * Unmarshals a message returning a message structure. Note that if a message is received but is not
  * recognized this function will return an uninitialized msg with msgId == 0. The application must
@@ -358,6 +372,22 @@ AJ_Status AJ_UnmarshalVariant(AJ_Message* msg, const char** sig);
 AJ_EXPORT
 AJ_Status AJ_CloseMsg(AJ_Message* msg);
 
+/**
+ * Like AJ_CloseMsg(), this function closes an ummarshalled method call and release resources but
+ * returns a reply context that can be used later to generate a reply message. This allows an
+ * application to defer replying to a method call to a later time while still receiving other
+ * messages.
+ *
+ * @param msg       The message to close.
+ * @param replyCtx  A structure allocated by the caller to hold information needed to generate a
+ *                  reply message.
+ *
+ * @return   Return AJ_Status
+ */
+AJ_EXPORT
+AJ_Status AJ_CloseMsgAndSaveReplyContext(AJ_Message* msg, AJ_MsgReplyContext* replyCtx);
+
+
 /**
  * Type for a session identifier
  */
@@ -415,7 +445,7 @@ AJ_EXPORT
 AJ_Status AJ_MarshalReplyMsg(const AJ_Message* methodCall, AJ_Message* reply);
 
 /**
- * Initialize and marshal a message that is a error response to a method call.
+ * Initialize and marshal a message that is an error response to a method call.
  *
  * @param methodCall  The method call message that was received
  * @param reply       The reply to be initialized
@@ -427,7 +457,7 @@ AJ_EXPORT
 AJ_Status AJ_MarshalErrorMsg(const AJ_Message* methodCall, AJ_Message* reply, const char* error);
 
 /**
- * Initialize and marshal a message that is a error response to a method call.
+ * Initialize and marshal a message that is an error response to a method call.
  *
  * @param methodCall  The method call message that was received
  * @param reply       The reply to be initialized
@@ -441,9 +471,9 @@ AJ_EXPORT
 AJ_Status AJ_MarshalErrorMsgWithInfo(const AJ_Message* methodCall, AJ_Message* reply, const char* error, const char* info);
 
 /**
- * Initialize and marshal a message that is a error response to a method call. This is a wrapper
- * function around AJ_MarshalErrorMsg that chooses an appropriate error message depending on the
- * AJ_Status value passed in.
+ * Marshal a message that is an error response to a method call. This is a wrapper function around
+ * AJ_MarshalErrorMsg() that chooses an appropriate error message depending on the AJ_Status value
+ * passed in.
  *
  * @param methodCall  The method call message that was received
  * @param reply       The reply to be initialized
@@ -451,9 +481,69 @@ AJ_Status AJ_MarshalErrorMsgWithInfo(const AJ_Message* methodCall, AJ_Message* r
  *
  * @return   Return AJ_Status
  */
+
 AJ_EXPORT
 AJ_Status AJ_MarshalStatusMsg(const AJ_Message* methodCall, AJ_Message* reply, AJ_Status status);
 
+/**
+ * Use a saved reply context to initialize and marshal a message that is a reply to a method call.
+ *
+ * This function clears the reply context before returning.
+ *
+ * @param replyCtx  The saved context for generating a reply to the method call
+ * @param reply     The reply message to be initialized
+ *
+ * @return   Return AJ_Status
+ */
+AJ_EXPORT
+AJ_Status AJ_MarshalReplyMsgAsync(AJ_MsgReplyContext* replyCtx, AJ_Message* reply);
+
+/**
+ * Use a saved reply context to initialize and marshal a message that is an error response to a method call.
+ *
+ * This function clears the reply context before returning.
+ *
+ * @param replyCtx  The saved context for generating a reply to the method call
+ * @param reply     The reply message to be initialized
+ * @param error     The error name to use in the response.
+ *
+ * @return   Return AJ_Status
+ */
+AJ_EXPORT
+AJ_Status AJ_MarshalErrorMsgAsync(AJ_MsgReplyContext* replyCtx, AJ_Message* reply, const char* error);
+
+/**
+ * Use a saved reply context to initialize and marshal a message that is an error response to a method call.
+ *
+ * This function clears the reply context before returning.
+ *
+ * @param replyCtx  The saved context for generating a reply to the method call
+ * @param reply     The reply to be initialized
+ * @param error     The error name to use in the response.
+ * @param info      A text string that provides additional information about the error. Can be
+ *                  NULL in which case this is equivalent to calling AJ_MarshalErrorFromReplyContext().
+ *
+ * @return   Return AJ_Status
+ */
+AJ_EXPORT
+AJ_Status AJ_MarshalErrorMsgWithInfoAsync(AJ_MsgReplyContext* replyCtx, AJ_Message* reply, const char* error, const char* info);
+
+/**
+ * Use a saved reply context to initialize and marshal a message that is an error response to a
+ * method call.  This is a wrapper function around AJ_MarshalErrorMsgAsync() that chooses an
+ * appropriate error message depending on the AJ_Status value passed in.
+ *
+ * This function clears the reply context before returning.
+ *
+ * @param replyCtx  The saved context for generating a reply to the method call
+ * @param reply     The reply to be initialized
+ * @param status    The status code for the error
+ *
+ * @return   Return AJ_Status
+ */
+AJ_EXPORT
+AJ_Status AJ_MarshalStatusMsgAsync(AJ_MsgReplyContext* replyCtx, AJ_Message* reply, AJ_Status status);
+
 /**
  * Delivers a marshalled message to the network.
  *
@@ -462,6 +552,7 @@ AJ_Status AJ_MarshalStatusMsg(const AJ_Message* methodCall, AJ_Message* reply, A
  * @return
  *          - AJ_OK if the message was succesfully delivered
  *          - AJ_ERR_MARSHAL if the message arguments were incompletely marshaled
+ *          - AJ_ERR_WRITE if there was a write failure
  */
 AJ_EXPORT
 AJ_Status AJ_DeliverMsg(AJ_Message* msg);
@@ -480,7 +571,7 @@ AJ_Status AJ_DeliverMsg(AJ_Message* msg);
  * @return
  *          - AJ_OK if the message partial delivery was successful
  *          - AJ_ERR_SIGNATURE if there are no arguments left to marshal
- *
+ *          - AJ_ERR_WRITE if there was a write failure
  */
 AJ_EXPORT
 AJ_Status AJ_DeliverMsgPartial(AJ_Message* msg, uint32_t bytesRemaining);
@@ -601,6 +692,18 @@ AJ_Status AJ_MarshalCloseContainer(AJ_Message* msg, AJ_Arg* arg);
 AJ_EXPORT
 AJ_Status AJ_MarshalVariant(AJ_Message* msg, const char* sig);
 
+/**
+ * Create a message for local marshal or unmarshal
+ *
+ * @param bus  A dummy bus attachment
+ * @param hdr  The message header
+ * @param msg  The message
+ * @param sig  The message signature
+ * @param data The buffer for the message body
+ * @param size The buffer size
+ */
+void AJ_LocalMsg(AJ_BusAttachment* bus, AJ_MsgHeader* hdr, AJ_Message* msg, const char* sig, uint8_t* data, size_t size);
+
 /**
  * Checks an incoming method or signal for a valid serial number.
  *
