@@ -197,35 +197,35 @@ AJ_Status AJ_PeerHandleKeyAuthentication(AJ_Message* msg, AJ_Message* reply);
 AJ_Status AJ_PeerHandleKeyAuthenticationReply(AJ_Message* msg);
 
 /**
- * Handle a send manifest message, an array of guild memberships
+ * Handle a send manifests message
  *
- * @param msg    The send manifest message
- * @param reply  The send manifest reply message
+ * @param msg    The send manifests message
+ * @param reply  The send manifests reply message
  *
  * @return   Return AJ_Status
  *         - AJ_OK if successful
  *         - AJ_ERR_RESOURCES if resource error or authentication in progress
  *         - AJ_ERR_SECURITY if generic security violation
  */
-AJ_Status AJ_PeerHandleSendManifest(AJ_Message* msg, AJ_Message* reply);
+AJ_Status AJ_PeerHandleSendManifests(AJ_Message* msg, AJ_Message* reply);
 
 /**
- * Handle a send manifest reply
+ * Handle a send manifests reply
  *
- * @param msg    The send manifest reply message
+ * @param msg    The send manifests reply message
  *
  * @return   Return AJ_Status
  *         - AJ_OK if successful
  *         - AJ_ERR_RESOURCES if resource error or authentication in progress
  *         - AJ_ERR_SECURITY if generic security violation
  */
-AJ_Status AJ_PeerHandleSendManifestReply(AJ_Message* msg);
+AJ_Status AJ_PeerHandleSendManifestsReply(AJ_Message* msg);
 
 /**
  * Handle a send memberships message, an array of guild memberships
  *
- * @param msg    The send membserships message
- * @param reply  The send membserships reply message
+ * @param msg    The send memberships message
+ * @param reply  The send memberships reply message
  *
  * @return   Return AJ_Status
  *         - AJ_OK if successful
@@ -237,7 +237,7 @@ AJ_Status AJ_PeerHandleSendMemberships(AJ_Message* msg, AJ_Message* reply);
 /**
  * Handle a send memberships reply
  *
- * @param msg    The send membserships reply message
+ * @param msg    The send memberships reply message
  *
  * @return   Return AJ_Status
  *         - AJ_OK if successful
@@ -246,11 +246,29 @@ AJ_Status AJ_PeerHandleSendMemberships(AJ_Message* msg, AJ_Message* reply);
  */
 AJ_Status AJ_PeerHandleSendMembershipsReply(AJ_Message* msg);
 
+/**
+ * Request manifests be sent to peer
+ *
+ * @param msg      The message being sent or received
+ * @param outgoing TRUE if msg is an outgoing message, FALSE if not
+ *
+ * @return   Return AJ_Status
+ *         - AJ_OK if successful
+ *         - AJ_ERR_RESOURCES if resource error or authentication in progress
+ *         - AJ_ERR_SECURITY if generic security violation
+ */
+AJ_Status AJ_PeerSendManifests(AJ_Message* msg, uint8_t outgoing);
+
 /**
  * Clear the authentication handshake context
  */
 void AJ_ClearAuthContext();
 
+/**
+ * Clear the flag that determines if we've sent manifests to the peer
+ */
+void AJ_ClearSentManifests();
+
 #ifdef __cplusplus
 }
 #endif
