@@ -146,22 +146,22 @@ static AJ_Status MyAboutPropGetter(AJ_Message* reply, const char* language)
 
     status = AJ_MarshalContainer(reply, &array, AJ_ARG_ARRAY);
     if (status == AJ_OK) {
-        status = AJ_MarshalArgs(reply, "{sv}", "AppId", "ay", appId, 16);
+        status = AJ_MarshalArgs(reply, "{sv}", AJ_APP_ID_STR, "ay", appId, 16);
     }
     if (status == AJ_OK) {
-        status = AJ_MarshalArgs(reply, "{sv}", "AppName", "s", "eventaction_service");
+        status = AJ_MarshalArgs(reply, "{sv}", AJ_APP_NAME_STR, "s", "eventaction_service");
     }
     if (status == AJ_OK) {
-        status = AJ_MarshalArgs(reply, "{sv}", "DeviceId", "s", guidStr);
+        status = AJ_MarshalArgs(reply, "{sv}", AJ_DEVICE_ID_STR, "s", guidStr);
     }
     if (status == AJ_OK) {
-        status = AJ_MarshalArgs(reply, "{sv}", "DeviceName", "s", "Tester");
+        status = AJ_MarshalArgs(reply, "{sv}", AJ_DEVICE_NAME_STR, "s", "Tester");
     }
     if (status == AJ_OK) {
-        status = AJ_MarshalArgs(reply, "{sv}", "Manufacturer", "s", "QCE");
+        status = AJ_MarshalArgs(reply, "{sv}", AJ_MANUFACTURER_STR, "s", "QCE");
     }
     if (status == AJ_OK) {
-        status = AJ_MarshalArgs(reply, "{sv}", "ModelNumber", "s", "1.0");
+        status = AJ_MarshalArgs(reply, "{sv}", AJ_MODEL_NUMBER_STR, "s", "1.0");
     }
     //SupportedLanguages
     if (status == AJ_OK) {
@@ -169,7 +169,7 @@ static AJ_Status MyAboutPropGetter(AJ_Message* reply, const char* language)
         AJ_Arg languageListArray;
         status = AJ_MarshalContainer(reply, &dict, AJ_ARG_DICT_ENTRY);
         if (status == AJ_OK) {
-            status = AJ_MarshalArgs(reply, "s", "SupportedLanguages");
+            status = AJ_MarshalArgs(reply, "s", AJ_SUPPORTED_LANGUAGES_STR);
         }
         if (status == AJ_OK) {
             status = AJ_MarshalVariant(reply, "as");
@@ -191,16 +191,16 @@ static AJ_Status MyAboutPropGetter(AJ_Message* reply, const char* language)
         }
     }
     if (status == AJ_OK) {
-        status = AJ_MarshalArgs(reply, "{sv}", "Description", "s", "eventaction_service test app");
+        status = AJ_MarshalArgs(reply, "{sv}", AJ_DESCRIPTION_STR, "s", "eventaction_service test app");
     }
     if (status == AJ_OK) {
-        status = AJ_MarshalArgs(reply, "{sv}", "DefaultLanguage", "s", languages[0]);
+        status = AJ_MarshalArgs(reply, "{sv}", AJ_DEFAULT_LANGUAGE_STR, "s", languages[0]);
     }
     if (status == AJ_OK) {
-        status = AJ_MarshalArgs(reply, "{sv}", "SoftwareVersion", "s", AJ_GetVersion());
+        status = AJ_MarshalArgs(reply, "{sv}", AJ_SOFTWARE_VERSION_STR, "s", AJ_GetVersion());
     }
     if (status == AJ_OK) {
-        status = AJ_MarshalArgs(reply, "{sv}", "AJSoftwareVersion", "s", AJ_GetVersion());
+        status = AJ_MarshalArgs(reply, "{sv}", AJ_AJSOFTWARE_VERSION_STR, "s", AJ_GetVersion());
     }
     if (status == AJ_OK) {
         status = AJ_MarshalCloseContainer(reply, &array);
@@ -241,10 +241,9 @@ static const AJ_Object AppObjects[] = {
 
 static AJ_Status AppHandleCat(AJ_Message* msg)
 {
-#define BUFFER_SIZE 256
     const char* string0;
     const char* string1;
-    char buffer[BUFFER_SIZE];
+    char tmpBuffer[256];
     AJ_Message reply;
     AJ_Arg replyArg;
 
@@ -252,17 +251,14 @@ static AJ_Status AppHandleCat(AJ_Message* msg)
     AJ_MarshalReplyMsg(msg, &reply);
 
     /* We have the arguments. Now do the concatenation. */
-    strncpy(buffer, string0, BUFFER_SIZE);
-    buffer[BUFFER_SIZE - 1] = '\0';
-    strncat(buffer, string1, BUFFER_SIZE - strlen(buffer));
-    buffer[BUFFER_SIZE - 1] = '\0';
+    strncpy(tmpBuffer, string0, ArraySize(tmpBuffer));
+    tmpBuffer[ArraySize(tmpBuffer) - 1] = '\0';
+    strncat(tmpBuffer, string1, ArraySize(tmpBuffer) - strlen(tmpBuffer) - 1);
 
-    AJ_InitArg(&replyArg, AJ_ARG_STRING, 0, buffer, 0);
+    AJ_InitArg(&replyArg, AJ_ARG_STRING, 0, tmpBuffer, 0);
     AJ_MarshalArg(&reply, &replyArg);
 
     return AJ_DeliverMsg(&reply);
-
-#undef BUFFER_SIZE
 }
 
 /* All times are expressed in milliseconds. */
