@@ -48,6 +48,7 @@ static struct {
     const uint8_t* data;
     const char* mime;
     const char* URL;
+    uint8_t isSet;
 } icon;
 
 /*
@@ -82,12 +83,18 @@ AJ_Status AJ_AboutInit(AJ_BusAttachment* bus, uint16_t boundPort)
     return AJ_AboutAnnounce(bus);
 }
 
+uint8_t AJ_AboutHasIcon()
+{
+    return icon.isSet;
+}
+
 void AJ_AboutSetIcon(const uint8_t* data, uint16_t size, const char* mime, const char* url)
 {
     icon.data = data;
     icon.size = data ? size : 0;
     icon.mime = mime;
     icon.URL = url;
+    icon.isSet = TRUE;
 }
 
 /*
@@ -492,19 +499,19 @@ AJ_Status AJ_AboutUnmarshalProps(AJ_Message* msg, AJ_AboutHandleMandatoryProps o
             break;
         }
 
-        if (!strcmp(key, "AppId")) {
+        if (!strcmp(key, AJ_APP_ID_STR)) {
             status = AJ_AboutUnmarshalAppIdFromVariant(msg, appId, sizeof(appId));
-        } else if (!strcmp(key, "AppName")) {
+        } else if (!strcmp(key, AJ_APP_NAME_STR)) {
             status = AJ_UnmarshalArgs(msg, "v", "s", &appName);
-        } else if (!strcmp(key, "DeviceId")) {
+        } else if (!strcmp(key, AJ_DEVICE_ID_STR)) {
             status = AJ_UnmarshalArgs(msg, "v", "s", &deviceId);
-        } else if (!strcmp(key, "DeviceName")) {
+        } else if (!strcmp(key, AJ_DEVICE_NAME_STR)) {
             status = AJ_UnmarshalArgs(msg, "v", "s", &deviceName);
-        } else if (!strcmp(key, "Manufacturer")) {
+        } else if (!strcmp(key, AJ_MANUFACTURER_STR)) {
             status = AJ_UnmarshalArgs(msg, "v", "s", &manufacturer);
-        } else if (!strcmp(key, "ModelNumber")) {
+        } else if (!strcmp(key, AJ_MODEL_NUMBER_STR)) {
             status = AJ_UnmarshalArgs(msg, "v", "s", &modelNumber);
-        } else if (!strcmp(key, "DefaultLanguage")) {
+        } else if (!strcmp(key, AJ_DEFAULT_LANGUAGE_STR)) {
             status = AJ_UnmarshalArgs(msg, "v", "s", &defaultLanguage);
         } else {
             if (onOptionalProperty == NULL) {
