@@ -27,12 +27,12 @@ extern "C" {
 
 #define AJ_MODULE ARDP
 
-#include "aj_ardp.h"
-#include "aj_msg.h"
-#include "aj_net.h"
-#include "aj_crypto.h"
-#include "aj_debug.h"
-#include "aj_util.h"
+#include <ajtcl/aj_ardp.h>
+#include <ajtcl/aj_msg.h>
+#include <ajtcl/aj_net.h>
+#include <ajtcl/aj_crypto.h>
+#include <ajtcl/aj_debug.h>
+#include <ajtcl/aj_util.h>
 
 #ifndef NDEBUG
 uint8_t dbgARDP = 0;
@@ -76,6 +76,12 @@ uint8_t dbgARDP = 0;
 /* Minimum Delayed ACK Timeout */
 #define ARDP_MIN_DELAYED_ACK_TIMEOUT 10
 
+/* ARDP backpressure relief retry interval  */
+#define ARDP_BACKPRESSURE_INTERVAL 100
+
+/* ARDP backpressure relief maximum number of retries */
+#define ARDP_MAX_BACKPRESSURE_RETRIES ((UDP_LINK_TIMEOUT) / (ARDP_BACKPRESSURE_INTERVAL))
+
 #define MIN(a, b) ((a) < (b) ? (a) : (b))
 #define MAX(a, b) ((a) > (b) ? (a) : (b))
 #define ABS(a) ((a) >= 0 ? (a) : -(a))
@@ -227,6 +233,7 @@ struct ArdpConnection {
     uint32_t backoff;       /* Backoff factor accounting for retransmits on connection, resets to 1 when receive "good ack" */
     uint32_t rttMeanUnit;   /* Smoothed RTT value per UDP MTU */
     uint8_t rttInit;        /* Flag indicating that the first RTT was measured and SRTT calculation applies */
+    uint8_t confirm;        /* Flag to indicate that progress happened, do not send ARP re-probe */
 };
 
 static struct ArdpConnection* conn = NULL;
@@ -262,15 +269,15 @@ static SendFunction sendFunction;
  * End of definitions
  */
 
-void AJ_ARDP_InitFunctions(ReceiveFunction recv, SendFunction send)
+void AJ_ARDP_InitFunctions(ReceiveFunction rcvFunc, SendFunction sndFunc)
 {
-    recvFunction = recv;
-    sendFunction = send;
+    recvFunction = rcvFunc;
+    sendFunction = sndFunc;
 }
 
 static AJ_Status InitConnection()
 {
-    uint32_t rand;
+    uint32_t rand32;
     uint32_t i;
 
     conn = (struct ArdpConnection*) AJ_Malloc(sizeof(struct ArdpConnection));
@@ -279,8 +286,8 @@ static AJ_Status InitConnection()
     }
     memset(conn, 0, sizeof(struct ArdpConnection));
 
-    AJ_RandBytes((uint8_t*) &rand, sizeof(uint32_t));
-    conn->local = (rand % 65534) + 1;  /* Allocate an "ephemeral" source port */
+    AJ_RandBytes((uint8_t*) &rand32, sizeof(uint32_t));
+    conn->local = (rand32 % 65534) + 1;  /* Allocate an "ephemeral" source port */
 
     /* Initialize the sender side of the connection */
     AJ_RandBytes((uint8_t*) &conn->snd.ISS, sizeof(conn->snd.ISS));
@@ -328,8 +335,9 @@ static AJ_Status SendHeader(uint8_t flags)
 {
     uint32_t buf32[ARDP_HEADER_SIZE >> 2];
     size_t sent;
+    AJ_Status status;
 
-    AJ_InfoPrintf(("SendHeader(flags=0x%02x, seq=%u, ack=%u)\n", flags, conn->snd.NXT, conn->rcv.CUR));
+    AJ_InfoPrintf(("SendHeader(flags=0x%02x, seq=%u, ack=%u, lcs=%u)\n", flags, conn->snd.NXT, conn->rcv.CUR, conn->rcv.LCS));
 
     /* Marshal the header structure into a byte buffer */
     MarshalHeader(buf32, flags, 0, ARDP_TTL_INFINITE, 0, 0);
@@ -338,7 +346,11 @@ static AJ_Status SendHeader(uint8_t flags)
     conn->ackTimer.retry = 0;
     conn->rcv.pending = 0;
 
-    return (*sendFunction)(conn->context, (uint8_t*) &buf32, ARDP_HEADER_SIZE, &sent);
+    status = (*sendFunction)(conn->context, (uint8_t*) &buf32, ARDP_HEADER_SIZE, &sent, conn->confirm);
+    if (status == AJ_OK) {
+        conn->confirm = FALSE;
+    }
+    return status;
 }
 
 static AJ_Status SendSyn(uint16_t dataLen)
@@ -360,7 +372,15 @@ static AJ_Status SendSyn(uint16_t dataLen)
     *((uint16_t*) (txbuf + OPTIONS_OFFSET)) = htons(ARDP_FLAG_SIMPLE_MODE | ARDP_FLAG_SDM);
     *((uint16_t*) (txbuf + SYN_RSRV_OFFSET)) = 0;
 
-    return (*sendFunction)(conn->context, (uint8_t*) &conn->snd.buf[0].data[0], ARDP_SYN_HEADER_SIZE + dataLen, &sent);
+    return (*sendFunction)(conn->context, (uint8_t*) &conn->snd.buf[0].data[0], ARDP_SYN_HEADER_SIZE + dataLen, &sent, FALSE);
+}
+
+static uint8_t IsDataRetransmitScheduled()
+{
+    if (((conn->snd.UNA + 1) != conn->snd.NXT) && (conn->snd.UNA != conn->snd.NXT)) {
+        return TRUE;
+    }
+    return FALSE;
 }
 
 static void InitTimer(struct ArdpTimer* timer, uint32_t delta, uint8_t retry)
@@ -380,7 +400,7 @@ static AJ_Status ConnectTimerHandler()
         size_t sent;
         uint16_t len = conn->snd.buf[0].dataLen + ARDP_SYN_HEADER_SIZE;
         AJ_InfoPrintf(("ConnectTimerHandler: send %d bytes\n", len));
-        status = (*sendFunction)(conn->context, (uint8_t*) conn->snd.buf[0].data, len, &sent);
+        status = (*sendFunction)(conn->context, (uint8_t*) conn->snd.buf[0].data, len, &sent, FALSE);
         if (status == AJ_ERR_WOULD_BLOCK) {
             status = AJ_OK;
         }
@@ -393,7 +413,6 @@ static AJ_Status ConnectTimerHandler()
         conn->state = CLOSED;
         conn->connectTimer.retry = 0;
         AJ_ErrPrintf(("ConnectTimerHandler(): %s\n", AJ_StatusText(status)));
-        printf("ConnectTimerHandler(): %s\n", AJ_StatusText(status));
         AJ_Free(conn);
         conn = NULL;
         return AJ_ERR_CONNECT;
@@ -427,14 +446,16 @@ static AJ_Status DataTimerHandler(ArdpSBuf* sBuf)
     struct ArdpTimer* timer = &sBuf->timer;
     uint32_t msElapsed = AJ_GetElapsedTime(&sBuf->tStart, FALSE);
     uint32_t timeout = GetDataTimeout();
-
-#ifndef NDEBUG
-    uint32_t seq = ntohl(*(uint32_t*)((uint8_t*)sBuf->data + SEQ_OFFSET));
-#endif
+    ArdpSBuf* startBuf = sBuf;
 
     sBuf->retransmits++;
 
     do {
+
+#ifndef NDEBUG
+        uint32_t seq = ntohl(*(uint32_t*)((uint8_t*)sBuf->data + SEQ_OFFSET));
+#endif
+
         if ((msElapsed >= timeout) && (timer->retry > UDP_MIN_DATA_RETRIES)) {
             AJ_ErrPrintf(("DataTimerHandler(): hit timeout for %u\n", seq));
             status = AJ_ERR_TIMEOUT;
@@ -450,7 +471,7 @@ static AJ_Status DataTimerHandler(ArdpSBuf* sBuf)
             *((uint32_t*) (txbuf + ACKNXT_OFFSET)) = htonl(conn->snd.UNA);
 
             AJ_InfoPrintf(("DataTimerHandler: send %d bytes (seq %u, ack %u)\n", len, seq, conn->rcv.CUR));
-            status =  (*sendFunction)(conn->context, (uint8_t*) sBuf->data, len, &sent);
+            status =  (*sendFunction)(conn->context, (uint8_t*) sBuf->data, len, &sent, conn->confirm);
             AJ_InitTimer(&sBuf->timer.tStart);
 
             if (status == AJ_OK) {
@@ -458,32 +479,35 @@ static AJ_Status DataTimerHandler(ArdpSBuf* sBuf)
                 if (conn->rttInit) {
                     timer->delta = GetRTO();
                 } else {
-                    timer->delta = UDP_INITIAL_DATA_TIMEOUT;
+                    timer->delta = MIN(UDP_INITIAL_DATA_TIMEOUT << conn->backoff, (uint32_t)ARDP_MAX_RTO);
                 }
+                AJ_InfoPrintf(("DataTimerHandler: backoff %u, delta %u\n", conn->backoff, timer->delta));
+
                 timer->retry++;
                 AJ_InfoPrintf(("DataTimerHandler: cancel ackTimer\n"));
                 conn->ackTimer.retry = 0;
                 conn->rcv.pending = 0;
+                conn->confirm = FALSE;
             } else {
                 AJ_ErrPrintf(("DataTimerHandler():Write to Socket went bad"));
             }
         }
         sBuf = sBuf->next;
-    } while (status == AJ_OK && (sBuf->timer.retry != 0)); /* Here "retry" check equates checking for "in flight" */
+    } while (status == AJ_OK && (sBuf->timer.retry != 0) && (sBuf != startBuf)); /* Here "retry" check equates checking for "in flight" */
 
     return status;
 }
 
 static AJ_Status CheckDataTimers()
 {
-    uint32_t index;
+    uint32_t idx;
 
     /* Check data retransmit timer */
-    index = conn->snd.UNA % UDP_SEGMAX;
-    if (conn->snd.buf[index].timer.retry != 0 &&
-        AJ_GetElapsedTime(&conn->snd.buf[index].timer.tStart, TRUE) >= conn->snd.buf[index].timer.delta) {
+    idx = conn->snd.UNA % UDP_SEGMAX;
+    if (conn->snd.buf[idx].timer.retry != 0 &&
+        AJ_GetElapsedTime(&conn->snd.buf[idx].timer.tStart, TRUE) >= conn->snd.buf[idx].timer.delta) {
         AJ_InfoPrintf(("CheckDataTimers: Fire data timer\n"));
-        return DataTimerHandler(&conn->snd.buf[index]);
+        return DataTimerHandler(&conn->snd.buf[idx]);
     }
     return AJ_OK;
 }
@@ -516,6 +540,9 @@ static AJ_Status CheckTimers()
         status = SendHeader(ARDP_FLAG_ACK | ARDP_FLAG_VER | ARDP_FLAG_NUL);
         AJ_InitTimer(&conn->probeTimer.tStart);
         conn->probeTimer.retry--;
+        if (IsDataRetransmitScheduled() == FALSE) {
+            conn->rttInit = FALSE;
+        }
     }
 
     status = CheckDataTimers();
@@ -608,11 +635,10 @@ static AJ_Status RecvValidateSegment(uint8_t* rxbuf, uint16_t len, struct ArdpSe
     }
 
     /*
-     * SEQ and ACKNXT must fall within receive window. In case of segment with no payload,
-     * allow one extra.
+     * SEQ and ACKNXT must fall within receive window.
      */
-    if (((seg->SEQ - seg->ACKNXT) > UDP_SEGMAX) || (SEQ32_LT(seg->SEQ, seg->ACKNXT)) ||
-        ((seg->DLEN != 0) && ((seg->SEQ - seg->ACKNXT) == UDP_SEGMAX))) {
+    if ((SEQ32_LT(seg->SEQ, seg->ACKNXT)) ||
+        ((seg->DLEN != 0) && ((seg->SEQ - seg->ACKNXT) >= UDP_SEGMAX))) {
         AJ_ErrPrintf(("Receive: incorrect sequence numbers seg->seq = %u, seg->acknxt = %u\n",
                       seg->SEQ, seg->ACKNXT));
         return AJ_ERR_INVALID;
@@ -666,7 +692,7 @@ static void AdjustRTT(ArdpSBuf* sBuf)
         conn->rttMeanVar = (conn->rttMeanVar * 31 + ABS(err)) >> 5;
     }
 
-    rttUnit = (7 * conn->rttMeanUnit + rttUnit) >> 3;
+    conn->rttMeanUnit = (7 * conn->rttMeanUnit + rttUnit) >> 3;
 
     conn->backoff = 0;
 
@@ -675,8 +701,8 @@ static void AdjustRTT(ArdpSBuf* sBuf)
 
 static void UpdateSndSegments(uint32_t ack)
 {
-    uint16_t index = ack % UDP_SEGMAX;
-    ArdpSBuf* sBuf = &conn->snd.buf[index];
+    uint16_t idx = ack % UDP_SEGMAX;
+    ArdpSBuf* sBuf = &conn->snd.buf[idx];
     uint32_t i;
 
     /* Nothing to clean up */
@@ -697,10 +723,10 @@ static void UpdateSndSegments(uint32_t ack)
     for (i = 0; i < UDP_SEGMAX; i++) {
         uint32_t seq = ntohl(*(uint32_t*)((uint8_t*)(sBuf->data) + SEQ_OFFSET));
 
-        /* If the remote acknowledged the segment, stop retransmit attempts. */
-        AJ_InfoPrintf(("UpdateSndSegments(): cancel retransmit for %u\n", seq));
-
         if (SEQ32_LET(seq, ack) && (sBuf->inFlight != 0)) {
+            /* If the remote acknowledged the segment, stop retransmit attempts. */
+            AJ_InfoPrintf(("UpdateSndSegments(): cancel retransmit for %u\n", seq));
+
             sBuf->timer.retry = 0;
             sBuf->dataLen = 0;
             sBuf->inFlight = 0;
@@ -715,25 +741,54 @@ static void UpdateSndSegments(uint32_t ack)
     }
 }
 
-static void AddRcvBuffer(struct ArdpSeg* seg, uint8_t* rxBuf, uint16_t dataOffset)
+static void FlushExpiredRcvMessages(uint32_t seq, uint32_t ackNXT)
 {
-    uint32_t index = seg->SEQ % UDP_SEGMAX;
+    uint32_t idx =  conn->rcv.CUR % UDP_SEGMAX;
+    ArdpRBuf* rBuf = &conn->rcv.buf[idx];
 
-    AJ_InfoPrintf(("AddRcvBuffer: seq=%u\n", seg->SEQ));
-    AJ_ASSERT(conn->rcv.buf[index].fcnt == 0);
+    AJ_InfoPrintf(("FlushExpiredRcvMessages: seq = %u, expected %u got %u\n",
+                   seq, conn->rcv.CUR + 1, ackNXT));
+    while (SEQ32_LT(conn->rcv.CUR + 1, ackNXT)) {
+        rBuf->fcnt = 0;
+        rBuf->dataLen = 0;
+        rBuf = rBuf->next;
+        conn->rcv.CUR++;
+    }
 
-    conn->rcv.buf[index].seq = seg->SEQ;
-    conn->rcv.buf[index].som = seg->SOM;
-    conn->rcv.buf[index].fcnt = seg->FCNT;
-    conn->rcv.buf[index].dataLen = seg->DLEN;
-    memcpy(conn->rcv.buf[index].data, rxBuf + dataOffset, seg->DLEN);
-
-    if (UDP_Recv_State.rxContext == NULL) {
-        UDP_Recv_State.readBuf = conn->rcv.buf[index].data;
-        UDP_Recv_State.dataLen = seg->DLEN;
-        UDP_Recv_State.rxContext = (void*) &conn->rcv.buf[index];
+    /*
+     * If the next rBuf contains valid data, update internal ARDP Rx context
+     * to point to the payload. Otherwise, there are no pending received data,
+     * reset Rx context to NULL.
+     */
+    if (rBuf->dataLen != 0) {
+        UDP_Recv_State.readBuf = rBuf->data;
+        UDP_Recv_State.dataLen = rBuf->dataLen;
+        UDP_Recv_State.rxContext = (void*) rBuf;
+    } else {
+        UDP_Recv_State.rxContext = NULL;
     }
 }
+
+static void AddRcvBuffer(struct ArdpSeg* seg, uint8_t* rxBuf, uint16_t dataOffset)
+{
+    uint32_t idx = seg->SEQ % UDP_SEGMAX;
+
+    AJ_InfoPrintf(("AddRcvBuffer: seq=%u\n", seg->SEQ));
+    AJ_ASSERT(conn->rcv.buf[idx].fcnt == 0);
+
+    conn->rcv.buf[idx].seq = seg->SEQ;
+    conn->rcv.buf[idx].som = seg->SOM;
+    conn->rcv.buf[idx].fcnt = seg->FCNT;
+    conn->rcv.buf[idx].dataLen = seg->DLEN;
+    memcpy(conn->rcv.buf[idx].data, rxBuf + dataOffset, seg->DLEN);
+
+    if (UDP_Recv_State.rxContext == NULL) {
+        UDP_Recv_State.readBuf = conn->rcv.buf[idx].data;
+        UDP_Recv_State.dataLen = seg->DLEN;
+        UDP_Recv_State.rxContext = (void*) &conn->rcv.buf[idx];
+    }
+}
+
 static AJ_Status ArdpMachine(struct ArdpSeg* seg, uint8_t* rxBuf, uint16_t len)
 {
     AJ_Status status = AJ_OK;
@@ -770,21 +825,23 @@ static AJ_Status ArdpMachine(struct ArdpSeg* seg, uint8_t* rxBuf, uint16_t len)
                         /* Initialize and kick off link timeout timer */
                         InitTimer(&conn->probeTimer, UDP_LINK_TIMEOUT / UDP_KEEPALIVE_RETRIES, UDP_KEEPALIVE_RETRIES);
 
+                        conn->confirm = TRUE;
+
                         /*
                          * <SEQ=snd.NXT><ACK=RCV.CUR><ACK>
                          */
                         status = SendHeader(ARDP_FLAG_ACK | ARDP_FLAG_VER);
 
+                        if (status == AJ_OK) {
+                            AddRcvBuffer(seg, rxBuf, ARDP_SYN_HEADER_SIZE);
+                        }
                     }
                 }
-            }
 
-            if (status == AJ_OK) {
-                AddRcvBuffer(seg, rxBuf, ARDP_SYN_HEADER_SIZE);
-            }
+                /* Stop connect retry timer */
+                conn->connectTimer.retry = 0;
 
-            /* Stop connect retry timer */
-            conn->connectTimer.retry = 0;
+            }
 
             break;
         }
@@ -792,7 +849,7 @@ static AJ_Status ArdpMachine(struct ArdpSeg* seg, uint8_t* rxBuf, uint16_t len)
     case CLOSE_WAIT:
     case OPEN:
         {
-            AJ_InfoPrintf(("ArdpMachine(): conn->state = OPEN\n"));
+            AJ_InfoPrintf(("ArdpMachine(): conn->state = %s\n", (conn->state == CLOSE_WAIT) ? "CLOSE_WAIT" : "OPEN"));
 
             if (seg->FLG & ARDP_FLAG_SYN) {
                 /* Ignore */
@@ -800,7 +857,7 @@ static AJ_Status ArdpMachine(struct ArdpSeg* seg, uint8_t* rxBuf, uint16_t len)
             }
 
             if (seg->FLG & ARDP_FLAG_ACK) {
-                AJ_InfoPrintf(("ArdpMachine(): OPEN: Got ACK %u LCS %u\n", seg->ACK, seg->LCS));
+                AJ_InfoPrintf(("ArdpMachine(): Got ACK %u LCS %u ACKNXT %u\n", seg->ACK, seg->LCS, seg->ACKNXT));
 
                 if (IN_RANGE(uint32_t, conn->snd.UNA, ((conn->snd.NXT - conn->snd.UNA) + 1), seg->ACK) == TRUE) {
                     conn->snd.UNA = seg->ACK + 1;
@@ -816,11 +873,15 @@ static AJ_Status ArdpMachine(struct ArdpSeg* seg, uint8_t* rxBuf, uint16_t len)
                         return AJ_ERR_ARDP_DISCONNECTED;
                     }
                 }
+                conn->confirm = TRUE;
             }
 
+            AJ_InfoPrintf(("ArdpMachine(): OPEN: seq = %u, we are waiting for %u, they wait for %u\n",
+                           seg->SEQ, conn->rcv.CUR + 1, seg->ACKNXT));
             if (SEQ32_LT(conn->rcv.CUR + 1, seg->ACKNXT)) {
-                AJ_InfoPrintf(("ArdpMachine(): OPEN: FlushExpiredRcvMessages: seq = %u, expected %u got %u\n",
+                AJ_InfoPrintf(("ArdpMachine(): OPEN: seq = %u, expected %u got %u\n",
                                seg->SEQ, conn->rcv.CUR + 1, seg->ACKNXT));
+                FlushExpiredRcvMessages(seg->SEQ, seg->ACKNXT);
                 status = AJ_ERR_ARDP_RECV_EXPIRED;
             }
 
@@ -871,7 +932,7 @@ static AJ_Status ARDP_Recv(uint8_t* rxBuf, uint16_t len)
         status = ArdpMachine(&seg, rxBuf, len);
     }
 
-    if (status != AJ_OK) {
+    if (status != AJ_OK && status != AJ_ERR_ARDP_RECV_EXPIRED) {
         AJ_ErrPrintf(("ARDP_Recv(): returned %s (0x%x)\n", AJ_StatusText(status), status));
     }
     return status;
@@ -881,14 +942,7 @@ static void RecvReady(void* rxContext)
 {
     ArdpRBuf* rBuf = (ArdpRBuf*) rxContext;
 
-    AJ_InfoPrintf(("RecvReady: buf=%p, seq=%u\n", rBuf, rBuf->seq));
-
-    if (rBuf->fcnt != 0) {
-        AJ_ASSERT((conn->rcv.LCS + 1) == rBuf->seq);
-    } else {
-        /* This is the very first call into this function. Connection is being established */
-        AJ_ASSERT(conn->rcv.LCS == rBuf->seq);
-    }
+    AJ_InfoPrintf(("RecvReady: buf=%p, seq=%u, lcs=%u\n", rBuf, rBuf->seq, conn->rcv.LCS));
 
     rBuf->fcnt = 0;
     rBuf->dataLen = 0;
@@ -907,9 +961,13 @@ AJ_Status AJ_ARDP_StartMsgSend(uint32_t ttl)
 
     AJ_InfoPrintf(("ARDP_StartMsgSend: ttl=%d\n", ttl));
 
-    AJ_ASSERT(conn->snd.newMsg == FALSE);
+    if (conn->snd.msgLenSent != 0) {
+        AJ_ASSERT(conn->snd.newMsg == FALSE);
+    }
     conn->snd.newMsg = TRUE;
     conn->snd.msgTTL = ttl;
+    conn->snd.msgLenSent = 0;
+
     return AJ_OK;
 }
 
@@ -960,7 +1018,6 @@ static AJ_Status ARDP_Send(uint8_t* txBuf, uint16_t len)
     if (conn->snd.newMsg == TRUE) {
         AJ_MsgHeader* hdr = (AJ_MsgHeader*) txBuf;
         conn->snd.msgLenTotal = sizeof(AJ_MsgHeader) + ((hdr->headerLen + 7) & 0xFFFFFFF8) + hdr->bodyLen;
-        conn->snd.msgLenSent = 0;
         AJ_InfoPrintf(("ARDP_Send: new message len = %u\n", conn->snd.msgLenTotal));
         conn->snd.newMsg = FALSE;
         conn->snd.msgSOM = conn->snd.NXT;
@@ -997,7 +1054,6 @@ static AJ_Status ARDP_Send(uint8_t* txBuf, uint16_t len)
         uint16_t dataLen;
         size_t sent;
         uint32_t timeout;
-        AJ_Status status;
 
         /* Check whether the current buffer is not in a process of being populated with fragmented data */
         dataLen = (len <= (ARDP_MAX_DLEN - offset)) ? offset + len : ARDP_MAX_DLEN;
@@ -1010,7 +1066,7 @@ static AJ_Status ARDP_Send(uint8_t* txBuf, uint16_t len)
         /*
          * Check if the segment is not filled to full capacity and is not the last segment. If so, we need
          * to wait for more data to pack in. Do not send, do not update counters.
-         * Note: Soft check (instead of checking len == 0): a precaution for avoiding infinte loop in case we miscalculated.
+         * Note: Soft check (instead of checking len == 0): a precaution for avoiding infinite loop in case we miscalculated.
          */
         if ((dataLen < ARDP_MAX_DLEN) && dataLen != (conn->snd.msgLenTotal - conn->snd.msgLenSent)) {
             AJ_InfoPrintf(("ARDP_Send(): queued %d bytes\n", dataLen));
@@ -1022,8 +1078,8 @@ static AJ_Status ARDP_Send(uint8_t* txBuf, uint16_t len)
 
         MarshalHeader(sBuf->data, ARDP_FLAG_ACK | ARDP_FLAG_VER, dataLen, ttl, conn->snd.msgSOM, fcnt);
 
-        AJ_InfoPrintf(("ARDP_Send(): send %d bytes (seq %u, ack %u)\n", ARDP_HEADER_SIZE + dataLen, conn->snd.NXT, conn->rcv.CUR));
-        status = (*sendFunction)(conn->context, (uint8_t*) sBuf->data, ARDP_HEADER_SIZE + dataLen, &sent);
+        AJ_InfoPrintf(("ARDP_Send(): send %d bytes (seq %u, ack %u, lcs %u)\n", ARDP_HEADER_SIZE + dataLen, conn->snd.NXT, conn->rcv.CUR, conn->rcv.LCS));
+        status = (*sendFunction)(conn->context, (uint8_t*) sBuf->data, ARDP_HEADER_SIZE + dataLen, &sent, conn->confirm);
 
         if (status != AJ_OK) {
             AJ_ErrPrintf(("ARDP_Send(): %s\n", AJ_StatusText(status)));
@@ -1033,6 +1089,7 @@ static AJ_Status ARDP_Send(uint8_t* txBuf, uint16_t len)
         AJ_InfoPrintf(("ArdpSend(): cancel ackTimer\n"));
         conn->ackTimer.retry = 0;
         conn->rcv.pending = 0;
+        conn->confirm = FALSE;
 
         len -= (dataLen - offset);
 
@@ -1098,6 +1155,13 @@ void AJ_ARDP_Disconnect(uint8_t forced)
         conn->state = CLOSE_WAIT;
         /* Block here  to give data retransmits a chance to go through */
         AJ_ARDP_Recv(&conn->netSock->rx, 0, UDP_DISCONNECT_TIMEOUT);
+        /*
+         * If, while we are waiting, the remote disconnected, the connection is torn down at this point.
+         * Nothing to do, just return.
+         */
+        if (conn == NULL) {
+            return;
+        }
     }
 
     AJ_WarnPrintf(("ARDP_Disconnect: Send RST\n"));
@@ -1110,7 +1174,8 @@ void AJ_ARDP_Disconnect(uint8_t forced)
 AJ_Status AJ_ARDP_Send(AJ_IOBuffer* buf)
 {
     size_t tx = AJ_IO_BUF_AVAIL(buf);
-    AJ_Status status;
+    AJ_Status status = AJ_OK;
+    uint32_t retry = 0;
 
     AJ_InfoPrintf(("AJ_ARDP_Send(buf=0x%p)\n", buf));
 
@@ -1127,11 +1192,14 @@ AJ_Status AJ_ARDP_Send(AJ_IOBuffer* buf)
         if (status == AJ_ERR_ARDP_BACKPRESSURE) {
             do {
                 AJ_InfoPrintf(("AJ_ARDP_Send: dealing with backpressure\n"));
+                if (++retry >= ARDP_MAX_BACKPRESSURE_RETRIES) {
+                    return AJ_ERR_WRITE;
+                }
                 /*
                  * If we can't make room in the send window within a certain amount of time,
                  * assume that the connection has failed.
                  */
-                status = AJ_ARDP_Recv(&conn->netSock->rx, 0, UDP_BACKPRESSURE_TIMEOUT);
+                status = AJ_ARDP_Recv(&conn->netSock->rx, 0, ARDP_BACKPRESSURE_INTERVAL);
 
                 if (status != AJ_OK && status != AJ_ERR_TIMEOUT) {
                     /* Something has gone wrong */
@@ -1140,6 +1208,7 @@ AJ_Status AJ_ARDP_Send(AJ_IOBuffer* buf)
                 }
 
                 status = ARDP_Send(buf->readPtr, tx);
+
                 /* Loop while backpressure continues */
             } while (status == AJ_ERR_ARDP_BACKPRESSURE);
         } else if (status != AJ_OK) {
@@ -1164,7 +1233,7 @@ static void UpdateReadBuffer(AJ_IOBuffer* rxBuf, uint32_t len) {
 
     AJ_InfoPrintf(("UpdateRead: rxBuf %p, len %u\n", rxBuf, len));
 
-    while ((UDP_Recv_State.rxContext != NULL) && len != 0) {
+    while ((UDP_Recv_State.rxContext != NULL) && (UDP_Recv_State.readBuf != NULL) && (len != 0)) {
         ArdpRBuf* rBuf = UDP_Recv_State.rxContext;
         size_t rx = AJ_IO_BUF_SPACE(rxBuf);
         uint32_t consumed;
@@ -1215,7 +1284,7 @@ AJ_Status AJ_ARDP_Recv(AJ_IOBuffer* rxBuf, uint32_t len, uint32_t timeout)
     uint32_t timeout2 = min(timeout, UDP_MINIMUM_TIMEOUT);
     AJ_Time now, end;
 
-    AJ_InfoPrintf(("AJ_ARDP_Recv(rxBuf=0x%p, len=%u, timeout=%u)\n", rxBuf, len, timeout));
+    AJ_InfoPrintf(("AJ_ARDP_Recv(rxBuf=%p, len=%u, timeout=%u)\n", rxBuf, len, timeout));
 
     if (conn == NULL) {
         return AJ_ERR_READ;
@@ -1234,10 +1303,22 @@ AJ_Status AJ_ARDP_Recv(AJ_IOBuffer* rxBuf, uint32_t len, uint32_t timeout)
 
         status = (*recvFunction)(rxBuf->context, &buf, &received, timeout2);
 
+        localStatus = CheckTimers();
+
+        if (localStatus == AJ_ERR_CONNECT) {
+            return AJ_ERR_CONNECT;
+        } else if (localStatus != AJ_OK) {
+            AJ_InfoPrintf(("AJ_ARDP_Recv CheckTimers status %s\n", AJ_StatusText(localStatus)));
+            return AJ_ERR_READ;
+        }
+
         switch (status) {
         case AJ_ERR_TIMEOUT:
+            AJ_InfoPrintf(("AJ_ARDP_Recv status %s, len = %u, rxContext = %p\n", AJ_StatusText(status),
+                           len, UDP_Recv_State.rxContext));
             if ((len != 0) && (UDP_Recv_State.rxContext != NULL)) {
                 status = AJ_OK;
+                AJ_InitTimer(&conn->probeTimer.tStart);
                 goto UPDATE_READ;
             }
             break;
@@ -1245,12 +1326,19 @@ AJ_Status AJ_ARDP_Recv(AJ_IOBuffer* rxBuf, uint32_t len, uint32_t timeout)
         case AJ_OK:
             status = ARDP_Recv(buf, received);
 
-            if ((status == AJ_OK) || (status == AJ_ERR_ARDP_RECV_EXPIRED)) {
+            if (status == AJ_OK) {
                 goto UPDATE_READ;
+            } else if (status == AJ_ERR_ARDP_RECV_EXPIRED) {
+                AJ_InfoPrintf(("AJ_ARDP_Recv: Expired message, LCS %u\n", conn->rcv.LCS));
+                conn->rcv.LCS = conn->rcv.CUR;
+                if (conn->ackTimer.retry == 0) {
+                    InitTimer(&conn->ackTimer, ARDP_MIN_DELAYED_ACK_TIMEOUT, 1);
+                }
+                return status;
             } else if (status == AJ_ERR_ARDP_DISCONNECTING) {
                 /* We are waiting for either TX queue to drain or timeout */
                 break;
-            } else if (status != AJ_ERR_ARDP_REMOTE_CONNECTION_RESET) {
+            } else if ((status != AJ_ERR_ARDP_REMOTE_CONNECTION_RESET) && (conn->state != CLOSE_WAIT)) {
                 AJ_WarnPrintf(("AJ_ARDP_Recv: received bad data, disconnecting\n"));
                 AJ_ARDP_Disconnect(TRUE);
             }
@@ -1265,36 +1353,22 @@ AJ_Status AJ_ARDP_Recv(AJ_IOBuffer* rxBuf, uint32_t len, uint32_t timeout)
         default:
             AJ_WarnPrintf(("AJ_ARDP_Recv: Invalid\n"));
             AJ_ASSERT(!"this shouldn't happen!");
-            break;
+            return AJ_ERR_READ;
         }
 
         AJ_InitTimer(&now);
     } while (AJ_CompareTime(now, end) < 0);
 
 UPDATE_READ:
-
-    if (status == AJ_ERR_ARDP_RECV_EXPIRED) {
-        /*
-         * Currently we do not do anything with special expired messages.
-         * Just deliver the accumulated data and inform the upper layer
-         * (via status) that the the previous message may be incomplete.
-         */
-        AJ_WarnPrintf(("AJ_ARDP_Recv: Expired message\n"));
-    }
-
     if ((len != 0) && (UDP_Recv_State.rxContext != NULL)) {
         UpdateReadBuffer(rxBuf, len);
-    }
-
-    localStatus = CheckTimers();
-
-    if (localStatus != AJ_OK) {
-        status = localStatus;
+        // can't possibly time out if data was recved!
+        return AJ_OK;
     }
 
     AJ_InfoPrintf(("AJ_ARDP_Recv exit with %s\n", AJ_StatusText(status)));
 
-    return status;
+    return AJ_ERR_TIMEOUT;
 }
 
 #ifdef __cplusplus
