@@ -34,6 +34,7 @@
 #include <ajtcl/aj_crypto.h>
 #include <ajtcl/aj_msg_priv.h>
 #include <ajtcl/aj_security.h>
+#include <ajtcl/aj_conversationhash.h>
 
 /**
  * Turn on per-module debug printing by setting this variable to non-zero value
@@ -51,10 +52,12 @@ static AJ_Status ComputeMasterSecret(AJ_AuthenticationContext* ctx, uint8_t* pms
     const uint8_t* data[2];
     uint8_t lens[2];
 
-    AJ_InfoPrintf(("ComputeMasterSecret(ctx=%p, pms=%p, len=%d)\n", ctx, pms, len));
+    AJ_ASSERT(len <= UINT8_MAX);
+
+    AJ_InfoPrintf(("ComputeMasterSecret(ctx=%p, pms=%p, len=%u)\n", ctx, pms, (uint32_t)len));
 
     data[0] = pms;
-    lens[0] = len;
+    lens[0] = (uint8_t) len;
     data[1] = (uint8_t*) "master secret";
     lens[1] = 13;
 
@@ -66,14 +69,16 @@ static AJ_Status ComputeVerifier(AJ_AuthenticationContext* ctx, const char* labe
     const uint8_t* data[3];
     uint8_t lens[3];
 
+    AJ_ASSERT(bufferlen <= UINT32_MAX);
+
     data[0] = ctx->mastersecret;
     lens[0] = AJ_MASTER_SECRET_LEN;
     data[1] = (uint8_t*) label;
     lens[1] = (uint8_t) strlen(label);
     data[2] = ctx->digest;
-    lens[2] = sizeof (ctx->digest);
+    lens[2] = (uint8_t) sizeof (ctx->digest);
 
-    return AJ_Crypto_PRF_SHA256(data, lens, ArraySize(data), buffer, bufferlen);
+    return AJ_Crypto_PRF_SHA256(data, lens, ArraySize(data), buffer, (uint32_t) bufferlen);
 }
 
 static AJ_Status ComputePSKVerifier(AJ_AuthenticationContext* ctx, const char* label, uint8_t* buffer, size_t bufferlen)
@@ -149,12 +154,36 @@ static AJ_Status ECDHEMarshalV4(AJ_AuthenticationContext* ctx, AJ_Message* msg)
 static AJ_Status ECDHEMarshal(AJ_AuthenticationContext* ctx, AJ_Message* msg)
 {
     AJ_Status status;
+    AJ_Credential cred = { 0 };
 
     AJ_InfoPrintf(("ECDHEMarshal(ctx=%p, msg=%p)\n", ctx, msg));
 
     // Generate key pair if client
     if (AUTH_CLIENT == ctx->role) {
-        status = AJ_GenerateECCKeyPair(&ctx->kectx.pub, &ctx->kectx.prv);
+
+        if (ctx->suite == AUTH_SUITE_ECDHE_SPEKE) { /* EC-SPEKE keygen depends on the password, use the callback */
+
+            if (ctx->bus->authListenerCallback == NULL) {
+                AJ_InfoPrintf(("Authentication failure: Missing callback for ECDHE_SPEKE\n"));
+                return AJ_ERR_INVALID;
+            }
+
+            status = ctx->bus->authListenerCallback(AUTH_SUITE_ECDHE_SPEKE, AJ_CRED_PASSWORD, &cred);
+            if (status != AJ_OK) {
+                AJ_InfoPrintf(("Authentication failure: callback failed for ECDHE_SPEKE\n"));
+                return status;
+            }
+
+            if (cred.expiration) {
+                ctx->expiration = cred.expiration;
+            }
+
+            status = AJ_GenerateSPEKEKeyPair(cred.data, cred.len, &ctx->kactx.speke.localGUID, ctx->kactx.speke.remoteGUID,
+                                             &ctx->kectx.pub, &ctx->kectx.prv);
+        } else {   /* The other ECDH suites use traditional key generation */
+            status = AJ_GenerateECCKeyPair(&ctx->kectx.pub, &ctx->kectx.prv);
+        }
+
         if (AJ_OK != status) {
             AJ_InfoPrintf(("ECDHEMarshal(ctx=%p, msg=%p): Key generation failed\n", ctx, msg));
             return status;
@@ -331,12 +360,35 @@ static AJ_Status ECDHEUnmarshalV4(AJ_AuthenticationContext* ctx, AJ_Message* msg
 static AJ_Status ECDHEUnmarshal(AJ_AuthenticationContext* ctx, AJ_Message* msg)
 {
     AJ_Status status;
+    AJ_Credential cred = { 0 };
 
     AJ_InfoPrintf(("ECDHEUnmarshal(ctx=%p, msg=%p)\n", ctx, msg));
 
     // Generate key pair if server
     if (AUTH_SERVER == ctx->role) {
-        status = AJ_GenerateECCKeyPair(&ctx->kectx.pub, &ctx->kectx.prv);
+        if (ctx->suite == AUTH_SUITE_ECDHE_SPEKE) { /* EC-SPEKE keygen depends on the password, use the callback */
+
+            if (ctx->bus->authListenerCallback == NULL) {
+                AJ_InfoPrintf(("Authentication failure: Missing callback for ECDHE_SPEKE\n"));
+                return AJ_ERR_INVALID;
+            }
+
+            status = ctx->bus->authListenerCallback(AUTH_SUITE_ECDHE_SPEKE, AJ_CRED_PASSWORD, &cred);
+            if (status != AJ_OK) {
+                AJ_InfoPrintf(("Authentication failure: callback failed for ECDHE_SPEKE\n"));
+                return status;
+            }
+
+            if (cred.expiration) {
+                ctx->expiration = cred.expiration;
+            }
+
+            status = AJ_GenerateSPEKEKeyPair(cred.data, cred.len, ctx->kactx.speke.remoteGUID, &ctx->kactx.speke.localGUID,
+                                             &ctx->kectx.pub, &ctx->kectx.prv);
+        } else {   /* The other ECDH suites use traditional key generation */
+            status = AJ_GenerateECCKeyPair(&ctx->kectx.pub, &ctx->kectx.prv);
+        }
+
         if (AJ_OK != status) {
             AJ_InfoPrintf(("ECDHEUnmarshal(ctx=%p, msg=%p): Key generation failed\n", ctx, msg));
             return status;
@@ -511,7 +563,12 @@ static AJ_Status PSKCallbackV1(AJ_AuthenticationContext* ctx, AJ_Message* msg)
     }
     ctx->expiration = 0xFFFFFFFF;
     AJ_ConversationHash_Update_UInt8Array(ctx, CONVERSATION_V1, ctx->kactx.psk.hint, ctx->kactx.psk.hintSize);
+    /* Calling AJ_ConversationHash_SetSensitiveMode ensures the PSK won't end up in the log if conversation
+     * hash tracing is turned on.
+     */
+    AJ_ConversationHash_SetSensitiveMode(ctx, TRUE);
     AJ_ConversationHash_Update_UInt8Array(ctx, CONVERSATION_V1, ctx->kactx.psk.key, ctx->kactx.psk.keySize);
+    AJ_ConversationHash_SetSensitiveMode(ctx, FALSE);
     if (AJ_UNPACK_AUTH_VERSION(ctx->version) < CONVERSATION_V4) {
         status = AJ_ConversationHash_GetDigest(ctx);
     }
@@ -551,7 +608,12 @@ static AJ_Status PSKCallbackV2(AJ_AuthenticationContext* ctx, AJ_Message* msg)
     ctx->expiration = cred.expiration;
     // Hash in psk hint, then psk
     AJ_ConversationHash_Update_UInt8Array(ctx, CONVERSATION_V1, ctx->kactx.psk.hint, ctx->kactx.psk.hintSize);
+    /* Calling AJ_ConversationHash_SetSensitiveMode ensures the PSK won't end up in the log if conversation
+     * hash tracing is turned on.
+     */
+    AJ_ConversationHash_SetSensitiveMode(ctx, TRUE);
     AJ_ConversationHash_Update_UInt8Array(ctx, CONVERSATION_V1, cred.data, cred.len);
+    AJ_ConversationHash_SetSensitiveMode(ctx, FALSE);
     if (AJ_UNPACK_AUTH_VERSION(ctx->version) < CONVERSATION_V4) {
         status = AJ_ConversationHash_GetDigest(ctx);
         if (AJ_OK != status) {
@@ -831,6 +893,7 @@ static AJ_Status ECDSAUnmarshal(AJ_AuthenticationContext* ctx, AJ_Message* msg,
     AJ_Arg container1;
     AJ_Arg container2;
     uint8_t digest[AJ_SHA256_DIGEST_LENGTH];
+    AJ_SHA256_Context* thumbprintHashCtx;
     const char* variant;
     uint8_t fmt;
     DER_Element der;
@@ -942,7 +1005,7 @@ static AJ_Status ECDSAUnmarshal(AJ_AuthenticationContext* ctx, AJ_Message* msg,
 
         /*
          * If this is the first certificate, check that it signed the verifier
-         * Also save the subject public key and manifest digest for authorisation check
+         * Also save the subject public key and thumbprint for authorisation check
          */
         if (NULL == node->next) {
             status = AJ_ECDSAVerifyDigest(digest, &sig, &node->certificate.tbs.publickey);
@@ -950,11 +1013,18 @@ static AJ_Status ECDSAUnmarshal(AJ_AuthenticationContext* ctx, AJ_Message* msg,
                 AJ_InfoPrintf(("ECDSAUnmarshal(ctx=%p, msg=%p): Signature invalid\n", ctx, msg));
                 goto Exit;
             }
-            if (AJ_SHA256_DIGEST_LENGTH == node->certificate.tbs.extensions.digest.size) {
-                ctx->kactx.ecdsa.size = AJ_SHA256_DIGEST_LENGTH;
-                /* Copy the manifest digest */
-                memcpy(ctx->kactx.ecdsa.manifest, node->certificate.tbs.extensions.digest.data, AJ_SHA256_DIGEST_LENGTH);
+            thumbprintHashCtx = AJ_SHA256_Init();
+            if (!thumbprintHashCtx) {
+                AJ_InfoPrintf(("ECDSAUnmarshal(ctx=%p, msg=%p): Could not allocate SHA256 context\n", ctx, msg));
+                goto Exit;
             }
+            AJ_SHA256_Update(thumbprintHashCtx, node->certificate.der.data, node->certificate.der.size);
+            status = AJ_SHA256_Final(thumbprintHashCtx, ctx->kactx.ecdsa.thumbprint);
+            if (AJ_OK != status) {
+                AJ_InfoPrintf(("ECDSAUnmarshal(ctx=%p, msg=%p): Got status %u from AJ_SHA256_Final\n", ctx, msg, status));
+                goto Exit;
+            }
+            ctx->kactx.ecdsa.thumbprintSize = AJ_SHA256_DIGEST_LENGTH;
         }
         /* Copy the public key */
         ctx->kactx.ecdsa.num++;
@@ -1063,6 +1133,10 @@ AJ_Status AJ_KeyAuthenticationMarshal(AJ_AuthenticationContext* ctx, AJ_Message*
     case AUTH_SUITE_ECDHE_ECDSA:
         status = ECDSAMarshal(ctx, msg);
         break;
+
+    case AUTH_SUITE_ECDHE_SPEKE:        /* Same marshalling as ECDHE_NULL. */
+        status = NULLMarshal(ctx, msg);
+        break;
     }
     return status;
 }
@@ -1082,6 +1156,10 @@ AJ_Status AJ_KeyAuthenticationUnmarshal(AJ_AuthenticationContext* ctx, AJ_Messag
     case AUTH_SUITE_ECDHE_ECDSA:
         status = ECDSAUnmarshal(ctx, msg, ctx->version);
         break;
+
+    case AUTH_SUITE_ECDHE_SPEKE:        /* Same unmarshalling as ECDHE_NULL. */
+        status = NULLUnmarshal(ctx, msg);
+        break;
     }
     return status;
 }
@@ -1101,6 +1179,9 @@ uint8_t AJ_IsSuiteEnabled(AJ_BusAttachment* bus, uint32_t suite, uint32_t versio
         }
         return 1 == bus->suites[2];
 
+    case AUTH_SUITE_ECDHE_SPEKE:
+        return 1 == bus->suites[3];
+
     default:
         return 0;
     }
@@ -1122,130 +1203,10 @@ void AJ_EnableSuite(AJ_BusAttachment* bus, uint32_t suite)
     case AUTH_SUITE_ECDHE_ECDSA:
         bus->suites[2] = 1;
         break;
+
+    case AUTH_SUITE_ECDHE_SPEKE:
+        bus->suites[3] = 1;
+        break;
     }
 }
 
-AJ_Status AJ_ConversationHash_Initialize(AJ_AuthenticationContext* ctx)
-{
-    ctx->hash = AJ_SHA256_Init();
-
-    if (ctx->hash) {
-        return AJ_OK;
-    } else {
-        AJ_ErrPrintf(("AJ_ConversationHash_Initialize() failed\n"));
-        return AJ_ERR_RESOURCES;
-    }
-}
-
-uint8_t AJ_ConversationHash_IsInitialized(AJ_AuthenticationContext* ctx)
-{
-    if (ctx->hash) {
-        return 1;
-    }
-    return 0;
-}
-
-static inline int ConversationVersionDoesNotApply(uint32_t conversationVersion, uint32_t currentAuthVersion)
-{
-    AJ_ASSERT((CONVERSATION_V1 == conversationVersion) || (CONVERSATION_V4 == conversationVersion));
-
-    /* The conversation version itself is computed by taking (currentAuthVersion >> 16). We return true
-     * if the current conversation version does NOT apply to the conversation version for the message being hashed.
-     */
-    if (CONVERSATION_V4 == conversationVersion) {
-        return ((currentAuthVersion >> 16) != CONVERSATION_V4);
-    } else {
-        return ((currentAuthVersion >> 16) >= CONVERSATION_V4);
-    }
-}
-
-void AJ_ConversationHash_Update_UInt8(AJ_AuthenticationContext* ctx, uint32_t conversationVersion, uint8_t byte)
-{
-    if (ConversationVersionDoesNotApply(conversationVersion, ctx->version)) {
-        return;
-    }
-    AJ_SHA256_Update(ctx->hash, &byte, sizeof(byte));
-}
-
-static void ConversationHash_Update_UInt32(AJ_AuthenticationContext* ctx, uint32_t u32)
-{
-    uint8_t u32LE[sizeof(uint32_t)];
-    HostU32ToLittleEndianU8(&u32, sizeof(u32), u32LE);
-    AJ_SHA256_Update(ctx->hash, u32LE, sizeof(u32LE));
-}
-
-void AJ_ConversationHash_Update_UInt8Array(AJ_AuthenticationContext* ctx, uint32_t conversationVersion, const uint8_t* buf, size_t bufSize)
-{
-    if (ConversationVersionDoesNotApply(conversationVersion, ctx->version)) {
-        return;
-    }
-    if (conversationVersion >= CONVERSATION_V4) {
-        AJ_ASSERT(bufSize <= 0xFFFFFFFF);
-        ConversationHash_Update_UInt32(ctx, (uint32_t)bufSize);
-    }
-    AJ_SHA256_Update(ctx->hash, buf, bufSize);
-}
-
-void AJ_ConversationHash_Update_String(AJ_AuthenticationContext* ctx, uint32_t conversationVersion, const char* str, size_t strSize)
-{
-    AJ_ConversationHash_Update_UInt8Array(ctx, conversationVersion, (const uint8_t*)str, strSize);
-}
-
-void AJ_ConversationHash_Update_Message(AJ_AuthenticationContext* ctx, uint32_t conversationVersion, AJ_Message* msg, uint8_t isMarshaledMessage)
-{
-    uint8_t* data;
-    uint32_t size;
-    AJ_MsgHeader hdr;
-
-    if (ConversationVersionDoesNotApply(conversationVersion, ctx->version)) {
-        return;
-    }
-
-    AJ_ASSERT((HASH_MSG_UNMARSHALED == isMarshaledMessage) || (HASH_MSG_MARSHALED == isMarshaledMessage));
-
-    AJ_ASSERT(!(msg->hdr->flags & AJ_FLAG_ENCRYPTED));
-
-    if (HASH_MSG_MARSHALED == isMarshaledMessage) {
-        /* msg->hdr->bodyLen gets set by AJ_DeliverMsg when the message is sent out. We set it here as well
-         * so that the buffer we hash equals what will actually go out on the wire.
-         */
-        AJ_ASSERT(0 == msg->hdr->bodyLen);
-        msg->hdr->bodyLen = msg->bodyBytes;
-        data = msg->bus->sock.tx.bufStart;
-        size = sizeof(AJ_MsgHeader) + msg->hdr->headerLen + HEADERPAD(msg->hdr->headerLen) + msg->hdr->bodyLen;
-        AJ_ConversationHash_Update_UInt8Array(ctx, conversationVersion, data, size);
-    } else {
-        /*
-         * AJ_UnmarshalMsg ensures that the Peer.Authentication interface messages are loaded into the buffer.
-         * The message header may have been endian swapped, so we use the raw header saved during AJ_UnmarshalMsg.
-         */
-        data = msg->bus->sock.rx.bufStart;
-        size = sizeof(AJ_MsgHeader) + msg->hdr->headerLen + HEADERPAD(msg->hdr->headerLen) + msg->hdr->bodyLen;
-        /* Save the current header */
-        memcpy(&hdr, data, sizeof(AJ_MsgHeader));
-        /* Replace with original header */
-        memcpy(data, &msg->raw, sizeof(AJ_MsgHeader));
-        AJ_ConversationHash_Update_UInt8Array(ctx, conversationVersion, data, size);
-        /* Put the header back */
-        memcpy(data, &hdr, sizeof(AJ_MsgHeader));
-    }
-
-}
-
-AJ_Status AJ_ConversationHash_GetDigest(AJ_AuthenticationContext* ctx)
-{
-    return AJ_SHA256_GetDigest(ctx->hash, ctx->digest);
-}
-
-AJ_Status AJ_ConversationHash_Reset(AJ_AuthenticationContext* ctx)
-{
-    AJ_Status status;
-
-    status = AJ_SHA256_Final(ctx->hash, NULL);
-
-    if (status == AJ_OK) {
-        status = AJ_ConversationHash_Initialize(ctx);
-    }
-
-    return status;
-}
