@@ -508,7 +508,7 @@ AJ_Status AJ_BusHandleBusMessage(AJ_Message* msg)
 
     case AJ_METHOD_INTROSPECT:
         AJ_InfoPrintf(("AJ_BusHandleBusMessage(): AJ_METHOD_INTROSPECT\n"));
-        status = AJ_HandleIntrospectRequest(msg, &reply, NULL);
+        status = AJ_GetIntrospectionData(msg, &reply);
         break;
 
     case AJ_METHOD_GET_DESCRIPTION_LANG:
@@ -552,9 +552,9 @@ AJ_Status AJ_BusHandleBusMessage(AJ_Message* msg)
         status = AJ_PeerHandleKeyAuthentication(msg, &reply);
         break;
 
-    case AJ_METHOD_SEND_MANIFEST:
-        AJ_InfoPrintf(("AJ_BusHandleBusMessage(): AJ_METHOD_SEND_MANIFEST\n"));
-        status = AJ_PeerHandleSendManifest(msg, &reply);
+    case AJ_METHOD_SEND_MANIFESTS:
+        AJ_InfoPrintf(("AJ_BusHandleBusMessage(): AJ_METHOD_SEND_MANIFESTS\n"));
+        status = AJ_PeerHandleSendManifests(msg, &reply);
         break;
 
     case AJ_METHOD_SEND_MEMBERSHIPS:
@@ -592,9 +592,9 @@ AJ_Status AJ_BusHandleBusMessage(AJ_Message* msg)
         status = AJ_PeerHandleKeyAuthenticationReply(msg);
         break;
 
-    case AJ_REPLY_ID(AJ_METHOD_SEND_MANIFEST):
-        AJ_InfoPrintf(("AJ_BusHandleBusMessage(): AJ_REPLY_ID(AJ_METHOD_SEND_MANIFEST)\n"));
-        status = AJ_PeerHandleSendManifestReply(msg);
+    case AJ_REPLY_ID(AJ_METHOD_SEND_MANIFESTS):
+        AJ_InfoPrintf(("AJ_BusHandleBusMessage(): AJ_REPLY_ID(AJ_METHOD_SEND_MANIFESTS)\n"));
+        status = AJ_PeerHandleSendManifestsReply(msg);
         break;
 
     case AJ_REPLY_ID(AJ_METHOD_SEND_MEMBERSHIPS):
@@ -717,6 +717,18 @@ AJ_Status AJ_BusHandleBusMessage(AJ_Message* msg)
         status = AJ_SecurityRemoveMembershipMethod(msg, &reply);
         break;
 
+    case AJ_METHOD_MANAGED_START_MANAGEMENT:
+        status = AJ_SecurityStartManagementMethod(msg, &reply);
+        break;
+
+    case AJ_METHOD_MANAGED_END_MANAGEMENT:
+        status = AJ_SecurityEndManagementMethod(msg, &reply);
+        break;
+
+    case AJ_METHOD_MANAGED_INSTALL_MANIFESTS:
+        status = AJ_SecurityInstallManifestsMethod(msg, &reply);
+        break;
+
     default:
         AJ_InfoPrintf(("AJ_BusHandleBusMessage(): default\n"));
         if (msg->hdr->msgType == AJ_MSG_METHOD_CALL) {
@@ -1188,7 +1200,6 @@ AJ_Status AJ_BusHandleJoinSessionReply(AJ_Message* msg)
     uint32_t resultCode;
     uint32_t sessionId;
     AJ_SessionOpts opts = { 0 };
-    AJ_Arg arr;
     AJ_Status status;
     AJ_Session* session;
 
