@@ -107,7 +107,7 @@ void SetBusAuthPwdCallback(BusAuthPwdFunc callback)
      */
 }
 
-#ifdef AJ_TCP
+#if defined(AJ_TCP) || defined(AJ_SERIAL_CONNECTION)
 static AJ_Status SendHello(AJ_BusAttachment* bus)
 {
     AJ_Status status;
@@ -281,7 +281,7 @@ AJ_Status AJ_Authenticate(AJ_BusAttachment* bus)
         return AJ_OK;
     }
 
-#ifdef AJ_TCP
+#if defined(AJ_TCP) || defined(AJ_SERIAL_CONNECTION)
     /*
      * Send initial NUL byte
      */
@@ -336,128 +336,6 @@ ExitConnect:
 
 #define AJ_DHCP_TIMEOUT  5000
 
-// TODO: deprecate this function; replace it with AJ_FindBusAndConnect
-AJ_Status AJ_Connect(AJ_BusAttachment* bus, const char* serviceName, uint32_t timeout)
-{
-    AJ_Status status;
-    AJ_Service service;
-    bus->isAuthenticated = FALSE;
-
-#ifdef AJ_SERIAL_CONNECTION
-    AJ_Time start, now;
-    AJ_InitTimer(&start);
-#endif
-
-    AJ_InfoPrintf(("AJ_Connect(bus=0x%p, serviceName=\"%s\", timeout=%d, selectionTimeout=%d.)\n", bus, serviceName, timeout, selectionTimeout));
-
-    /*
-     * Clear the bus struct
-     */
-    memset(bus, 0, sizeof(AJ_BusAttachment));
-    /*
-     * Clear stale name->GUID mappings
-     */
-    AJ_GUID_ClearNameMap();
-
-#if !(defined(ARDUINO) || defined(__linux) || defined(_WIN32) || defined(__MACH__))
-    /*
-     * Get an IP address.  We don't want to break this older version
-     * of AJ_Connect, so acquire an IP if we don't already have one.
-     *
-     * This does not work on non-embedded platforms!
-     */
-    {
-        uint32_t ip, mask, gw;
-        status = AJ_AcquireIPAddress(&ip, &mask, &gw, AJ_DHCP_TIMEOUT);
-
-        if (status != AJ_OK) {
-            AJ_ErrPrintf(("AJ_Net_Up(): AJ_AcquireIPAddress Failed\n"));
-        }
-    }
-#endif
-
-    /*
-     * Discover a daemon or service to connect to
-     */
-    if (!serviceName) {
-        serviceName = daemonService;
-    }
-#if AJ_CONNECT_LOCALHOST
-    service.ipv4port = 9955;
-#if HOST_IS_LITTLE_ENDIAN
-    service.ipv4 = 0x0100007F; // 127.0.0.1
-#endif
-#if HOST_IS_BIG_ENDIAN
-    service.ipv4 = 0x7f000001; // 127.0.0.1
-#endif
-    service.addrTypes = AJ_ADDR_TCP4;
-#elif defined(ARDUINO)
-    service.ipv4port = 9955;
-    service.ipv4 = 0x6501A8C0; // 192.168.1.101
-    service.addrTypes = AJ_ADDR_TCP4;
-    status = AJ_Discover(serviceName, &service, timeout, selectionTimeout);
-    if (status != AJ_OK) {
-        AJ_InfoPrintf(("AJ_Connect(): AJ_Discover status=%s\n", AJ_StatusText(status)));
-        goto ExitConnect;
-    }
-#elif defined(AJ_SERIAL_CONNECTION)
-    // don't bother with discovery, we are connected to a daemon.
-    // however, take this opportunity to bring up the serial connection
-    // in a way that depends on the target
-    status = AJ_Serial_Up();
-    if (status != AJ_OK) {
-        AJ_InfoPrintf(("AJ_Connect(): AJ_Serial_Up status=%s\n", AJ_StatusText(status)));
-    }
-#else
-    status = AJ_Discover(serviceName, &service, timeout, selectionTimeout);
-    if (status != AJ_OK) {
-        AJ_InfoPrintf(("AJ_Connect(): AJ_Discover status=%s\n", AJ_StatusText(status)));
-        goto ExitConnect;
-    }
-#endif
-    status = AJ_Net_Connect(bus, &service);
-    if (status != AJ_OK) {
-        AJ_InfoPrintf(("AJ_Connect(): AJ_Net_Connect status=%s\n", AJ_StatusText(status)));
-        goto ExitConnect;
-    }
-
-#ifdef AJ_SERIAL_CONNECTION
-    // run the state machine for long enough to (hopefully) do the SLAP handshake
-    do {
-        AJ_StateMachine();
-        AJ_InitTimer(&now);
-    } while (AJ_SerialLinkParams.linkState != AJ_LINK_ACTIVE && AJ_GetTimeDifference(&now, &start) < timeout);
-
-    if (AJ_SerialLinkParams.linkState != AJ_LINK_ACTIVE) {
-        AJ_InfoPrintf(("Failed to establish active SLAP connection in %u msec\n", timeout));
-        AJ_SerialShutdown();
-        return AJ_ERR_TIMEOUT;
-    }
-#endif
-
-    status = AJ_Authenticate(bus);
-    if (status != AJ_OK) {
-        AJ_InfoPrintf(("AJ_Connect(): AJ_Authenticate status=%s\n", AJ_StatusText(status)));
-        goto ExitConnect;
-    }
-
-    status = SetSignalRules(bus);
-    if (status != AJ_OK) {
-        AJ_InfoPrintf(("AJ_Connect(): SetSignalRules status=%s\n", AJ_StatusText(status)));
-        goto ExitConnect;
-    }
-
-
-ExitConnect:
-
-    if (status != AJ_OK) {
-        AJ_InfoPrintf(("AJ_Connect(): status=%s\n", AJ_StatusText(status)));
-        AJ_Disconnect(bus);
-    }
-    AJ_InitRoutingNodeResponselist();
-    return status;
-}
-
 static void AddRoutingNodeToBlacklist(const AJ_Service* service, uint8_t addrTypes);
 
 AJ_Status AJ_FindBusAndConnect(AJ_BusAttachment* bus, const char* serviceName, uint32_t timeout)
@@ -725,6 +603,11 @@ void AJ_Disconnect(AJ_BusAttachment* bus)
      */
     AJ_ClearAuthContext();
 
+    /*
+     * Clear sent manifests flag
+     */
+    AJ_ClearSentManifests();
+
     /*
      * Set the routing nodes proto version to zero (not connected)
      */
