@@ -212,6 +212,8 @@ const char* AJ_StatusText(AJ_Status status)
         AJ_CASE(AJ_ERR_SECURITY_DUPLICATE_CERTIFICATE);
         AJ_CASE(AJ_ERR_SECURITY_POLICY_NOT_NEWER);
         AJ_CASE(AJ_ERR_SECURITY_CERTIFICATE_NOT_FOUND);
+        AJ_CASE(AJ_ERR_MANAGEMENT_ALREADY_STARTED);
+        AJ_CASE(AJ_ERR_MANAGEMENT_NOT_STARTED);
 
     default:
         return "<unknown>";
