@@ -23,16 +23,16 @@
  */
 #define AJ_MODULE DISCO
 
-#include "aj_target.h"
-#include "aj_status.h"
-#include "aj_util.h"
-#include "aj_net.h"
-#include "aj_disco.h"
-#include "aj_debug.h"
-#include "aj_config.h"
-#include "aj_connect.h"
-#include "aj_guid.h"
-#include "aj_creds.h"
+#include <ajtcl/aj_target.h>
+#include <ajtcl/aj_status.h>
+#include <ajtcl/aj_util.h>
+#include <ajtcl/aj_net.h>
+#include <ajtcl/aj_disco.h>
+#include <ajtcl/aj_debug.h>
+#include <ajtcl/aj_config.h>
+#include <ajtcl/aj_connect.h>
+#include <ajtcl/aj_guid.h>
+#include <ajtcl/aj_creds.h>
 
 /**
  * Turn on per-module debug printing by setting this variable to non-zero value
@@ -949,24 +949,34 @@ static AJ_Status ParseMDNSResp(AJ_IOBuffer* rxBuf, const char* prefix, AJ_Servic
     // compatibility with other record types that may be in use in the future.
     if ((alljoyn_ptr_record_tcp || alljoyn_ptr_record_udp) && (service_port_tcp || service_port_udp)
         && bus_transport && bus_protocol && bus_a_record) {
+        // ignore this record if it's TCP-only but we want ARDP-only, or vice-versa
+        AJ_Status status = AJ_ERR_NO_MATCH;
 
+        // Check for a TCP response, but only if we're looking for TCP responses
+#ifdef AJ_TCP
         if (alljoyn_ptr_record_tcp && service_port_tcp) {
             service->ipv4port = service_port_tcp;
             memcpy(&service->ipv4, bus_addr, sizeof(service->ipv4));
             service->addrTypes |= AJ_ADDR_TCP4;
             service->pv = protocol_version;
             service->priority = service_priority;
+            status = AJ_OK;
         }
+#endif
 
+        // similarly, check ARDP only if we care about it.
+#ifdef AJ_ARDP
         if (alljoyn_ptr_record_udp && service_port_udp) {
             service->ipv4portUdp = service_port_udp;
             memcpy(&service->ipv4Udp, bus_addr, sizeof(service->ipv4Udp));
             service->addrTypes |= AJ_ADDR_UDP4;
             service->pv = protocol_version;
             service->priority = service_priority;
+            status = AJ_OK;
         }
+#endif
 
-        return AJ_OK;
+        return status;
     } else {
         return AJ_ERR_NO_MATCH;
     }
@@ -986,7 +996,7 @@ AJ_Status AJ_Discover(const char* prefix, AJ_Service* service, uint32_t timeout,
     uint32_t queries = 0;
     int32_t discover = (int32_t) timeout;
     int32_t selection = (int32_t) selectionTimeout;
-    int32_t listen;
+    int32_t listenInt;
     AJ_Time discoverTimer;
     AJ_Time listenTimer;
     AJ_Time selectionTimer;
@@ -1076,32 +1086,32 @@ AJ_Status AJ_Discover(const char* prefix, AJ_Service* service, uint32_t timeout,
             } else if (queries >= 12) {
                 interval = 40000;
             }
-            listen = interval + AJ_BURST_INTERVAL;
+            listenInt = interval + AJ_BURST_INTERVAL;
         } else {
-            listen = AJ_BURST_INTERVAL;
+            listenInt = AJ_BURST_INTERVAL;
         }
 
         /*
          * If selection period has not passed do not listen longer than the selection timeout
          */
-        if ((selection > 0) && (listen > selection)) {
-            listen = selection;
+        if ((selection > 0) && (listenInt > selection)) {
+            listenInt = selection;
         }
 
         /*
          * Do not listen longer than the overall discover timeout
          */
-        if (listen > discover) {
-            listen = discover;
+        if (listenInt > discover) {
+            listenInt = discover;
         }
 
         /*
          * recv for the listen period
          */
         AJ_InitTimer(&listenTimer);
-        while (listen > 0) {
+        while (listenInt > 0) {
             AJ_IO_BUF_RESET(&sock.rx);
-            status = sock.rx.recv(&sock.rx, AJ_IO_BUF_SPACE(&sock.rx), listen);
+            status = sock.rx.recv(&sock.rx, AJ_IO_BUF_SPACE(&sock.rx), listenInt);
             if (status != AJ_OK) {
                 /*
                  * Anything other than AJ_ERR_TIMEOUT means bail
@@ -1139,7 +1149,7 @@ AJ_Status AJ_Discover(const char* prefix, AJ_Service* service, uint32_t timeout,
                     }
                 }
             }
-            listen -= AJ_GetElapsedTime(&listenTimer, FALSE);
+            listenInt -= AJ_GetElapsedTime(&listenTimer, FALSE);
         }
         selection -= AJ_GetElapsedTime(&selectionTimer, FALSE);
         if (selection < 0 && AJ_GetRoutingNodeResponseListSize() > 0) {
