@@ -25,24 +25,26 @@
 
 #include <stdarg.h>
 
-#include "aj_target.h"
-#include "aj_status.h"
-#include "aj_msg.h"
-#include "aj_msg_priv.h"
-#include "aj_bufio.h"
-#include "aj_connect.h"
-#include "aj_guid.h"
-#include "aj_peer.h"
-#include "aj_util.h"
-#include "aj_crypto.h"
-#include "aj_introspect.h"
-#include "aj_std.h"
-#include "aj_bus.h"
-#include "aj_debug.h"
-#include "aj_config.h"
+#include <ajtcl/aj_target.h>
+#include <ajtcl/aj_status.h>
+#include <ajtcl/aj_msg.h>
+#include <ajtcl/aj_msg_priv.h>
+#include <ajtcl/aj_bufio.h>
+#include <ajtcl/aj_connect.h>
+#include <ajtcl/aj_guid.h>
+#include <ajtcl/aj_peer.h>
+#include <ajtcl/aj_util.h>
+#include <ajtcl/aj_crypto.h>
+#include <ajtcl/aj_introspect.h>
+#include <ajtcl/aj_std.h>
+#include <ajtcl/aj_bus.h>
+#include <ajtcl/aj_bus_priv.h>
+#include <ajtcl/aj_debug.h>
+#include <ajtcl/aj_config.h>
+#include <ajtcl/aj_authorisation.h>
 
 #ifdef AJ_ARDP
-#include "aj_ardp.h"
+#include <ajtcl/aj_ardp.h>
 #endif
 
 /**
@@ -476,8 +478,7 @@ static AJ_Status DecryptMessage(AJ_Message* msg)
         }
     }
     if (status != AJ_OK) {
-        AJ_ErrPrintf(("DecryptMessage(): AJ_ERR_SECURITY\n"));
-        status = AJ_ERR_SECURITY;
+        AJ_ErrPrintf(("DecryptMessage(): AJ_ERR_NO_MATCH\n"));
     } else {
         macLen = GetMACLength(msg);
         nonceLen = GetNonceLength(msg);
@@ -518,42 +519,64 @@ static AJ_Status EncryptMessage(AJ_Message* msg)
      */
     if ((msg->hdr->msgType == AJ_MSG_SIGNAL) && !msg->destination) {
         status = AJ_GetGroupKey(NULL, key);
-        msg->authVersion = MIN_AUTH_FALLBACK_VERSION;
+        if (AJ_OK == status) {
+            msg->authVersion = MIN_AUTH_FALLBACK_VERSION;
+        }
     } else {
         status = AJ_GetSessionKey(msg->destination, key, &role, &msg->authVersion);
     }
-    macLen = GetMACLength(msg);
-    nonceLen = GetNonceLength(msg);
-    extraNonceLen = nonceLen - PREVIOUS_NONCE_LENGTH;
-    cryptoValsLen = macLen + extraNonceLen;
 
-    /*
-     * Check there is room to append the MAC and Nonce
-     */
-    if (AJ_IO_BUF_SPACE(ioBuf) < cryptoValsLen) {
-        AJ_ErrPrintf(("EncryptMessage(): AJ_ERR_RESOURCES\n"));
-        return AJ_ERR_RESOURCES;
-    }
-    msg->hdr->bodyLen += cryptoValsLen;
-    ioBuf->writePtr += cryptoValsLen;
+    if (AJ_OK == status) {
+        macLen = GetMACLength(msg);
+        nonceLen = GetNonceLength(msg);
+        extraNonceLen = nonceLen - PREVIOUS_NONCE_LENGTH;
+        cryptoValsLen = macLen + extraNonceLen;
 
+        /*
+         * Check there is room to append the MAC and Nonce
+         */
+        if (AJ_IO_BUF_SPACE(ioBuf) < cryptoValsLen) {
+            AJ_ErrPrintf(("EncryptMessage(): AJ_ERR_RESOURCES\n"));
+            AJ_MemZeroSecure(key, 16);
+            return AJ_ERR_RESOURCES;
+        }
+        msg->hdr->bodyLen += cryptoValsLen;
+        ioBuf->writePtr += cryptoValsLen;
 
-    if (status != AJ_OK) {
-        AJ_ErrPrintf(("EncryptMesssage(): peer %s not authenticated", msg->destination));
-        AJ_ErrPrintf(("EncryptMessage(): AJ_ERR_SECURITY\n"));
-        status = AJ_ERR_SECURITY;
-    } else {
         if (MessageRequiresLongerCryptoValues(msg, MIN_AUTH_FULL_NONCE_LENGTH)) {
             AJ_RandBytes(ioBuf->bufStart + mlen + macLen, extraNonceLen);
         }
         AJ_InfoPrintf(("EncryptMessage(): "));
         InitNonce(msg, role, nonce, sizeof(nonce), ioBuf->bufStart + mlen + macLen, extraNonceLen);
         status = AJ_Encrypt_CCM(key, ioBuf->bufStart, mlen, hlen, macLen, nonce, nonceLen);
+    } else {
+        AJ_ErrPrintf(("EncryptMesssage(): peer %s not authenticated", msg->destination));
+        /* Leave status from AJ_GetGroupKey/AJ_GetStatusKey unmodified.
+         * Caller checks for AJ_ERR_NO_MATCH.
+         */
     }
     AJ_MemZeroSecure(key, 16);
     return status;
 }
 
+static AJ_Status AuthoriseOutgoingMessage(const AJ_Message* msg)
+{
+    if ((msg->hdr->msgType != AJ_MSG_METHOD_CALL) && (msg->hdr->msgType != AJ_MSG_SIGNAL)) {
+        return AJ_OK;
+    }
+
+    return AJ_AccessControlCheckMessage(msg, msg->destination, AJ_ACCESS_OUTGOING);
+}
+
+static AJ_Status AuthoriseIncomingMessage(const AJ_Message* msg)
+{
+    if ((msg->hdr->msgType != AJ_MSG_METHOD_CALL) && (msg->hdr->msgType != AJ_MSG_SIGNAL)) {
+        return AJ_OK;
+    }
+
+    return AJ_AccessControlCheckMessage(msg, msg->sender, AJ_ACCESS_INCOMING);
+}
+
 AJ_Status AJ_DeliverMsg(AJ_Message* msg)
 {
     AJ_Status status = AJ_OK;
@@ -575,7 +598,15 @@ AJ_Status AJ_DeliverMsg(AJ_Message* msg)
         msg->hdr->bodyLen = msg->bodyBytes;
         AJ_DumpMsg("SENDING", msg, TRUE);
         if (msg->hdr->flags & AJ_FLAG_ENCRYPTED) {
-            status = EncryptMessage(msg);
+            status = AuthoriseOutgoingMessage(msg);
+            if (AJ_OK == status) {
+                status = EncryptMessage(msg);
+            }
+
+            if (AJ_ERR_NO_MATCH == status && AJ_MSG_ERROR == msg->hdr->msgType && msg->error == AJ_ErrSecurityViolation) {
+                /* Allow an unencrypted reply in this specific key-not-found error case. */
+                status = AJ_OK;
+            }
         }
     } else {
         /*
@@ -597,10 +628,11 @@ AJ_Status AJ_DeliverMsg(AJ_Message* msg)
 /*
  * Make sure we have the required number of bytes in the I/O buffer
  */
-static AJ_Status LoadBytes(AJ_IOBuffer* ioBuf, uint16_t numBytes, uint8_t pad, uint32_t* timeout)
+static AJ_Status LoadBytes(AJ_IOBuffer* ioBuf, uint16_t numBytes, uint8_t pad, AJ_Message* msg)
 {
     AJ_Status status = AJ_OK;
     AJ_Time msgTimer;
+    uint32_t* timeout = &msg->timeout;
 
     AJ_InitTimer(&msgTimer);
     numBytes += pad;
@@ -639,6 +671,10 @@ static AJ_Status LoadBytes(AJ_IOBuffer* ioBuf, uint16_t numBytes, uint8_t pad, u
                 }
                 AJ_ErrPrintf(("LoadBytes(): AJ_ERR_READ\n"));
                 status = AJ_ERR_READ;
+            } else if (status == AJ_ERR_ARDP_RECV_EXPIRED) {
+                AJ_ErrPrintf(("LoadBytes(): AJ_ERR_ARDP_RECV_EXPIRED -> AJ_ERR_UNMARSHAL\n"));
+                msg->expired  = TRUE;
+                status = AJ_ERR_UNMARSHAL;
             }
             break;
         }
@@ -714,23 +750,28 @@ AJ_Status AJ_CloseMsg(AJ_Message* msg)
      */
     if (msg->bus) {
         AJ_IOBuffer* ioBuf = &msg->bus->sock.rx;
-        /*
-         * Skip any unconsumed bytes
-         */
-        while (msg->bodyBytes) {
-            uint16_t sz = AJ_IO_BUF_AVAIL(ioBuf);
-            sz = min(sz, msg->bodyBytes);
-            if (!sz) {
-                AJ_IO_BUF_RESET(ioBuf);
-                sz = min(msg->bodyBytes, ioBuf->bufSize);
+        if (msg->expired == TRUE) {
+            AJ_IO_BUF_RESET(ioBuf);
+        } else {
+            /*
+             * Skip any unconsumed bytes
+             */
+            while (msg->bodyBytes) {
+                uint16_t sz = AJ_IO_BUF_AVAIL(ioBuf);
+                sz = min(sz, msg->bodyBytes);
+                if (!sz) {
+                    AJ_IO_BUF_RESET(ioBuf);
+                    sz = min(msg->bodyBytes, ioBuf->bufSize);
+                }
+                status = LoadBytes(ioBuf, sz, 0, msg);
+                if (status != AJ_OK) {
+                    break;
+                }
+                msg->bodyBytes -= sz;
+                ioBuf->readPtr += sz;
             }
-            status = LoadBytes(ioBuf, sz, 0, &msg->timeout);
-            if (status != AJ_OK) {
-                break;
-            }
-            msg->bodyBytes -= sz;
-            ioBuf->readPtr += sz;
         }
+
         memset(msg, 0, sizeof(AJ_Message));
 #ifndef NDEBUG
         currentMsg = NULL;
@@ -739,6 +780,24 @@ AJ_Status AJ_CloseMsg(AJ_Message* msg)
     return status;
 }
 
+AJ_Status AJ_CloseMsgAndSaveReplyContext(AJ_Message* msg, AJ_MsgReplyContext* replyCtx)
+{
+    if (replyCtx) {
+        size_t len = strlen(msg->sender) + 1;
+        if (len > sizeof(replyCtx->sender)) {
+            AJ_CloseMsg(msg);
+            return AJ_ERR_RESOURCES;
+        }
+        memcpy(replyCtx->sender, msg->sender, len + 1);
+        replyCtx->bus = msg->bus;
+        replyCtx->flags = msg->hdr->flags;
+        replyCtx->serialNum = msg->hdr->serialNum;
+        replyCtx->msgId = msg->msgId;
+        replyCtx->sessionId = msg->sessionId;
+    }
+    return AJ_CloseMsg(msg);
+}
+
 /**
  * Get the length of the signature of the first complete type in sig
  */
@@ -784,7 +843,7 @@ static AJ_Status Unmarshal(AJ_Message* msg, const char** sig, AJ_Arg* arg);
 static AJ_Status UnmarshalStruct(AJ_Message* msg, const char** sig, AJ_Arg* arg, uint8_t pad)
 {
     AJ_IOBuffer* ioBuf = &msg->bus->sock.rx;
-    AJ_Status status = LoadBytes(ioBuf, 0, pad, &msg->timeout);
+    AJ_Status status = LoadBytes(ioBuf, 0, pad, msg);
     arg->val.v_data = ioBuf->readPtr;
     arg->sigPtr = *sig;
     /*
@@ -819,7 +878,7 @@ static AJ_Status UnmarshalArray(AJ_Message* msg, const char** sig, AJ_Arg* arg,
     /*
      * Get the byte count for the array
      */
-    status = LoadBytes(ioBuf, 4, pad, &msg->timeout);
+    status = LoadBytes(ioBuf, 4, pad, msg);
     if (status != AJ_OK) {
         return status;
     }
@@ -831,7 +890,7 @@ static AJ_Status UnmarshalArray(AJ_Message* msg, const char** sig, AJ_Arg* arg,
      * the array element types align on an 8 byte boundary.
      */
     pad = PadForType(typeId, ioBuf);
-    status = LoadBytes(ioBuf, numBytes, pad, &msg->timeout);
+    status = LoadBytes(ioBuf, numBytes, pad, msg);
     if (status != AJ_OK) {
         return status;
     }
@@ -883,7 +942,7 @@ static AJ_Status Unmarshal(AJ_Message* msg, const char** sig, AJ_Arg* arg)
 
     if (IsScalarType(typeId)) {
         sz = SizeOfType(typeId);
-        status = LoadBytes(ioBuf, sz, pad, &msg->timeout);
+        status = LoadBytes(ioBuf, sz, pad, msg);
         if (status != AJ_OK) {
             return status;
         }
@@ -904,7 +963,7 @@ static AJ_Status Unmarshal(AJ_Message* msg, const char** sig, AJ_Arg* arg)
          * Read the string length. Note the length doesn't include the terminating NUL
          * so an empty string in encoded as two zero bytes.
          */
-        status = LoadBytes(ioBuf, lenSize, pad, &msg->timeout);
+        status = LoadBytes(ioBuf, lenSize, pad, msg);
         if (status != AJ_OK) {
             return status;
         }
@@ -915,7 +974,7 @@ static AJ_Status Unmarshal(AJ_Message* msg, const char** sig, AJ_Arg* arg)
             sz = (uint32_t)(*ioBuf->readPtr);
         }
         ioBuf->readPtr += lenSize;
-        status = LoadBytes(ioBuf, sz + 1, 0, &msg->timeout);
+        status = LoadBytes(ioBuf, sz + 1, 0, msg);
         if (status != AJ_OK) {
             return status;
         }
@@ -1014,7 +1073,7 @@ AJ_Status AJ_ResetArgs(AJ_Message* msg)
     AJ_ASSERT(msg->sigOffset == strlen(msg->signature));
     if (status == AJ_OK) {
         AJ_IOBuffer* ioBuf = &msg->bus->sock.rx;
-        size_t hdrSize = sizeof(AJ_MsgHeader) + msg->hdr->headerLen + ((8 - msg->hdr->headerLen) & 7);
+        size_t hdrSize = sizeof(AJ_MsgHeader) + msg->hdr->headerLen + HEADERPAD(msg->hdr->headerLen);
         /*
          * Args have already been converted to native endianess in place in the input buffer, this
          * prevents the unmarshaler from incorrectly undoing the conversion.
@@ -1040,63 +1099,66 @@ AJ_Status AJ_ResetArgs(AJ_Message* msg)
  *
  * @param msg   The message
  */
+
 static AJ_Status ProcessBusMessages(AJ_Message* msg)
 {
     AJ_Status status = AJ_OK;
+    int resetArgs = TRUE;
     /* Check for the special case messages */
     switch (msg->msgId) {
     case AJ_REPLY_ID(AJ_METHOD_NAME_HAS_OWNER):
         AJ_InfoPrintf(("ProcessBusMessages(): AJ_REPLY_ID(AJ_METHOD_NAME_HAS_OWNER)\n"));
         status = AJ_GUID_HandleNameHasOwnerReply(msg);
-        /*
-         * Reset so the application can handle this too.  Don't overwrite any
-         * error detected during AJ_GUID_HandleNameHasOwnerReply() above.
-         */
-        if (status == AJ_OK) {
-            status = AJ_ResetArgs(msg);
-        }
         break;
 
     case AJ_REPLY_ID(AJ_METHOD_ADD_MATCH):
         AJ_InfoPrintf(("ProcessBusMessages(): AJ_REPLY_ID(AJ_METHOD_ADD_MATCH)\n"));
         status = AJ_GUID_HandleAddMatchReply(msg);
-        /*
-         * Reset so the application can handle this too.  Don't overwrite any
-         * error detected during AJ_GUID_HandleAddMatchReply() above.
-         */
-        if (status == AJ_OK) {
-            status = AJ_ResetArgs(msg);
-        }
         break;
 
     case AJ_REPLY_ID(AJ_METHOD_REMOVE_MATCH):
         AJ_InfoPrintf(("ProcessBusMessages(): AJ_REPLY_ID(AJ_METHOD_REMOVE_MATCH)\n"));
         status = AJ_GUID_HandleRemoveMatchReply(msg);
-        /*
-         * Reset so the application can handle this too.  Don't overwrite any
-         * error detected during AJ_GUID_HandleRemoveMatchReply() above.
-         */
-        if (status == AJ_OK) {
-            status = AJ_ResetArgs(msg);
-        }
         break;
 
     case AJ_SIGNAL_NAME_OWNER_CHANGED:
         AJ_InfoPrintf(("ProcessBusMessages(): AJ_SIGNAL_NAME_OWNER_CHANGED)\n"));
         status = AJ_GUID_HandleNameOwnerChanged(msg);
-        /*
-         * Reset so the application can handle this too.  Don't overwrite any
-         * error detected during AJ_GUID_HandleNameOwnerChanged() above.
-         */
-        if (status == AJ_OK) {
-            status = AJ_ResetArgs(msg);
-        }
+        break;
+
+    case AJ_SIGNAL_SESSION_JOINED:
+        AJ_InfoPrintf(("ProcessBusMessages(): AJ_SIGNAL_SESSION_JOINED\n"));
+        status = AJ_BusHandleSessionJoined(msg);
+        break;
+
+    case AJ_SIGNAL_SESSION_LOST:
+        AJ_InfoPrintf(("ProcessBusMessages(): AJ_SIGNAL_SESSION_LOST\n"));
+        status = AJ_BusHandleSessionLost(msg);
+        break;
+
+    case AJ_SIGNAL_SESSION_LOST_WITH_REASON:
+        AJ_InfoPrintf(("ProcessBusMessages(): AJ_SIGNAL_SESSION_LOST_WITH_REASON\n"));
+        status = AJ_BusHandleSessionLostWithReason(msg);
+        break;
+
+    case AJ_REPLY_ID(AJ_METHOD_JOIN_SESSION):
+        AJ_InfoPrintf(("ProcessBusMessages(): AJ_REPLY_ID(AJ_METHOD_JOIN_SESSION)\n"));
+        status = AJ_BusHandleJoinSessionReply(msg);
         break;
 
     default:
+        resetArgs = FALSE;
         break;
     }
 
+    if (resetArgs && status == AJ_OK) {
+        /*
+         * Reset the message so the application can handle it too.  Don't
+         * overwrite any error detected during the handling of messages above.
+         */
+        status = AJ_ResetArgs(msg);
+    }
+
     return status;
 }
 
@@ -1162,15 +1224,21 @@ AJ_Status AJ_UnmarshalMsg(AJ_BusAttachment* bus, AJ_Message* msg, uint32_t timeo
                 msg->sender = AJ_GetUniqueName(msg->bus);
                 msg->destination = msg->sender;
                 status = AJ_OK;
+            } else if (status == AJ_ERR_ARDP_RECV_EXPIRED) {
+                status = AJ_ERR_UNMARSHAL;
+                msg->expired = TRUE;
             }
 
             return status;
         }
     }
     /*
-     * Header was unmarsalled directly into the rx buffer
+     * Header was unmarshalled directly into the rx buffer
      */
     msg->hdr = (AJ_MsgHeader*)ioBuf->bufStart;
+    AJ_ASSERT(msg->hdr);
+    /* Save raw header before endian swaps */
+    memcpy(&msg->raw, msg->hdr, sizeof (AJ_MsgHeader));
     ioBuf->readPtr += sizeof(AJ_MsgHeader);
     /*
      * Quick sanity check on the header - unrecoverable error if this check fails
@@ -1187,7 +1255,7 @@ AJ_Status AJ_UnmarshalMsg(AJ_BusAttachment* bus, AJ_Message* msg, uint32_t timeo
     /*
      * The header is null-padded to an 8-byte boundary
      */
-    hdrPad = (8 - msg->hdr->headerLen) & 7;
+    hdrPad = HEADERPAD(msg->hdr->headerLen);
     /*
      * Make sure the header (plus pad) isn't going to overrun the buffer
      * and that the total header length doesn't overflow.
@@ -1199,7 +1267,7 @@ AJ_Status AJ_UnmarshalMsg(AJ_BusAttachment* bus, AJ_Message* msg, uint32_t timeo
     /*
      * Load the header
      */
-    status = LoadBytes(ioBuf, msg->hdr->headerLen + hdrPad, 0, &msg->timeout);
+    status = LoadBytes(ioBuf, msg->hdr->headerLen + hdrPad, 0, msg);
     if (status != AJ_OK) {
         return status;
     }
@@ -1211,11 +1279,11 @@ AJ_Status AJ_UnmarshalMsg(AJ_BusAttachment* bus, AJ_Message* msg, uint32_t timeo
     currentMsg = msg;
 #endif
     /*
-     * If the message is encrypted and the endianess of the message is different than the local host
+     * If the endianess of the message is different than the local host
      * endianness we need to copy the header bytes before we unmarshal the header and swizzle all
      * the integers.
      */
-    if ((msg->hdr->endianess != HOST_ENDIANESS) && (msg->hdr->flags & AJ_FLAG_ENCRYPTED)) {
+    if ((msg->hdr->endianess != HOST_ENDIANESS)) {
         hdrRaw = AJ_Malloc(msg->hdr->headerLen);
         if (hdrRaw) {
             memcpy(hdrRaw, ioBuf->readPtr, msg->hdr->headerLen);
@@ -1237,7 +1305,7 @@ AJ_Status AJ_UnmarshalMsg(AJ_BusAttachment* bus, AJ_Message* msg, uint32_t timeo
         /*
          * Custom unmarshal the header field - signature is "(yv)" so starts off with STRUCT aligment.
          */
-        status = LoadBytes(ioBuf, 4, PadForType(AJ_ARG_STRUCT, ioBuf), &msg->timeout);
+        status = LoadBytes(ioBuf, 4, PadForType(AJ_ARG_STRUCT, ioBuf), msg);
         if (status != AJ_OK) {
             break;
         }
@@ -1325,8 +1393,8 @@ AJ_Status AJ_UnmarshalMsg(AJ_BusAttachment* bus, AJ_Message* msg, uint32_t timeo
         status = ValidateHeader(msg);
     }
     /*
-     * If we copied the raw header earlier we copy it back now. This only happens if the message is
-     * encrypted and the endianness of the message is different from the local host's endianness.
+     * If we copied the raw header earlier we copy it back now.
+     * This only happens if endianness of the message is different from the local host's endianness.
      */
     if (hdrRaw) {
         memcpy(ioBuf->bufStart + sizeof(AJ_MsgHeader), hdrRaw, msg->hdr->headerLen);
@@ -1346,9 +1414,26 @@ AJ_Status AJ_UnmarshalMsg(AJ_BusAttachment* bus, AJ_Message* msg, uint32_t timeo
          * If the message is encrypted load the entire message body and decrypt it.
          */
         if (msg->hdr->flags & AJ_FLAG_ENCRYPTED) {
-            status = LoadBytes(ioBuf, msg->hdr->bodyLen, 0, &msg->timeout);
+            status = LoadBytes(ioBuf, msg->hdr->bodyLen, 0, msg);
             if (status == AJ_OK) {
                 status = DecryptMessage(msg);
+                /*
+                 * If session key missing, reply with error message.
+                 * If decryption failed, silently ignore message below.
+                 */
+                if ((AJ_ERR_NO_MATCH == status) && (msg->hdr->msgType == AJ_MSG_METHOD_CALL) && !(msg->hdr->flags & AJ_FLAG_NO_REPLY_EXPECTED)) {
+                    AJ_Message reply;
+                    AJ_Status replyStatus;
+                    status = AJ_ERR_SECURITY;
+                    replyStatus = AJ_MarshalStatusMsg(msg, &reply, status);
+                    if (AJ_OK == replyStatus) {
+                        replyStatus = AJ_DeliverMsg(&reply);
+                    }
+                    if (AJ_OK != replyStatus) {
+                        /* Fail to send an error reply, log and continue */
+                        AJ_InfoPrintf(("AJ_UnmarshalMsg(): %s\n", AJ_StatusText(replyStatus)));
+                    }
+                }
             }
         }
         /*
@@ -1363,6 +1448,36 @@ AJ_Status AJ_UnmarshalMsg(AJ_BusAttachment* bus, AJ_Message* msg, uint32_t timeo
         if (status == AJ_OK) {
             status = AJ_IdentifyMessage(msg);
         }
+
+        /*
+         * If this is the Peer.Authentication interface,
+         * load the entire message into the buffer.
+         * Only do this if it wasn't done above (encrypted message).
+         */
+        if ((AJ_BUS_MESSAGE_ID(0xFF, 0xFF, 0) & msg->msgId) == AJ_PEER_AUTHENTICATION_IFN) {
+            if (!(msg->hdr->flags & AJ_FLAG_ENCRYPTED)) {
+                status = LoadBytes(ioBuf, msg->hdr->bodyLen, 0, msg);
+            }
+        }
+
+        /*
+         * Check incoming policy
+         */
+        if ((AJ_OK == status) && (msg->hdr->flags & AJ_FLAG_ENCRYPTED)) {
+            status = AuthoriseIncomingMessage(msg);
+            if ((AJ_OK != status) && (msg->hdr->msgType == AJ_MSG_METHOD_CALL) && !(msg->hdr->flags & AJ_FLAG_NO_REPLY_EXPECTED)) {
+                AJ_Message reply;
+                AJ_Status replyStatus;
+                replyStatus = AJ_MarshalStatusMsg(msg, &reply, status);
+                if (AJ_OK == replyStatus) {
+                    replyStatus = AJ_DeliverMsg(&reply);
+                }
+                if (AJ_OK != replyStatus) {
+                    /* Fail to send an error reply, log and continue */
+                    AJ_InfoPrintf(("AJ_UnmarshalMsg(): %s\n", AJ_StatusText(replyStatus)));
+                }
+            }
+        }
     } else {
         /*
          * Consume entire header
@@ -1417,7 +1532,7 @@ AJ_Status AJ_SkipArg(AJ_Message* msg)
             /*
              * Just consume the array bytes
              */
-            status = LoadBytes(ioBuf, skippy.len, 0, &msg->timeout);
+            status = LoadBytes(ioBuf, skippy.len, 0, msg);
             if (status == AJ_OK) {
                 ioBuf->readPtr += skippy.len;
                 msg->bodyBytes -= skippy.len;
@@ -1426,8 +1541,8 @@ AJ_Status AJ_SkipArg(AJ_Message* msg)
             /*
              * Skip the individual elements
              */
-            char close = (skippy.typeId == AJ_ARG_STRUCT) ?  AJ_STRUCT_CLOSE : AJ_DICT_ENTRY_CLOSE;
-            while (*skippy.sigPtr != close) {
+            char closeType = (skippy.typeId == AJ_ARG_STRUCT) ?  AJ_STRUCT_CLOSE : AJ_DICT_ENTRY_CLOSE;
+            while (*skippy.sigPtr != closeType) {
                 status = AJ_SkipArg(msg);
                 if (status != AJ_OK) {
                     break;
@@ -1487,7 +1602,7 @@ AJ_Status AJ_UnmarshalArg(AJ_Message* msg, AJ_Arg* arg)
              */
             if (len == container->len) {
                 memset(arg, 0, sizeof(AJ_Arg));
-                AJ_ErrPrintf(("AJ_UnmarshalMsg(): AJ_ERR_NO_MORE\n"));
+                AJ_InfoPrintf(("AJ_UnmarshalMsg(): AJ_ERR_NO_MORE\n"));
                 status = AJ_ERR_NO_MORE;
             } else {
                 status = Unmarshal(msg, &sig, arg);
@@ -1656,7 +1771,7 @@ AJ_Status AJ_UnmarshalRaw(AJ_Message* msg, const void** data, size_t len, size_t
     AJ_Status status;
     size_t sz;
     AJ_IOBuffer* ioBuf = &msg->bus->sock.rx;
-    size_t hdrSize = 8 + msg->hdr->headerLen + ((8 - msg->hdr->headerLen) & 7);
+    size_t hdrSize = sizeof(AJ_MsgHeader) + msg->hdr->headerLen + HEADERPAD(msg->hdr->headerLen);
 
     /*
      * A sig offset of 0xFF indicates we are already doing raw unnmarshaling
@@ -1679,7 +1794,10 @@ AJ_Status AJ_UnmarshalRaw(AJ_Message* msg, const void** data, size_t len, size_t
             AJ_ErrPrintf(("AJ_UnmarshalRaw(): AJ_ERR_UNMARSHAL\n"));
             return AJ_ERR_UNMARSHAL;
         }
-        LoadBytes(ioBuf, 0, pad, &msg->timeout);
+        status = LoadBytes(ioBuf, 0, pad, msg);
+        if (status != AJ_OK) {
+            return status;
+        }
         msg->bodyBytes -= pad;
         /*
          * Standard signature matching is now meaningless
@@ -1708,7 +1826,7 @@ AJ_Status AJ_UnmarshalRaw(AJ_Message* msg, const void** data, size_t len, size_t
      * If we try to load more than the available space we will get an error
      */
     len = min(len, AJ_IO_BUF_SPACE(ioBuf));
-    status = LoadBytes(ioBuf, (uint16_t)len, 0, &msg->timeout);
+    status = LoadBytes(ioBuf, (uint16_t)len, 0, msg);
     if (status == AJ_OK) {
         sz = AJ_IO_BUF_AVAIL(ioBuf);
         if (sz < len) {
@@ -1916,6 +2034,10 @@ static AJ_Status MarshalMsg(AJ_Message* msg, uint8_t msgType, uint32_t msgId, ui
     uint8_t fieldId;
     uint8_t secure = FALSE;
 
+    if (!ioBuf->bufStart) {
+        AJ_ErrPrintf(("MarshalMsg(): ioBuf has not been initialized\n"));
+        return AJ_ERR_IO_BUFFER;
+    }
 #ifdef AJ_ARDP
     status = AJ_ARDP_StartMsgSend(msg->ttl);
 #endif
@@ -2066,7 +2188,7 @@ static AJ_Status MarshalMsg(AJ_Message* msg, uint8_t msgType, uint32_t msgId, ui
         /*
          * Header must be padded to an 8 byte boundary
          */
-        status = WritePad(msg, (8 - msg->hdr->headerLen) & 7);
+        status = WritePad(msg, HEADERPAD(msg->hdr->headerLen));
     }
     return status;
 }
@@ -2416,6 +2538,18 @@ AJ_Status AJ_MarshalSignal(AJ_BusAttachment* bus, AJ_Message* msg, uint32_t msgI
     msg->bus = bus;
     msg->destination = destination;
     msg->sessionId = sessionId;
+    /* Special case: if destination == NULL and sessionId != 0 and the corresponding session
+     * is unicast, this is functionally equivalent to a unicast signal over that session.
+     * As a matter of fact, it is common practice to emit signals in this way. The Security 2.0
+     * policy checking code treats both cases differently, though, and to preserve the
+     * equivalent behavior of before, we just fill in the destination here, if it's NULL.
+     */
+    if (destination == NULL && sessionId != 0) {
+        AJ_Session* session = AJ_BusGetOngoingSession(bus, sessionId);
+        if (session && !session->multipoint) {
+            msg->destination = session->otherParticipant;
+        }
+    }
     msg->ttl = ttl;
     return MarshalMsg(msg, AJ_MSG_SIGNAL, msgId, flags);
 }
@@ -2428,7 +2562,6 @@ AJ_Status AJ_MarshalReplyMsg(const AJ_Message* methodCall, AJ_Message* reply)
     reply->destination = methodCall->sender;
     reply->sessionId = methodCall->sessionId;
     reply->replySerial = methodCall->hdr->serialNum;
-    reply->ttl = 0;
     return MarshalMsg(reply, AJ_MSG_METHOD_RET, methodCall->msgId, methodCall->hdr->flags & AJ_FLAG_ENCRYPTED);
 }
 
@@ -2443,7 +2576,6 @@ AJ_Status AJ_MarshalErrorMsgWithInfo(const AJ_Message* methodCall, AJ_Message* r
     reply->sessionId = methodCall->sessionId;
     reply->replySerial = methodCall->hdr->serialNum;
     reply->error = error;
-    reply->ttl = 0;
     if (info) {
         reply->signature = "s";
     }
@@ -2460,27 +2592,151 @@ AJ_Status AJ_MarshalErrorMsg(const AJ_Message* methodCall, AJ_Message* reply, co
 }
 
 
-AJ_Status AJ_MarshalStatusMsg(const AJ_Message* methodCall, AJ_Message* reply, AJ_Status status)
+static const char* StatusToErrorStrings(AJ_Status status, const char** info)
 {
     switch (status) {
     case AJ_ERR_NO_MATCH:
-        status = AJ_MarshalErrorMsg(methodCall, reply, AJ_ErrServiceUnknown);
-        break;
+        *info = NULL;
+        return AJ_ErrServiceUnknown;
 
-    case  AJ_ERR_SECURITY:
-        status = AJ_MarshalErrorMsg(methodCall, reply, AJ_ErrSecurityViolation);
-        /*
-         * We get a security violation error so if we encrypt the error message the receiver
-         * won't be able to decrypt it. We can fix this by clearing the header flags.
-         */
-        if (status == AJ_OK) {
-            reply->hdr->flags = 0;
-        }
-        break;
+    case AJ_ERR_SECURITY:
+        *info = NULL;
+        return AJ_ErrSecurityViolation;
+
+    case  AJ_ERR_ACCESS:
+        *info = NULL;
+        return AJ_ErrPermissionDenied;
+
+    case  AJ_ERR_SECURITY_DIGEST_MISMATCH:
+        *info = NULL;
+        return AJ_ErrDigestMismatch;
+
+    case  AJ_ERR_SECURITY_INVALID_CERTIFICATE:
+        *info = NULL;
+        return AJ_ErrInvalidCertificate;
+
+    case  AJ_ERR_SECURITY_DUPLICATE_CERTIFICATE:
+        *info = NULL;
+        return AJ_ErrDuplicateCertificate;
+
+    case  AJ_ERR_SECURITY_CERTIFICATE_NOT_FOUND:
+        *info = NULL;
+        return AJ_ErrCertificateNotFound;
+
+    case  AJ_ERR_SECURITY_POLICY_NOT_NEWER:
+        *info = NULL;
+        return AJ_ErrPolicyNotNewer;
 
     default:
-        status = AJ_MarshalErrorMsgWithInfo(methodCall, reply, AJ_ErrRejected, AJ_StatusText(status));
-        break;
+        *info = AJ_StatusText(status);
+        return AJ_ErrRejected;
     }
+}
+
+AJ_Status AJ_MarshalStatusMsg(const AJ_Message* methodCall, AJ_Message* reply, AJ_Status status)
+{
+    const char* info;
+    const char* err = StatusToErrorStrings(status, &info);
+    return AJ_MarshalErrorMsgWithInfo(methodCall, reply, err, info);
+}
+
+AJ_Status AJ_MarshalReplyMsgAsync(AJ_MsgReplyContext* replyCtx, AJ_Message* reply)
+{
+    AJ_Status status;
+
+    if (!replyCtx) {
+        return AJ_ERR_NULL;
+    }
+    if (!replyCtx->msgId) {
+        return AJ_ERR_INVALID;
+    }
+    memset(reply, 0, sizeof(AJ_Message));
+    reply->bus = replyCtx->bus;
+    reply->destination = replyCtx->sender;
+    reply->sessionId = replyCtx->sessionId;
+    reply->replySerial = replyCtx->serialNum;
+    status = MarshalMsg(reply, AJ_MSG_METHOD_RET, replyCtx->msgId, replyCtx->flags & AJ_FLAG_ENCRYPTED);
+    /*
+     * Ensure reply context is only be used once
+     */
+    memset(replyCtx, 0, sizeof(*replyCtx));
     return status;
 }
+
+
+AJ_Status AJ_MarshalErrorMsgAsync(AJ_MsgReplyContext* replyCtx, AJ_Message* reply, const char* error)
+{
+    return AJ_MarshalErrorMsgWithInfoAsync(replyCtx, reply, error, NULL);
+}
+
+AJ_Status AJ_MarshalErrorMsgWithInfoAsync(AJ_MsgReplyContext* replyCtx, AJ_Message* reply, const char* error, const char* info)
+{
+    AJ_Status status;
+
+    if (!replyCtx) {
+        return AJ_ERR_NULL;
+    }
+    if (!replyCtx->msgId) {
+        return AJ_ERR_INVALID;
+    }
+    memset(reply, 0, sizeof(AJ_Message));
+    reply->bus = replyCtx->bus;
+    reply->destination = replyCtx->sender;
+    reply->sessionId = replyCtx->sessionId;
+    reply->replySerial = replyCtx->serialNum;
+    reply->error = error;
+    if (info) {
+        reply->signature = "s";
+    }
+    status = MarshalMsg(reply, AJ_MSG_ERROR, replyCtx->msgId, replyCtx->flags & AJ_FLAG_ENCRYPTED);
+    if ((status == AJ_OK) && info) {
+        status = AJ_MarshalArgs(reply, "s", info);
+    }
+    /*
+     * Ensure reply context is only be used once
+     */
+    memset(replyCtx, 0, sizeof(*replyCtx));
+    return status;
+}
+
+AJ_Status AJ_MarshalStatusMsgAsync(AJ_MsgReplyContext* replyCtx, AJ_Message* reply, AJ_Status status)
+{
+    const char* info;
+    const char* err = StatusToErrorStrings(status, &info);
+    return AJ_MarshalErrorMsgWithInfoAsync(replyCtx, reply, err, info);
+}
+
+AJ_Status rx_noop(AJ_IOBuffer* buf, uint32_t len, uint32_t timeout)
+{
+    buf->writePtr += len;
+    return AJ_OK;
+}
+
+AJ_Status tx_noop(AJ_IOBuffer* buf)
+{
+    return AJ_OK;
+}
+
+/* This is the minimum requirements to unmarshal a message locally */
+void AJ_LocalMsg(AJ_BusAttachment* bus, AJ_MsgHeader* hdr, AJ_Message* msg, const char* sig, uint8_t* data, size_t size)
+{
+    memset(msg, 0, sizeof (AJ_Message));
+    memset(hdr, 0, sizeof (AJ_MsgHeader));
+    hdr->endianess = AJ_NATIVE_ENDIAN;
+    msg->hdr = hdr;
+    msg->bus = bus;
+    msg->signature = sig;
+    msg->bodyBytes = size;
+    bus->sock.rx.direction = AJ_IO_BUF_RX;
+    bus->sock.rx.recv = rx_noop;
+    bus->sock.rx.bufSize = size;
+    bus->sock.rx.bufStart = data;
+    bus->sock.rx.readPtr = bus->sock.rx.bufStart;
+    bus->sock.rx.writePtr = bus->sock.rx.bufStart;
+    bus->sock.tx.direction = AJ_IO_BUF_TX;
+    bus->sock.tx.send = tx_noop;
+    bus->sock.tx.bufSize = size;
+    bus->sock.tx.bufStart = data;
+    bus->sock.tx.readPtr = bus->sock.tx.bufStart;
+    bus->sock.tx.writePtr = bus->sock.tx.bufStart;
+}
