@@ -38,6 +38,7 @@
 #include <ajtcl/aj_cert.h>
 #include <ajtcl/aj_authorisation.h>
 #include <ajtcl/aj_security.h>
+#include <ajtcl/aj_conversationhash.h>
 
 /**
  * Turn on per-module debug printing by setting this variable to non-zero value
@@ -71,7 +72,6 @@ static AJ_Status ExchangeSuites(AJ_Message* msg);
 static AJ_Status KeyExchange(AJ_BusAttachment* bus);
 static AJ_Status KeyAuthentication(AJ_Message* msg);
 static AJ_Status GenSessionKey(AJ_Message* msg);
-static AJ_Status SendManifest(AJ_Message* msg);
 static AJ_Status SendMemberships(AJ_Message* msg);
 
 typedef enum {
@@ -92,6 +92,7 @@ typedef struct _PeerContext {
 
 static PeerContext peerContext;
 static AJ_AuthenticationContext authContext = { 0 };
+static uint8_t sentManifests = FALSE;
 
 static uint32_t GetAcceptableVersion(uint32_t srcV)
 {
@@ -162,6 +163,11 @@ static AJ_Status KeyGen(const char* peerName, uint8_t role, const char* nonce1,
     return status;
 }
 
+void AJ_ClearSentManifests()
+{
+    sentManifests = FALSE;
+}
+
 void AJ_ClearAuthContext()
 {
     /* Free issuers, hash, and PSK */
@@ -277,8 +283,8 @@ static AJ_Status SaveECDSAContext(const AJ_GUID* peerGuid, uint32_t expiration)
         return AJ_ERR_SECURITY;
     }
 
-    if ((AJ_AUTH_SUCCESS == peerContext.state) && authContext.kactx.ecdsa.size) {
-        status = AJ_CredentialSetPeer(AJ_GENERIC_ECDSA_MANIFEST, peerGuid, expiration, authContext.kactx.ecdsa.manifest, (uint16_t)authContext.kactx.ecdsa.size);
+    if ((AJ_AUTH_SUCCESS == peerContext.state) && (authContext.kactx.ecdsa.thumbprintSize > 0)) {
+        status = AJ_CredentialSetPeer(AJ_GENERIC_ECDSA_THUMBPRINT, peerGuid, expiration, authContext.kactx.ecdsa.thumbprint, (uint16_t)authContext.kactx.ecdsa.thumbprintSize);
         if (AJ_OK != status) {
             return status;
         }
@@ -298,18 +304,18 @@ static AJ_Status LoadECDSAContext(const AJ_GUID* peerGuid)
 
     AJ_InfoPrintf(("LoadECDSAContext(peerGuid=%p)\n", peerGuid));
 
-    /* Check if we have a stored manifest */
+    /* Check if we have a stored identity thumbprint */
     data.size = AJ_SHA256_DIGEST_LENGTH;
-    data.data = authContext.kactx.ecdsa.manifest;
-    status = AJ_CredentialGetPeer(AJ_GENERIC_ECDSA_MANIFEST, peerGuid, NULL, &data);
+    data.data = authContext.kactx.ecdsa.thumbprint;
+    status = AJ_CredentialGetPeer(AJ_GENERIC_ECDSA_THUMBPRINT, peerGuid, NULL, &data);
     if (AJ_OK == status) {
-        authContext.kactx.ecdsa.size = data.size;
+        authContext.kactx.ecdsa.thumbprintSize = data.size;
     } else {
         peerContext.state = AJ_AUTH_SUCCESS;
         return AJ_OK;
     }
 
-    /* If we have a manifest, we require stored public keys */
+    /* If we have an identity certificate thumbprint, we require stored public keys */
     data.size = 0;
     /* Keys is NULL, AJ_CredentialGetPeer will allocate the memory */
     data.data = (uint8_t*) authContext.kactx.ecdsa.key;
@@ -435,6 +441,7 @@ AJ_Status AJ_PeerAuthenticate(AJ_BusAttachment* bus, const char* peerName, AJ_Pe
      * No handshake in progress or previous timed-out
      */
     AJ_ClearAuthContext();
+    AJ_ClearSentManifests();
     peerContext.callback = callback;
     peerContext.cbContext = cbContext;
     peerContext.peerName = peerName;
@@ -450,6 +457,7 @@ AJ_Status AJ_PeerAuthenticate(AJ_BusAttachment* bus, const char* peerName, AJ_Pe
 
     if (bus->pwdCallback) {
         AJ_EnableSuite(bus, AUTH_SUITE_ECDHE_PSK);
+        AJ_EnableSuite(bus, AUTH_SUITE_ECDHE_SPEKE);
     }
 
     /*
@@ -512,6 +520,7 @@ AJ_Status AJ_PeerHandleExchangeGUIDs(AJ_Message* msg, AJ_Message* reply)
      * No handshake in progress or previous timed-out
      */
     AJ_ClearAuthContext();
+    AJ_ClearSentManifests();
     AJ_InitTimer(&peerContext.timer);
     authContext.bus = msg->bus;
     authContext.role = AUTH_SERVER;
@@ -524,6 +533,7 @@ AJ_Status AJ_PeerHandleExchangeGUIDs(AJ_Message* msg, AJ_Message* reply)
 
     if (msg->bus->pwdCallback) {
         AJ_EnableSuite(msg->bus, AUTH_SUITE_ECDHE_PSK);
+        AJ_EnableSuite(msg->bus, AUTH_SUITE_ECDHE_SPEKE);
     }
 
     status = AJ_UnmarshalArgs(msg, "su", &str, &authContext.version);
@@ -642,6 +652,7 @@ AJ_Status AJ_PeerHandleExchangeGUIDsReply(AJ_Message* msg)
         AJ_WarnPrintf(("AJ_PeerHandleExchangeGUIDsReply(msg=%p): Invalid GUID\n", msg));
         goto Exit;
     }
+
     /*
      * Two name mappings to add, the well known name, and the unique name from the message.
      */
@@ -707,6 +718,9 @@ static AJ_Status ExchangeSuites(AJ_Message* msg)
     if (AJ_IsSuiteEnabled(msg->bus, AUTH_SUITE_ECDHE_PSK, AJ_UNPACK_AUTH_VERSION(authContext.version))) {
         suites[num++] = AUTH_SUITE_ECDHE_PSK;
     }
+    if (AJ_IsSuiteEnabled(msg->bus, AUTH_SUITE_ECDHE_SPEKE, AJ_UNPACK_AUTH_VERSION(authContext.version))) {
+        suites[num++] = AUTH_SUITE_ECDHE_SPEKE;
+    }
     if (AJ_IsSuiteEnabled(msg->bus, AUTH_SUITE_ECDHE_NULL, AJ_UNPACK_AUTH_VERSION(authContext.version))) {
         suites[num++] = AUTH_SUITE_ECDHE_NULL;
     }
@@ -901,6 +915,23 @@ Exit:
     return AJ_ERR_SECURITY;
 }
 
+static AJ_Status AJ_SetKeyAuthContext(AJ_AuthenticationContext* ctx, const char* peerName)
+{
+    AJ_Status status;
+
+    if (ctx->suite == AUTH_SUITE_ECDHE_SPEKE) {
+        status = AJ_GetLocalGUID(&ctx->kactx.speke.localGUID);
+        if (status != AJ_OK) {
+            return status;
+        }
+        ctx->kactx.speke.remoteGUID = AJ_GUID_Find(peerName);
+        if (ctx->kactx.speke.remoteGUID == NULL) {
+            return AJ_ERR_NO_MATCH;
+        }
+    }
+    return AJ_OK;
+}
+
 static AJ_Status KeyExchange(AJ_BusAttachment* bus)
 {
     AJ_Status status;
@@ -925,6 +956,11 @@ static AJ_Status KeyExchange(AJ_BusAttachment* bus)
         goto Exit;
     }
 
+    status = AJ_SetKeyAuthContext(&authContext, peerContext.peerName);
+    if (AJ_OK != status) {
+        goto Exit;
+    }
+
     HostU32ToBigEndianU8(&authContext.suite, sizeof (authContext.suite), suiteb8);
     AJ_ConversationHash_Update_UInt8Array(&authContext, CONVERSATION_V1, suiteb8, sizeof (suiteb8));
     status = AJ_KeyExchangeMarshal(&authContext, &call);
@@ -971,6 +1007,11 @@ AJ_Status AJ_PeerHandleKeyExchange(AJ_Message* msg, AJ_Message* reply)
     HostU32ToBigEndianU8(&authContext.suite, sizeof (authContext.suite), suiteb8);
     AJ_ConversationHash_Update_UInt8Array(&authContext, CONVERSATION_V1, suiteb8, sizeof (suiteb8));
 
+    status = AJ_SetKeyAuthContext(&authContext, msg->sender);
+    if (AJ_OK != status) {
+        goto Exit;
+    }
+
     /*
      * Receive key material
      */
@@ -1555,6 +1596,16 @@ AJ_Status AJ_PeerHandleExchangeGroupKeys(AJ_Message* msg, AJ_Message* reply)
         HandshakeComplete(status);
     }
 
+    /* Search for membership certificates from the beginning */
+    authContext.slot = AJ_CREDS_NV_ID_BEGIN;
+    authContext.code = SEND_MEMBERSHIPS_NONE;
+    status = AJ_CredentialGetNext(AJ_CERTIFICATE_MBR_X509 | AJ_CRED_TYPE_CERTIFICATE, NULL, NULL, NULL, &authContext.slot);
+    if (AJ_OK == status) {
+        /* There is at least one cert to send, we don't know if the last yet */
+        authContext.code = SEND_MEMBERSHIPS_MORE;
+    }
+    status = AJ_OK;
+
     return status;
 
 Exit:
@@ -1596,7 +1647,17 @@ AJ_Status AJ_PeerHandleExchangeGroupKeysReply(AJ_Message* msg)
         goto Exit;
     }
     if ((AUTH_SUITE_ECDHE_ECDSA == authContext.suite) && (AJ_UNPACK_AUTH_VERSION(authContext.version) >= CONVERSATION_V4)) {
-        status = SendManifest(msg);
+        /* Search for membership certificates from the beginning */
+        authContext.slot = AJ_CREDS_NV_ID_BEGIN;
+        authContext.code = SEND_MEMBERSHIPS_NONE;
+        status = AJ_CredentialGetNext(AJ_CERTIFICATE_MBR_X509 | AJ_CRED_TYPE_CERTIFICATE, NULL, NULL, NULL, &authContext.slot);
+        AJ_InfoPrintf(("AJ_PeerHandleExchangeGroupKeysReply(msg=%p): Membership slot %d\n", msg, authContext.slot));
+        if (AJ_OK == status) {
+            /* There is at least one certificate to send, we don't know if the last yet */
+            authContext.code = SEND_MEMBERSHIPS_MORE;
+        }
+
+        status = AJ_PeerSendManifests(msg, FALSE);
     } else {
         HandshakeComplete(status);
     }
@@ -1608,146 +1669,152 @@ Exit:
     return AJ_ERR_SECURITY;
 }
 
-static AJ_Status SendManifest(AJ_Message* msg)
+AJ_Status AJ_PeerSendManifests(AJ_Message* msg, uint8_t outgoing)
 {
     AJ_Status status;
     AJ_Message call;
+    const AJ_GUID* peerGuid = AJ_GUID_Find(outgoing ? msg->destination : msg->sender);
     AJ_CredField field = { 0, NULL };
-    AJ_Manifest* manifest = NULL;
+    AJ_ManifestArray* manifests = NULL;
+    uint8_t mustClearAuthContext = FALSE;
 
-    AJ_InfoPrintf(("SendManifest(msg=%p)\n", msg));
+    AJ_InfoPrintf(("AJ_PeerSendManifests(msg=%p, outgoing=%u)\n", msg, outgoing));
 
-    status = AJ_MarshalMethodCall(msg->bus, &call, AJ_METHOD_SEND_MANIFEST, msg->sender, 0, AJ_FLAG_ENCRYPTED, AJ_AUTH_CALL_TIMEOUT);
+    if (sentManifests) {
+        /* Already sent. */
+        return AJ_OK;
+    }
+
+    if (NULL == authContext.bus) {
+        status = LoadECDSAContext(peerGuid);
+        if (AJ_OK != status) {
+            AJ_InfoPrintf(("AJ_PeerSendManifests(msg=%p, outgoing=%u): Could not load ECDSA context for peer\n", msg, outgoing));
+            return AJ_ERR_SECURITY;
+        }
+        mustClearAuthContext = TRUE;
+    }
+
+    if (AUTH_SUITE_ECDHE_ECDSA != authContext.suite) {
+        /* No need to send. */
+        status = AJ_OK;
+        goto Exit;
+    }
+
+    status = AJ_MarshalMethodCall(msg->bus, &call, AJ_METHOD_SEND_MANIFESTS, (outgoing ? msg->destination : msg->sender), 0, AJ_FLAG_ENCRYPTED, AJ_AUTH_CALL_TIMEOUT);
     if (AJ_OK != status) {
         goto Exit;
     }
-    status = AJ_CredentialGet(AJ_CRED_TYPE_MANIFEST, NULL, NULL, &field);
+    status = AJ_CredentialGet(AJ_CRED_TYPE_MANIFESTS, NULL, NULL, &field);
     if (AJ_OK == status) {
-        status = AJ_ManifestFromBuffer(&manifest, &field);
+        status = AJ_ManifestArrayFromBuffer(&manifests, &field);
         if (AJ_OK != status) {
-            AJ_InfoPrintf(("SendManifest(msg=%p): Manifest buffer failed\n", msg));
+            AJ_InfoPrintf(("AJ_PeerSendManifests(msg=%p): Manifests buffer failed\n", msg));
             goto Exit;
         }
-    } else {
-        /* Create empty manifest */
-        manifest = (AJ_Manifest*) AJ_Malloc(sizeof (AJ_Manifest));
-        manifest->rules = NULL;
     }
-    status = AJ_ManifestMarshal(manifest, &call);
+
+    status = AJ_ManifestArrayMarshal(manifests, &call);
     if (AJ_OK != status) {
-        AJ_InfoPrintf(("SendManifest(msg=%p): Manifest marshal failed\n", msg));
+        AJ_InfoPrintf(("AJ_PeerSendManifests(msg=%p): Manifests marshal failed\n", msg));
         goto Exit;
     }
 
 Exit:
     AJ_CredFieldFree(&field);
-    AJ_ManifestFree(manifest);
+    AJ_ManifestArrayFree(manifests);
     if (AJ_OK == status) {
-        return AJ_DeliverMsg(&call);
-    } else {
-        HandshakeComplete(AJ_ERR_SECURITY);
-        return AJ_ERR_SECURITY;
+        status = AJ_DeliverMsg(&call);
+        if (AJ_OK == status) {
+            sentManifests = TRUE;
+        }
     }
+    if (mustClearAuthContext) {
+        AJ_ClearAuthContext();
+    }
+
+    return status;
 }
 
-AJ_Status AJ_PeerHandleSendManifest(AJ_Message* msg, AJ_Message* reply)
+AJ_Status AJ_PeerHandleSendManifests(AJ_Message* msg, AJ_Message* reply)
 {
     AJ_Status status;
     AJ_CredField field = { 0, NULL };
     const AJ_GUID* peerGuid = AJ_GUID_Find(msg->sender);
-    AJ_Manifest* manifest = NULL;
-    uint8_t digest[AJ_SHA256_DIGEST_LENGTH];
+    AJ_ManifestArray* manifests = NULL;
+    uint8_t mustClearAuthContext = FALSE;
 
-    AJ_InfoPrintf(("AJ_PeerHandleSendManifest(msg=%p, reply=%p)\n", msg, reply));
+    AJ_InfoPrintf(("AJ_PeerHandleSendManifests(msg=%p, reply=%p)\n", msg, reply));
 
-    status = HandshakeValid(peerGuid);
-    if (AJ_OK != status) {
-        return AJ_MarshalErrorMsg(msg, reply, AJ_ErrResources);
-    }
-
-    field.data = msg->bus->sock.rx.readPtr;
-    status = AJ_ManifestUnmarshal(&manifest, msg);
-    if (AJ_OK != status) {
-        AJ_InfoPrintf(("AJ_PeerHandleSendManifest(msg=%p, reply=%p): Manifest unmarshal failed\n", msg, reply));
-        field.data = NULL;
-        goto Exit;
-    }
-    field.size = msg->bus->sock.rx.readPtr - field.data;
-    status = AJ_ManifestDigest(&field, digest);
-    field.data = NULL;
-    field.size = 0;
-    if (AJ_OK != status) {
-        goto Exit;
-    }
-    /* Compare with digest from certificate */
-    if (authContext.kactx.ecdsa.size && (0 == memcmp(digest, authContext.kactx.ecdsa.manifest, AJ_SHA256_DIGEST_LENGTH))) {
-        status = AJ_ManifestApply(manifest, msg->sender);
+    /*
+     * This might get called during handshake, or after. If after, load the ECDSA context
+     * so the identity certificate thumbprint is available.
+     */
+    if (NULL == authContext.bus) {
+        status = LoadECDSAContext(peerGuid);
         if (AJ_OK != status) {
-            AJ_InfoPrintf(("AJ_PeerHandleSendManifest(msg=%p, reply=%p): Manifest apply failed\n", msg, reply));
-            goto Exit;
+            AJ_InfoPrintf(("AJ_PeerHandleSendManifests(msg=%p, reply=%p): Could not load ECDSA context for peer\n", msg, reply));
+            return AJ_MarshalErrorMsg(msg, reply, AJ_ErrSecurityViolation);
         }
+        mustClearAuthContext = TRUE;
     }
-    AJ_ManifestFree(manifest);
-    manifest = NULL;
+
+    status = AJ_ManifestArrayUnmarshal(&manifests, msg);
+    if (AJ_OK != status) {
+        AJ_InfoPrintf(("AJ_PeerHandleSendManifests(msg=%p, reply=%p): Manifests unmarshal failed\n", msg, reply));
+        goto Exit;
+    }
+    AJ_ManifestArrayApply(manifests, msg->sender, &authContext);
+
+    AJ_ManifestArrayFree(manifests);
+    manifests = NULL;
 
     status = AJ_MarshalReplyMsg(msg, reply);
     if (AJ_OK != status) {
-        AJ_InfoPrintf(("AJ_PeerHandleSendManifest(msg=%p, reply=%p): Manifest marshal failed\n", msg, reply));
+        AJ_InfoPrintf(("AJ_PeerHandleSendManifests(msg=%p, reply=%p): Manifest marshal failed\n", msg, reply));
         goto Exit;
     }
-    status = AJ_CredentialGet(AJ_CRED_TYPE_MANIFEST, NULL, NULL, &field);
+    status = AJ_CredentialGet(AJ_CRED_TYPE_MANIFESTS, NULL, NULL, &field);
     if (AJ_OK == status) {
-        status = AJ_ManifestFromBuffer(&manifest, &field);
+        status = AJ_ManifestArrayFromBuffer(&manifests, &field);
         if (AJ_OK != status) {
-            AJ_InfoPrintf(("AJ_PeerHandleSendManifest(msg=%p, reply=%p): Manifest buffer failed\n", msg, reply));
+            AJ_InfoPrintf(("AJ_PeerHandleSendManifests(msg=%p, reply=%p): Manifests buffer failed\n", msg, reply));
             goto Exit;
         }
-    } else {
-        /* Create empty manifest */
-        manifest = (AJ_Manifest*) AJ_Malloc(sizeof (AJ_Manifest));
-        manifest->rules = NULL;
     }
-    status = AJ_ManifestMarshal(manifest, reply);
+
+    status = AJ_ManifestArrayMarshal(manifests, reply);
     if (AJ_OK != status) {
-        AJ_InfoPrintf(("AJ_PeerHandleSendManifest(msg=%p, reply=%p): Manifest marshal failed\n", msg, reply));
+        AJ_InfoPrintf(("AJ_PeerHandleSendManifests(msg=%p, reply=%p): Manifests marshal failed\n", msg, reply));
         goto Exit;
     }
-    AJ_ManifestFree(manifest);
-    manifest = NULL;
-
-    /* Search for membership certificates from the beginning */
-    authContext.slot = AJ_CREDS_NV_ID_BEGIN;
-    authContext.code = SEND_MEMBERSHIPS_NONE;
-    status = AJ_CredentialGetNext(AJ_CERTIFICATE_MBR_X509 | AJ_CRED_TYPE_CERTIFICATE, NULL, NULL, NULL, &authContext.slot);
-    if (AJ_OK == status) {
-        /* There is at least one cert to send, we don't know if the last yet */
-        authContext.code = SEND_MEMBERSHIPS_MORE;
-    }
-    status = AJ_OK;
+    AJ_ManifestArrayFree(manifests);
+    manifests = NULL;
 
 Exit:
     AJ_CredFieldFree(&field);
-    AJ_ManifestFree(manifest);
+    AJ_ManifestArrayFree(manifests);
+    if (mustClearAuthContext) {
+        AJ_ClearAuthContext();
+    }
     if (AJ_OK == status) {
+        sentManifests = TRUE;
         return status;
     } else {
-        HandshakeComplete(AJ_ERR_SECURITY);
         return AJ_MarshalErrorMsg(msg, reply, AJ_ErrSecurityViolation);
     }
 }
 
-AJ_Status AJ_PeerHandleSendManifestReply(AJ_Message* msg)
+AJ_Status AJ_PeerHandleSendManifestsReply(AJ_Message* msg)
 {
     AJ_Status status;
     const AJ_GUID* peerGuid = AJ_GUID_Find(msg->sender);
-    AJ_Manifest* manifest = NULL;
-    AJ_CredField field = { 0, NULL };
-    uint8_t digest[AJ_SHA256_DIGEST_LENGTH];
+    AJ_ManifestArray* manifests = NULL;
+    uint8_t mustClearAuthContext = FALSE;
 
-    AJ_InfoPrintf(("AJ_PeerHandleSendManifestReply(msg=%p)\n", msg));
+    AJ_InfoPrintf(("AJ_PeerHandleSendManifestsReply(msg=%p)\n", msg));
 
     if (msg->hdr->msgType == AJ_MSG_ERROR) {
-        AJ_WarnPrintf(("AJ_PeerHandleSendManifestReply(msg=%p): error=%s.\n", msg, msg->error));
+        AJ_WarnPrintf(("AJ_PeerHandleSendManifestsReply(msg=%p): error=%s.\n", msg, msg->error));
         goto Exit;
     }
 
@@ -1756,35 +1823,34 @@ AJ_Status AJ_PeerHandleSendManifestReply(AJ_Message* msg)
         return status;
     }
 
-    field.data = msg->bus->sock.rx.readPtr;
-    status = AJ_ManifestUnmarshal(&manifest, msg);
-    if (AJ_OK != status) {
-        AJ_InfoPrintf(("AJ_PeerHandleSendManifestReply(msg=%p): Manifest unmarshal failed\n", msg));
-        goto Exit;
-    }
-    field.size = msg->bus->sock.rx.readPtr - field.data;
-    status = AJ_ManifestDigest(&field, digest);
-    field.data = NULL;
-    field.size = 0;
-    if (AJ_OK != status) {
-        goto Exit;
-    }
-    /* Compare with digest from certificate */
-    if (authContext.kactx.ecdsa.size && (0 == memcmp(digest, authContext.kactx.ecdsa.manifest, AJ_SHA256_DIGEST_LENGTH))) {
-        status = AJ_ManifestApply(manifest, msg->sender);
+    /*
+     * This might get called during handshake, or after. If after, load the ECDSA context
+     * so the identity certificate thumbprint is available.
+     */
+    if (NULL == authContext.bus) {
+        status = LoadECDSAContext(peerGuid);
         if (AJ_OK != status) {
-            AJ_InfoPrintf(("AJ_PeerHandleSendManifestReply(msg=%p): Manifest apply failed\n", msg));
+            AJ_InfoPrintf(("AJ_PeerHandleSendManifestsReply(msg=%p): Could not load ECDSA context for peer\n", msg));
+            status = AJ_ERR_SECURITY;
             goto Exit;
         }
+        mustClearAuthContext = TRUE;
     }
-    AJ_ManifestFree(manifest);
-    manifest = NULL;
+
+    status = AJ_ManifestArrayUnmarshal(&manifests, msg);
+    if (AJ_OK != status) {
+        AJ_InfoPrintf(("AJ_PeerHandleSendManifestsReply(msg=%p): Manifests unmarshal failed\n", msg));
+        goto Exit;
+    }
+    AJ_ManifestArrayApply(manifests, msg->sender, &authContext);
+    AJ_ManifestArrayFree(manifests);
+    manifests = NULL;
 
     /* Search for membership certificates from the beginning */
     authContext.slot = AJ_CREDS_NV_ID_BEGIN;
     authContext.code = SEND_MEMBERSHIPS_NONE;
     status = AJ_CredentialGetNext(AJ_CERTIFICATE_MBR_X509 | AJ_CRED_TYPE_CERTIFICATE, NULL, NULL, NULL, &authContext.slot);
-    AJ_InfoPrintf(("AJ_PeerHandleSendManifestReply(msg=%p): Membership slot %d\n", msg, authContext.slot));
+    AJ_InfoPrintf(("AJ_PeerHandleSendManifestsReply(msg=%p): Membership slot %d\n", msg, authContext.slot));
     if (AJ_OK == status) {
         /* There is at least one cert to send, we don't know if the last yet */
         authContext.code = SEND_MEMBERSHIPS_MORE;
@@ -1792,8 +1858,12 @@ AJ_Status AJ_PeerHandleSendManifestReply(AJ_Message* msg)
     status = AJ_OK;
 
 Exit:
-    AJ_ManifestFree(manifest);
+    AJ_ManifestArrayFree(manifests);
+    if (mustClearAuthContext) {
+        AJ_ClearAuthContext();
+    }
     if (AJ_OK == status) {
+        sentManifests = TRUE;
         return SendMemberships(msg);
     } else {
         HandshakeComplete(AJ_ERR_SECURITY);
@@ -2128,8 +2198,17 @@ AJ_Status AJ_PeerHandleSendMemberships(AJ_Message* msg, AJ_Message* reply)
     }
 
     if ((SEND_MEMBERSHIPS_NONE == authContext.code) && (SEND_MEMBERSHIPS_NONE == code)) {
-        /* Nothing more to send or receive */
+        /*
+         * Nothing more to send or receive. Try to send manifests. Don't call HandshakeComplete
+         * until after so the authContext is still present.
+         */
+        status = AJ_PeerSendManifests(msg, FALSE);
+        if (AJ_OK != status) {
+            AJ_InfoPrintf(("AJ_PeerHandleSendMemberships(msg=%p, reply=%p): Couldn't AJ_PeerSendManifests; got %u\n",
+                           msg, reply, status));
+        }
         HandshakeComplete(status);
+
     }
 
     return status;
