@@ -23,11 +23,11 @@
 
 #define W_VARBASE 6     /* Parameter for scalar multiplication.  Should use 2-2.5 KB.  Must be >= 2. */
 
-static digit256_tc P256_A = { 18446744073709551612ULL, 4294967295ULL, 0ULL, 18446744069414584321ULL };
-static digit256_tc P256_B = { 4309448131093880907ULL, 7285987128567378166ULL, 12964664127075681980ULL, 6540974713487397863ULL };
-static digit256_tc P256_ORDER = { 17562291160714782033ULL, 13611842547513532036ULL, 18446744073709551615ULL, 18446744069414584320ULL };
-static digit256_tc P256_GENERATOR_X = { 17627433388654248598ULL, 8575836109218198432ULL, 17923454489921339634ULL, 7716867327612699207ULL };
-static digit256_tc P256_GENERATOR_Y = { 14678990851816772085ULL, 3156516839386865358ULL, 10297457778147434006ULL, 5756518291402817435ULL };
+static digit256_tc P256_A = { 0xFFFFFFFFFFFFFFFCULL, 0x00000000FFFFFFFFULL, 0x0000000000000000ULL, 0xFFFFFFFF00000001ULL };
+static digit256_tc P256_B = { 0x3BCE3C3E27D2604BULL, 0x651D06B0CC53B0F6ULL, 0xB3EBBD55769886BCULL, 0x5AC635D8AA3A93E7ULL };
+static digit256_tc P256_ORDER = { 0xF3B9CAC2FC632551ULL, 0xBCE6FAADA7179E84ULL, 0xFFFFFFFFFFFFFFFFULL, 0xFFFFFFFF00000000ULL };
+static digit256_tc P256_GENERATOR_X = { 0xF4A13945D898C296ULL, 0x77037D812DEB33A0ULL, 0xF8BCE6E563A440F2ULL, 0x6B17D1F2E12C4247ULL };
+static digit256_tc P256_GENERATOR_Y = { 0xCBB6406837BF51F5ULL, 0x2BCE33576B315ECEULL, 0x8EE7EB4A7C0F9E16ULL, 0x4FE342E2FE1A7F9BULL };
 
 AJ_Status ec_getcurve(ec_t* curve, curveid_t curveid)
 {
