@@ -35,6 +35,7 @@
 #include <assert.h>
 #include <stdio.h>
 #include <time.h>
+#include <stdlib.h>
 
 #include <ajtcl/aj_target.h>
 #include <ajtcl/aj_bufio.h>
@@ -703,13 +704,13 @@ static void Mcast6Up(const char* group, uint16_t port, uint8_t mdns, uint16_t re
 
     // Get the IPv6 adapter addresses
     if (ERROR_SUCCESS != GetAdaptersAddresses(AF_INET6, GAA_FLAG_SKIP_MULTICAST | GAA_FLAG_SKIP_ANYCAST | GAA_FLAG_SKIP_DNS_SERVER, NULL, interfaces, &num_bytes)) {
-        AJ_ErrPrintf(("Mcast6Up(): GetAdaptersAddresses for IPv6 failed. WSAGetLastError()=%0x%x\n", WSAGetLastError()));
+        AJ_ErrPrintf(("Mcast6Up(): GetAdaptersAddresses for IPv6 failed. WSAGetLastError()=0x%x\n", WSAGetLastError()));
         return;
     }
 
     // Get the IPv4 adapter addresses
     if (ERROR_SUCCESS != GetAdaptersAddresses(AF_INET, GAA_FLAG_SKIP_MULTICAST | GAA_FLAG_SKIP_ANYCAST | GAA_FLAG_SKIP_DNS_SERVER, NULL, v4_interfaces, &num_bytes)) {
-        AJ_ErrPrintf(("Mcast6Up(): GetAdaptersAddresses for IPv4 failed. WSAGetLastError()=%0x%x\n", WSAGetLastError()));
+        AJ_ErrPrintf(("Mcast6Up(): GetAdaptersAddresses for IPv4 failed. WSAGetLastError()=0x%x\n", WSAGetLastError()));
         return;
     }
 
@@ -805,12 +806,12 @@ static void Mcast4Up(const char* group, uint16_t port, uint8_t mdns, uint16_t re
 
     tmp_sock = socket(AF_INET, SOCK_DGRAM, 0);
     if (tmp_sock == INVALID_SOCKET) {
-        AJ_ErrPrintf(("Mcast4Up(): socket failed. WSAGetLastError()=%0x%x\n", WSAGetLastError()));
+        AJ_ErrPrintf(("Mcast4Up(): socket failed. WSAGetLastError()=0x%x\n", WSAGetLastError()));
         return;
     }
 
     if (SOCKET_ERROR == WSAIoctl(tmp_sock, SIO_GET_INTERFACE_LIST, 0, 0, &interfaces, sizeof(interfaces), &num_bytes, 0, 0)) {
-        AJ_ErrPrintf(("Mcast4Up(): WSAIoctl failed. WSAGetLastError()=%0x%x\n", WSAGetLastError()));
+        AJ_ErrPrintf(("Mcast4Up(): WSAIoctl failed. WSAGetLastError()=0x%x\n", WSAGetLastError()));
         return;
     }
 
@@ -843,7 +844,7 @@ static void Mcast4Up(const char* group, uint16_t port, uint8_t mdns, uint16_t re
             continue;
         }
 
-        ret = setsockopt(new_sock.sock, SOL_SOCKET, SO_REUSEADDR, &reuse, sizeof(reuse));
+        ret = setsockopt(new_sock.sock, SOL_SOCKET, SO_REUSEADDR, (char*) &reuse, sizeof(reuse));
 
         if (ret != 0) {
             AJ_ErrPrintf(("MCast4Up(): setsockopt(SO_REUSEADDR) failed. errno=\"%s\", status=AJ_ERR_READ\n", strerror(errno)));
@@ -926,7 +927,7 @@ static SOCKET MDnsRecvUp()
 
     tmp_sock = socket(AF_INET, SOCK_DGRAM, 0);
     if (tmp_sock == INVALID_SOCKET) {
-        AJ_ErrPrintf(("MDnsRecvUp(): socket failed. WSAGetLastError()=%0x%x\n", WSAGetLastError()));
+        AJ_ErrPrintf(("MDnsRecvUp(): socket failed. WSAGetLastError()=0x%x\n", WSAGetLastError()));
         return tmp_sock;
     }
 
@@ -971,7 +972,7 @@ AJ_Status AJ_Net_MCastUp(AJ_MCastSocket* mcastSock)
     AJ_InfoPrintf(("AJ_Net_MCastUp(mcastSock=0x%p)\n", mcastSock));
 
     // create the mDNS recv socket
-    tmp_sock = MDnsRecvUp(mcastSock);
+    tmp_sock = MDnsRecvUp();
     if (tmp_sock != INVALID_SOCKET) {
         getsockname(tmp_sock, (struct sockaddr*) &addrBuf, &addrLen);
         sin = (struct sockaddr_in*) &addrBuf;
@@ -1062,7 +1063,7 @@ static AJ_Status AJ_ARDP_UDP_Send(void* context, uint8_t* buf, size_t len, size_
         return AJ_ERR_WRITE;
     }
 
-    if (!WSAGetOverlappedResult(ctx->udpSock, &ov, sent, TRUE, &flags)) {
+    if (!WSAGetOverlappedResult(ctx->udpSock, &ov, (DWORD*) sent, TRUE, &flags)) {
         AJ_ErrPrintf(("AJ_ARDP_UDP_Send(): WSAGetOverlappedResult() failed. WSAGetLastError()=0x%x, status=AJ_ERR_WRITE\n", WSAGetLastError()));
         return AJ_ERR_WRITE;
     }
