@@ -24,10 +24,17 @@
 #include <ajtcl/aj_crypto_ecc.h>
 #include <ajtcl/aj_crypto_fp.h>
 #include <ajtcl/aj_crypto_ec_p256.h>
-#include <ajtcl/aj_crypto.h>      // for the RNG
-#include <ajtcl/aj_crypto_ecc.h>  // ECDH/ECDSA APIs
+#include <ajtcl/aj_crypto.h>      /* for the RNG */
+#include <ajtcl/aj_crypto_ecc.h>  /* ECDH/ECDSA APIs */
 #include <ajtcl/aj_crypto_sha2.h>
 
+
+/*
+ * Set this variable to print out the standard EC-SPEKE basepoints used in
+ * AllJoyn for ECDHE_SPEKE.
+ */
+static const int g_print_speke_basepoints = 0;
+
 /* Benchmark support code */
 /* Note: For a given processor, more precise timing methods often exist.
  *       Since currently the build system does not tell us the processor, we
@@ -45,9 +52,9 @@ typedef struct {
 /* Access system counter for benchmarking. */
 uint64_t benchmark_time(void)
 {
-#if _WIN32
+#if defined _WIN32
     return __rdtsc();
-#elif __linux
+#elif defined __linux
     struct timespec time;
     clock_gettime(CLOCK_REALTIME, &time);
     return (int64_t)(time.tv_sec * 1e9 + time.tv_nsec);
@@ -55,9 +62,9 @@ uint64_t benchmark_time(void)
 }
 
 /* Print benchmarking result with the corresponding unit */
-void bench_print(char* label, unsigned long long count, unsigned long long num_runs)
+void bench_print(const char* label, unsigned long long count, unsigned long long num_runs)
 {
-    char* unit;
+    const char* unit;
     unsigned long long result = count / num_runs;
 
 #if __linux
@@ -102,30 +109,30 @@ int run_fpmul_p256_kats()
     int run = 0;
 
     // Known answer tests.  c1_known = a*b (mod p256).  The product a*b before modular reduction is included for debugging.
-    digit256_tc a1 = { 17719206558549681838, 6431396540947310048, 11612935573663628158, 8718051496037544076 };
-    digit256_tc b1 = { 4921253721701162537, 17209563459544305479, 1017636069198536828, 5517976548754532891 };
-    digit256_tc c1_known = { 961789641737072625, 10225457115821151364, 4102039298977734239, 1404790894956925208 };
-    // a*b = [14274785008760427998, 6946299620016575489, 13709194129124787162, 17909878623608041999, 419040080746784108, 5344702675657996016, 14552306481969204026, 2607831686380395122]
+    digit256_tc a1 = { 17719206558549681838UL, 6431396540947310048UL, 11612935573663628158UL, 8718051496037544076UL };
+    digit256_tc b1 = { 4921253721701162537UL, 17209563459544305479UL, 1017636069198536828UL, 5517976548754532891UL };
+    digit256_tc c1_known = { 961789641737072625UL, 10225457115821151364UL, 4102039298977734239UL, 1404790894956925208UL };
+    // a*b = [14274785008760427998UL, 6946299620016575489UL, 13709194129124787162UL, 17909878623608041999UL, 419040080746784108UL, 5344702675657996016UL, 14552306481969204026UL, 2607831686380395122]
 
-    digit256_tc a2 = { 8844934698490770636, 9829389649759050548, 15580190479849132290, 17766681537739872130 };
-    digit256_tc b2 = { 14500914995729698198, 6422638149855794201, 15664328861594059002, 6404777437367841893 };
-    digit256_tc c2_known = { 2097716429123545365, 17656009607115261235, 9344564188714855756, 5082079834809801308 };
-    // a*b = [1300317245404892040, 6008198196536125206, 16022114046911353928, 5876577119790533637, 14048949065398320954, 15111932852584245813, 1471281526380311976, 6168657221845067557]
+    digit256_tc a2 = { 8844934698490770636UL, 9829389649759050548UL, 15580190479849132290UL, 17766681537739872130UL };
+    digit256_tc b2 = { 14500914995729698198UL, 6422638149855794201UL, 15664328861594059002UL, 6404777437367841893UL };
+    digit256_tc c2_known = { 2097716429123545365UL, 17656009607115261235UL, 9344564188714855756UL, 5082079834809801308UL };
+    // a*b = [1300317245404892040UL, 6008198196536125206UL, 16022114046911353928UL, 5876577119790533637UL, 14048949065398320954UL, 15111932852584245813UL, 1471281526380311976UL, 6168657221845067557]
 
-    digit256_tc a3 = { 15939615155660301108, 7143350094927966522, 3883951360218364098, 15133662551349039251 };
-    digit256_tc b3 = { 18150268098113480338, 11312047171767748001, 6186193757377372050, 8019375174056696995 };
-    digit256_tc c3_known = { 11452255603278212989, 12257383617576985661, 11163574383076909462, 1558810016341072940 };
-    // a*b = [5776732926367550376, 1772296329390911596, 9566540135247392647, 4895202992860378391, 17996356884603099097, 9232508218673844893, 5182695418892870641, 6579075270513827835]
+    digit256_tc a3 = { 15939615155660301108UL, 7143350094927966522UL, 3883951360218364098UL, 15133662551349039251UL };
+    digit256_tc b3 = { 18150268098113480338UL, 11312047171767748001UL, 6186193757377372050UL, 8019375174056696995UL };
+    digit256_tc c3_known = { 11452255603278212989UL, 12257383617576985661UL, 11163574383076909462UL, 1558810016341072940UL };
+    // a*b = [5776732926367550376UL, 1772296329390911596UL, 9566540135247392647UL, 4895202992860378391UL, 17996356884603099097UL, 9232508218673844893UL, 5182695418892870641UL, 6579075270513827835]
 
-    digit256_tc a4 = { 14318944385485372598, 883375215920048957, 13803891150075560442, 6611050328294776182 };
-    digit256_tc b4 = { 13788568089797780523, 14427058084629806332, 14551108227132870340, 612024294718074594 };
-    digit256_tc c4_known = { 4847932808027178217, 16808312094030226880, 378517564788830623, 327361809514739816 };
-    // a*b = [4169236071965493906, 2577223082178085076, 8612931833157935979, 17698379716881045988, 8647078201883724018, 17217140294649640218, 14905457703337800644, 219340789808369142]
+    digit256_tc a4 = { 14318944385485372598UL, 883375215920048957UL, 13803891150075560442UL, 6611050328294776182UL };
+    digit256_tc b4 = { 13788568089797780523UL, 14427058084629806332UL, 14551108227132870340UL, 612024294718074594UL };
+    digit256_tc c4_known = { 4847932808027178217UL, 16808312094030226880UL, 378517564788830623UL, 327361809514739816UL };
+    // a*b = [4169236071965493906UL, 2577223082178085076UL, 8612931833157935979UL, 17698379716881045988UL, 8647078201883724018UL, 17217140294649640218UL, 14905457703337800644UL, 219340789808369142]
 
-    digit256_tc a5 = { 6907620683598084025, 10652677734757169213, 12111960731393070133, 18325545720791423845 };
-    digit256_tc b5 = { 16421853107984745044, 7149540827219402707, 7591632485150638734, 11078530975399126193 };
-    digit256_tc c5_known = { 813005960448587351, 15100031179218961217, 296293684846639486, 5443020030248087442 };
-    // a*b = [7766518664827697844, 1761751396603640420, 4677571217510298465, 11145029642123640723, 2694824567421187998, 4143170301981517467, 17398917319244501262, 11005743078434454794]
+    digit256_tc a5 = { 6907620683598084025UL, 10652677734757169213UL, 12111960731393070133UL, 18325545720791423845UL };
+    digit256_tc b5 = { 16421853107984745044UL, 7149540827219402707UL, 7591632485150638734UL, 11078530975399126193UL };
+    digit256_tc c5_known = { 813005960448587351UL, 15100031179218961217UL, 296293684846639486UL, 5443020030248087442UL };
+    // a*b = [7766518664827697844UL, 1761751396603640420UL, 4677571217510298465UL, 11145029642123640723UL, 2694824567421187998UL, 4143170301981517467UL, 17398917319244501262UL, 11005743078434454794]
 
     digit256_tc a6 = { 10, 0, 0, 0 };
     digit256_tc b6 = { 17, 0, 0, 0 };
@@ -173,17 +180,17 @@ int run_fpadd_p256_kats()
     int run = 0;
     int passed = 0;
 
-    digit256_tc a1 = { 13389093525700345293, 9786536739768428856, 2828400845031766471, 14419212017040060345 };
-    digit256_tc b1 = { 14294722422393585579, 17460355475368744786, 3501978364702603477, 3400043727025504347 };
-    digit256_tc c1_known = { 9237071874384379256, 8800148141427622027, 6330379209734369949, 17819255744065564692 };
+    digit256_tc a1 = { 13389093525700345293UL, 9786536739768428856UL, 2828400845031766471UL, 14419212017040060345UL };
+    digit256_tc b1 = { 14294722422393585579UL, 17460355475368744786UL, 3501978364702603477UL, 3400043727025504347UL };
+    digit256_tc c1_known = { 9237071874384379256UL, 8800148141427622027UL, 6330379209734369949UL, 17819255744065564692UL };
 
-    digit256_tc a2 = { 9046118443709457292, 13349558207210907587, 7658398902856321151, 3862605742198849968 };
-    digit256_tc b2 = { 13188724448030810238, 8350995109368408207, 875478512590890457, 16363082593212770616 };
-    digit256_tc c2_known = { 3788098818030715915, 3253809238574796883, 8533877415447211609, 1778944265997036263 };
+    digit256_tc a2 = { 9046118443709457292UL, 13349558207210907587UL, 7658398902856321151UL, 3862605742198849968UL };
+    digit256_tc b2 = { 13188724448030810238UL, 8350995109368408207UL, 875478512590890457UL, 16363082593212770616UL };
+    digit256_tc c2_known = { 3788098818030715915UL, 3253809238574796883UL, 8533877415447211609UL, 1778944265997036263UL };
 
-    digit256_tc a3 = { 13389093525700345293, 9786536739768428856, 2828400845031766471, 14419212017040060345 };
-    digit256_tc b3 = { 14294722422393585579, 17460355475368744786, 3501978364702603477, 3400043727025504347 };
-    digit256_tc c3_known = { 9237071874384379256, 8800148141427622027, 6330379209734369949, 17819255744065564692 };
+    digit256_tc a3 = { 13389093525700345293UL, 9786536739768428856UL, 2828400845031766471UL, 14419212017040060345UL };
+    digit256_tc b3 = { 14294722422393585579UL, 17460355475368744786UL, 3501978364702603477UL, 3400043727025504347UL };
+    digit256_tc c3_known = { 9237071874384379256UL, 8800148141427622027UL, 6330379209734369949UL, 17819255744065564692UL };
 
     digit256_tc a4 = { 10, 0, 0, 0 };
     digit256_tc b4 = { 7, 0, 0, 0 };
@@ -222,17 +229,17 @@ int run_fpsub_p256_kats()
     int run = 0;
     int passed = 0;
 
-    digit256_tc a1 = { 11700990786256262891, 5637983711336958193, 5959611888226728961, 11869998447973167452 };
-    digit256_tc b1 = { 4819216106114708977, 14863724383416041616, 12346602459755209691, 15988320929136601693 };
-    digit256_tc c1_known = { 6881774680141553913, 9221003405925435489, 12059753502181070885, 14328421588251150079 };
+    digit256_tc a1 = { 11700990786256262891UL, 5637983711336958193UL, 5959611888226728961UL, 11869998447973167452UL };
+    digit256_tc b1 = { 4819216106114708977UL, 14863724383416041616UL, 12346602459755209691UL, 15988320929136601693UL };
+    digit256_tc c1_known = { 6881774680141553913UL, 9221003405925435489UL, 12059753502181070885UL, 14328421588251150079UL };
 
-    digit256_tc a2 = { 17719206558549681838, 6431396540947310048, 11612935573663628158, 8718051496037544076 };
-    digit256_tc b2 = { 4921253721701162537, 17209563459544305479, 1017636069198536828, 5517976548754532891 };
-    digit256_tc c2_known = { 12797952836848519301, 7668577155112556185, 10595299504465091329, 3200074947283011185 };
+    digit256_tc a2 = { 17719206558549681838UL, 6431396540947310048UL, 11612935573663628158UL, 8718051496037544076UL };
+    digit256_tc b2 = { 4921253721701162537UL, 17209563459544305479UL, 1017636069198536828UL, 5517976548754532891UL };
+    digit256_tc c2_known = { 12797952836848519301UL, 7668577155112556185UL, 10595299504465091329UL, 3200074947283011185UL };
 
-    digit256_tc a3 = { 8844934698490770636, 9829389649759050548, 15580190479849132290, 17766681537739872130 };
-    digit256_tc b3 = { 14500914995729698198, 6422638149855794201, 15664328861594059002, 6404777437367841893 };
-    digit256_tc c3_known = { 12790763776470624054, 3406751499903256346, 18362605691964624904, 11361904100372030236 };
+    digit256_tc a3 = { 8844934698490770636UL, 9829389649759050548UL, 15580190479849132290UL, 17766681537739872130UL };
+    digit256_tc b3 = { 14500914995729698198UL, 6422638149855794201UL, 15664328861594059002UL, 6404777437367841893UL };
+    digit256_tc c3_known = { 12790763776470624054UL, 3406751499903256346UL, 18362605691964624904UL, 11361904100372030236UL };
 
     digit256_tc a4 = { 17, 0, 0, 0 };
     digit256_tc b4 = { 7, 0, 0, 0 };
@@ -279,14 +286,14 @@ int run_fpneg_p256_kats()
     int run = 0;
     int passed = 0;
 
-    digit256_tc a1 = { 15939615155660301108, 7143350094927966522, 3883951360218364098, 15133662551349039251 };
-    digit256_tc b1_known = { 2507128918049250507, 11303393983076552389, 14562792713491187517, 3313081518065545069 };
+    digit256_tc a1 = { 15939615155660301108UL, 7143350094927966522UL, 3883951360218364098UL, 15133662551349039251UL };
+    digit256_tc b1_known = { 2507128918049250507UL, 11303393983076552389UL, 14562792713491187517UL, 3313081518065545069UL };
 
-    digit256_tc a2 = { 18150268098113480338, 11312047171767748001, 6186193757377372050, 8019375174056696995 };
-    digit256_tc b2_known = { 296475975596071277, 7134696906236770910, 12260550316332179565, 10427368895357887325 };
+    digit256_tc a2 = { 18150268098113480338UL, 11312047171767748001UL, 6186193757377372050UL, 8019375174056696995UL };
+    digit256_tc b2_known = { 296475975596071277UL, 7134696906236770910UL, 12260550316332179565UL, 10427368895357887325UL };
 
-    digit256_tc a3 = { 14318944385485372598, 883375215920048957, 13803891150075560442, 6611050328294776182 };
-    digit256_tc b3_known = { 4127799688224179017, 17563368862084469954, 4642852923633991173, 11835693741119808138 };
+    digit256_tc a3 = { 14318944385485372598UL, 883375215920048957UL, 13803891150075560442UL, 6611050328294776182UL };
+    digit256_tc b3_known = { 4127799688224179017UL, 17563368862084469954UL, 4642852923633991173UL, 11835693741119808138UL };
 
     passed += test_fpneg_p256_kat(a1, b1_known, 1);
     run++;
@@ -318,15 +325,15 @@ int run_fpequal_p256_kats()
     int run = 0;
     int passed = 0;
 
-    digit256_tc a1 = { 15939615155660301108, 7143350094927966522, 3883951360218364098, 15133662551349039251 };
-    digit256_tc b1 = { 15939615155660301108, 7143350094927966522, 3883951360218364098, 15133662551349039251 };
+    digit256_tc a1 = { 15939615155660301108UL, 7143350094927966522UL, 3883951360218364098UL, 15133662551349039251UL };
+    digit256_tc b1 = { 15939615155660301108UL, 7143350094927966522UL, 3883951360218364098UL, 15133662551349039251UL };
     boolean_t c1 = B_TRUE;
 
-    digit256_tc a2 = { 15939615155660301108, 7143350094927966522, 3883951360218364098, 15133662551349039251 };
-    digit256_tc b2 = { 15939615155660301108, 7143350094927966522, 3883951360218364098, 15133662551349039252 };
+    digit256_tc a2 = { 15939615155660301108UL, 7143350094927966522UL, 3883951360218364098UL, 15133662551349039251UL };
+    digit256_tc b2 = { 15939615155660301108UL, 7143350094927966522UL, 3883951360218364098UL, 15133662551349039252UL };
     boolean_t c2 = B_FALSE;
 
-    digit256_tc a3 = { 0, 4294967296, 0, 18446744069414584321, }; // p + 1
+    digit256_tc a3 = { 0, 4294967296UL, 0, 18446744069414584321UL, }; // p + 1
     digit256_tc b3 = { 1, 0, 0, 0 };  // 1
     boolean_t c3 = B_FALSE;    // Note that fpequal requires inputs be reduced mod p, so these will not be considered equal.
 
@@ -345,7 +352,6 @@ int run_fpequal_p256_kats()
 
 int test_fpinv_p256_kat(digit256_tc a, digit256_tc b_known, int i)
 {
-    boolean_t status = B_FALSE;
     digit_t temps[P256_TEMPS];
     digit256_t b;
 
@@ -363,14 +369,14 @@ int run_fpinv_p256_kats()
     int run = 0;
     int passed = 0;
 
-    digit256_tc a1 = { 6907620683598084025, 10652677734757169213, 12111960731393070133, 18325545720791423845 };
-    digit256_tc b1 = { 14025966668329367848, 3595156072241463650, 15456073631280180716, 14150861440201384475 };
+    digit256_tc a1 = { 6907620683598084025UL, 10652677734757169213UL, 12111960731393070133UL, 18325545720791423845UL };
+    digit256_tc b1 = { 14025966668329367848UL, 3595156072241463650UL, 15456073631280180716UL, 14150861440201384475UL };
 
-    digit256_tc a2 = { 16421853107984745044, 7149540827219402707, 7591632485150638734, 11078530975399126193 };
-    digit256_tc b2 = { 5524053351190660477, 14052510857214807129, 17409895116542148161, 5055399665644516158 };
+    digit256_tc a2 = { 16421853107984745044UL, 7149540827219402707UL, 7591632485150638734UL, 11078530975399126193UL };
+    digit256_tc b2 = { 5524053351190660477UL, 14052510857214807129UL, 17409895116542148161UL, 5055399665644516158UL };
 
-    digit256_tc a3 = { 9046118443709457292, 13349558207210907587, 7658398902856321151, 3862605742198849968 };
-    digit256_tc b3 = { 17723950800386319926, 12284076655406226634, 17109263073168979093, 71817302339060736 };
+    digit256_tc a3 = { 9046118443709457292UL, 13349558207210907587UL, 7658398902856321151UL, 3862605742198849968UL };
+    digit256_tc b3 = { 17723950800386319926UL, 12284076655406226634UL, 17109263073168979093UL, 71817302339060736UL };
 
     passed += test_fpinv_p256_kat(a1, b1, 1);
     run++;
@@ -386,7 +392,6 @@ int run_fpinv_p256_kats()
 
 int test_fpdiv2_p256_kat(digit256_tc a, digit256_tc b_known, int i)
 {
-    boolean_t status = B_FALSE;
     digit256_t b;
     digit_t temps[P256_TEMPS];
 
@@ -404,14 +409,14 @@ int run_fpdiv2_p256_kats()
     int run = 0;
     int passed = 0;
 
-    digit256_tc a1 = { 13188724448030810238, 8350995109368408207, 875478512590890457, 16363082593212770616 };
-    digit256_tc b1 = { 15817734260870180927, 13398869591538979911, 437739256295445228, 8181541296606385308 };
+    digit256_tc a1 = { 13188724448030810238UL, 8350995109368408207UL, 875478512590890457UL, 16363082593212770616UL };
+    digit256_tc b1 = { 15817734260870180927UL, 13398869591538979911UL, 437739256295445228UL, 8181541296606385308UL };
 
-    digit256_tc a2 = { 13389093525700345293, 9786536739768428856, 2828400845031766471, 14419212017040060345 };
-    digit256_tc b2 = { 6694546762850172646, 14116640408886473884, 1414200422515883235, 16432978043227322333 };
+    digit256_tc a2 = { 13389093525700345293UL, 9786536739768428856UL, 2828400845031766471UL, 14419212017040060345UL };
+    digit256_tc b2 = { 6694546762850172646UL, 14116640408886473884UL, 1414200422515883235UL, 16432978043227322333UL };
 
-    digit256_tc a3 = { 14294722422393585579, 17460355475368744786, 3501978364702603477, 3400043727025504347 };
-    digit256_tc b3 = { 7147361211196792789, 17953549776686631849, 1750989182351301738, 10923393898220044334 };
+    digit256_tc a3 = { 14294722422393585579UL, 17460355475368744786UL, 3501978364702603477UL, 3400043727025504347UL };
+    digit256_tc b3 = { 7147361211196792789UL, 17953549776686631849UL, 1750989182351301738UL, 10923393898220044334UL };
 
     passed += test_fpdiv2_p256_kat(a1, b1, 1);
     run++;
@@ -540,11 +545,10 @@ int test_curve_arith_basics()
     ecpoint_jacobian_t G = { 0 };
     ecpoint_jacobian_t anotherG = { 0 };
     AJ_Status status;
-    ecpoint_t R2_known = { { 11964737083406719352, 13873736548487404341, 9967090510939364035, 9003393950442278782 },
-                           { 11386427643415524305, 13438088067519447593, 2971701507003789531, 537992211385471040 } };       // 2*G
-    ecpoint_t R3_known = { { 18104864246493347180, 16629180030495074693, 14481306550553801061, 6830804848925149764 },
-                           { 11131122737810853938, 15576456008133752893, 3984285777615168236, 9742521897846374270 } };   // 3*G
-    ecpoint_t R = { 0 };
+    ecpoint_t R2_known = { { 11964737083406719352UL, 13873736548487404341UL, 9967090510939364035UL, 9003393950442278782UL },
+                           { 11386427643415524305UL, 13438088067519447593UL, 2971701507003789531UL, 537992211385471040UL } };       // 2*G
+    ecpoint_t R3_known = { { 18104864246493347180UL, 16629180030495074693UL, 14481306550553801061UL, 6830804848925149764UL },
+                           { 11131122737810853938UL, 15576456008133752893UL, 3984285777615168236UL, 9742521897846374270UL } };   // 3*G
     ecpoint_jacobian_t R2 = { 0 };
     ecpoint_jacobian_t R3 = { 0 };
 
@@ -612,13 +616,13 @@ int test_scalarmul_kat(digit256_t k, digit256_t x, digit256_t y, int i)
 
     status = ec_getcurve(&curve, NISTP256r1);
     if (status != AJ_OK) {
-        AJ_Printf("ec_getcurve failed\n", i);
+        AJ_Printf("ec_getcurve failed %d\n", i);
         goto Exit;
     }
     ec_get_generator(&P, &curve);
 
     if (ec_is_infinity(&P, &curve)) {
-        AJ_Printf("invalid generator\n", i);
+        AJ_Printf("invalid generator %d\n", i);
         status = AJ_ERR_UNKNOWN;
         goto Exit;
     }
@@ -650,13 +654,13 @@ int run_scalarmul_kats()
     // Test vectors: (x, y) = k*G (where G is the basepoint)
     // From http://point-at-infinity.org/ecc/nisttv
 
-    digit256_t k1 = { 17562291160714782032, 13611842547513532036, 18446744073709551615, 18446744069414584320 };
-    digit256_t x1 = { 17627433388654248598, 8575836109218198432, 17923454489921339634, 7716867327612699207 };
-    digit256_t y1 = { 3767753221892779530, 15290227238617653553, 8149286295562117609, 12690225778011766885 };
+    digit256_t k1 = { 17562291160714782032UL, 13611842547513532036UL, 18446744073709551615UL, 18446744069414584320UL };
+    digit256_t x1 = { 17627433388654248598UL, 8575836109218198432UL, 17923454489921339634UL, 7716867327612699207UL };
+    digit256_t y1 = { 3767753221892779530UL, 15290227238617653553UL, 8149286295562117609UL, 12690225778011766885UL };
 
-    digit256_t k2 = { 8234495237290528275, 6084187275451764, 0, 0 };
-    digit256_t x2 = { 13013376741987594852, 7335293150882016018, 7890206492658706934, 1981025739527209566 };
-    digit256_t y2 = { 5672504522216064379, 8327131327894173024, 4446911187859987120, 13828999463473408775 };
+    digit256_t k2 = { 8234495237290528275UL, 6084187275451764UL, 0, 0 };
+    digit256_t x2 = { 13013376741987594852UL, 7335293150882016018UL, 7890206492658706934UL, 1981025739527209566UL };
+    digit256_t y2 = { 5672504522216064379UL, 8327131327894173024UL, 4446911187859987120UL, 13828999463473408775UL };
 
     passed += test_scalarmul_kat(k1, x1, y1, 1);
     run++;
@@ -986,6 +990,146 @@ int test_conversion()
     return 1;
 }
 
+int test_redp()
+{
+    ec_t curve = { 0 };
+    ecpoint_t Q1 = { 0 };
+    ecpoint_t Q2 = { 0 };
+    ecpoint_t P1 = { 0 };
+    ecpoint_t P2 = { 0 };
+    ecpoint_t R = { 0 };
+    AJ_Status status;
+    unsigned char point1[18] = "ALLJOYN-ECSPEKE-1";
+    unsigned char point2[18] = "ALLJOYN-ECSPEKE-2";
+    unsigned char password[11] = "mypassword";
+
+    status = ec_getcurve(&curve, NISTP256r1);
+    if (status != AJ_OK) {
+        goto Exit;
+    }
+
+    status = ec_REDP1(point1, sizeof(point1), &Q1, &curve);
+    if (status != AJ_OK) {
+        AJ_Printf("REDP-1 failed with pi1, did not return AJ_OK.");
+        goto Exit;
+    }
+
+    status = ec_REDP1(point2, sizeof(point2), &Q2, &curve);
+    if (status != AJ_OK) {
+        AJ_Printf("REDP-1 failed with pi2, did not return AJ_OK.");
+        goto Exit;
+    }
+
+    if (g_print_speke_basepoints) {
+        /* Print the standard EC-SPEKE basepoints used in AllJoyn for ECDHE_SPEKE. */
+        AJ_Printf("REDP-1(ALLJOYN-ECSPEKE-1)\n");
+        print_digits("x = ", Q1.x);
+        print_digits("y = ", Q1.y);
+        AJ_Printf("REDP-1(ALLJOYN-ECSPEKE-2)\n");
+        print_digits("x = ", Q2.x);
+        print_digits("y = ", Q2.y);
+    }
+
+    status = ec_REDP2(password, &Q1, &Q2, &R, &curve);
+    if (status != AJ_OK) {
+        AJ_Printf("REDP-2 failed, did not return AJ_OK.");
+        goto Exit;
+    }
+
+    /* Make sure precomputed basepoints equal those we computed just now from the two constants. */
+    ec_get_REDP_basepoints(&P1, &P2, curve.curveid);
+    if (!ecpoint_areequal(&P1, &Q1, &curve)) {
+        status = AJ_ERR_UNKNOWN;
+        AJ_Printf("REDP precomputed basepoints incorrect, point 1 does not match REDP-1(%s).", point1);
+        goto Exit;
+    }
+    if (!ecpoint_areequal(&P2, &Q2, &curve)) {
+        status = AJ_ERR_UNKNOWN;
+        AJ_Printf("REDP precomputed basepoints incorrect, point 2 does not match REDP-1(%s).", point2);
+        goto Exit;
+    }
+
+Exit:
+    if (status == AJ_OK) {
+        return 1;
+    } else {
+        return 0;
+    }
+}
+
+// Test the ECDHE_ECSPEKE key generation and agreement functions from aj_crypto_ecc.h
+int test_ecspeke()
+{
+    AJ_GUID clientGUID;
+    AJ_GUID serviceGUID;
+    AJ_Status status;
+    AJ_ECCPublicKey pk1, pk2;
+    AJ_ECCPrivateKey sk1, sk2;
+    AJ_ECCSecret secret1, secret2;
+    const uint8_t pw[8] = { 1, 2, 3, 4, 5, 6, 7, 8 };
+    const uint8_t notpw[8] = { 8, 7, 6, 5, 4, 3, 2, 1 };
+
+    AJ_CreateNewGUID((uint8_t*)&clientGUID, sizeof(AJ_GUID));
+    AJ_CreateNewGUID((uint8_t*)&serviceGUID, sizeof(AJ_GUID));
+
+    // Create key pairs from matching password
+    status = AJ_GenerateSPEKEKeyPair(pw, sizeof(pw), &clientGUID, &serviceGUID, &pk1, &sk1);
+    if (status != AJ_OK) {
+        AJ_Printf("Failed to generate key pair 1 for ECDHE_SPEKE test\n");
+        return 0;
+    }
+    status = AJ_GenerateSPEKEKeyPair(pw, sizeof(pw), &clientGUID, &serviceGUID, &pk2, &sk2);
+    if (status != AJ_OK) {
+        AJ_Printf("Failed to generate key pair 2 for ECDHE_SPEKE test\n");
+        return 0;
+    }
+
+    // Do key agreement operation
+    status = AJ_GenerateShareSecret(&pk2, &sk1, &secret1);
+    if (status != AJ_OK) {
+        AJ_Printf("Failed to generate shared secret 1 for ECDHE_SPEKE test\n");
+        return 0;
+    }
+    status = AJ_GenerateShareSecret(&pk1, &sk2, &secret2);
+    if (status != AJ_OK) {
+        AJ_Printf("Failed to generate shared secret 2 for ECDHE_SPEKE test\n");
+        return 0;
+    }
+
+    // Shared secrets should be equal
+    if (memcmp(secret1.x, secret2.x, KEY_ECC_SZ) != 0) {
+        AJ_Printf("Shared secrets for ECHDE_SPEKE test do not match\n");
+        return 0;
+    }
+
+    // Re-create keypair 2 with a different password
+    status = AJ_GenerateSPEKEKeyPair(notpw, sizeof(notpw), &clientGUID, &serviceGUID, &pk2, &sk2);
+    if (status != AJ_OK) {
+        AJ_Printf("Failed to re-generate key pair 2 for ECDHE_SPEKE test\n");
+        return 0;
+    }
+
+    // Re-do key agreement operation
+    status = AJ_GenerateShareSecret(&pk2, &sk1, &secret1);
+    if (status != AJ_OK) {
+        AJ_Printf("Failed to generate shared secret 1 for ECDHE_SPEKE test\n");
+        return 0;
+    }
+    status = AJ_GenerateShareSecret(&pk1, &sk2, &secret2);
+    if (status != AJ_OK) {
+        AJ_Printf("Failed to generate shared secret 2 for ECDHE_SPEKE test\n");
+        return 0;
+    }
+
+    // Shared secrets should no longer be equal
+    if (memcmp(secret1.x, secret2.x, KEY_ECC_SZ) == 0) {
+        AJ_Printf("Shared secrets for ECHDE_SPEKE test match when they shouldn't\n");
+        return 0;
+    }
+
+    return 1;
+}
+
 int AJ_Main()
 {
     int tests_ran = 0;
@@ -1017,6 +1161,12 @@ int AJ_Main()
     passed += test_conversion();
     tests_ran++;
 
+    passed += test_redp();
+    tests_ran++;
+
+    passed += test_ecspeke();
+    tests_ran++;
+
     AJ_Printf("  Ran %d tests, %d passed.\n", tests_ran, passed);
 
     AJ_Printf("Running benchmarks...\n");
