@@ -110,7 +110,6 @@ static AJ_Object AppObjects[] = {
 
 static AJ_PermissionMember members[] = { { "*", AJ_MEMBER_TYPE_ANY, AJ_ACTION_PROVIDE | AJ_ACTION_OBSERVE, NULL } };
 static AJ_PermissionRule rules[] = { { "/org/alljoyn/alljoyn_test", "org.alljoyn.alljoyn_test", members, NULL } };
-static AJ_Manifest manifest = { rules };
 
 /*
  * Message identifiers for the method calls this application implements
@@ -304,7 +303,7 @@ static AJ_Status MarshalAppId(AJ_Message* msg, const char* appId)
     if (status != AJ_OK) {
         return status;
     }
-    status = AJ_MarshalArgs(msg, "{sv}", "AppId", APP_ID_SIGNATURE, binAppId, sz / 2);
+    status = AJ_MarshalArgs(msg, "{sv}", AJ_APP_ID_STR, APP_ID_SIGNATURE, binAppId, sz / 2);
 
     return status;
 }
@@ -332,19 +331,19 @@ static AJ_Status AboutPropGetter(AJ_Message* reply, const char* language)
             status = MarshalAppId(reply, &machineIdValue[0]);
         }
         if (status == AJ_OK) {
-            status = AJ_MarshalArgs(reply, "{sv}", "AppName", "s", "svclite");
+            status = AJ_MarshalArgs(reply, "{sv}", AJ_APP_NAME_STR, "s", "svclite");
         }
         if (status == AJ_OK) {
-            status = AJ_MarshalArgs(reply, "{sv}", "DeviceId", "s", machineIdValue);
+            status = AJ_MarshalArgs(reply, "{sv}", AJ_DEVICE_ID_STR, "s", machineIdValue);
         }
         if (status == AJ_OK) {
-            status = AJ_MarshalArgs(reply, "{sv}", "DeviceName", "s", "Tester");
+            status = AJ_MarshalArgs(reply, "{sv}", AJ_DEVICE_NAME_STR, "s", "Tester");
         }
         if (status == AJ_OK) {
-            status = AJ_MarshalArgs(reply, "{sv}", "Manufacturer", "s", "QCE");
+            status = AJ_MarshalArgs(reply, "{sv}", AJ_MANUFACTURER_STR, "s", "QCE");
         }
         if (status == AJ_OK) {
-            status = AJ_MarshalArgs(reply, "{sv}", "ModelNumber", "s", "1.0");
+            status = AJ_MarshalArgs(reply, "{sv}", AJ_MODEL_NUMBER_STR, "s", "1.0");
         }
         //SupportedLanguages
         if (status == AJ_OK) {
@@ -352,7 +351,7 @@ static AJ_Status AboutPropGetter(AJ_Message* reply, const char* language)
             AJ_Arg languageListArray;
             status = AJ_MarshalContainer(reply, &dict, AJ_ARG_DICT_ENTRY);
             if (status == AJ_OK) {
-                status = AJ_MarshalArgs(reply, "s", "SupportedLanguages");
+                status = AJ_MarshalArgs(reply, "s", AJ_SUPPORTED_LANGUAGES_STR);
             }
             if (status == AJ_OK) {
                 status = AJ_MarshalVariant(reply, "as");
@@ -372,16 +371,16 @@ static AJ_Status AboutPropGetter(AJ_Message* reply, const char* language)
 
         }
         if (status == AJ_OK) {
-            status = AJ_MarshalArgs(reply, "{sv}", "Description", "s", "svclite test app");
+            status = AJ_MarshalArgs(reply, "{sv}", AJ_DESCRIPTION_STR, "s", "svclite test app");
         }
         if (status == AJ_OK) {
-            status = AJ_MarshalArgs(reply, "{sv}", "DefaultLanguage", "s", "en");
+            status = AJ_MarshalArgs(reply, "{sv}", AJ_DEFAULT_LANGUAGE_STR, "s", "en");
         }
         if (status == AJ_OK) {
-            status = AJ_MarshalArgs(reply, "{sv}", "SoftwareVersion", "s", AJ_GetVersion());
+            status = AJ_MarshalArgs(reply, "{sv}", AJ_SOFTWARE_VERSION_STR, "s", AJ_GetVersion());
         }
         if (status == AJ_OK) {
-            status = AJ_MarshalArgs(reply, "{sv}", "AJSoftwareVersion", "s", AJ_GetVersion());
+            status = AJ_MarshalArgs(reply, "{sv}", AJ_AJSOFTWARE_VERSION_STR, "s", AJ_GetVersion());
         }
     }
     if (status == AJ_OK) {
@@ -454,7 +453,7 @@ int AJ_Main()
 
             AJ_BusEnableSecurity(&bus, suites, ArraySize(suites));
             AJ_BusSetAuthListenerCallback(&bus, AuthListenerCallback);
-            AJ_ManifestTemplateSet(&manifest);
+            AJ_ManifestTemplateSet(rules);
             if (claim) {
                 AJ_SecuritySetClaimConfig(&bus, APP_STATE_CLAIMABLE, CLAIM_CAPABILITY_ECDHE_PSK, 0);
             }
